/*
   Simple device driver for access to the DCARD (Porgamrays) resources on the Xilinx ML403 evaluation board
*/

#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>   /* printk() */
#include <linux/fs.h>       /* everything... */
#include <linux/ioport.h>
#include <linux/slab.h>

#include <asm/io.h>
#include <asm/uaccess.h>

#include "dcard.h"

MODULE_AUTHOR("STFC Daresbury Laboratory");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Xilinx ML40X DCARD Device Driver");

MODULE_PARM(debug_level, "i");

#define DEVICE_NAME "dcard"

static struct file_operations dcard_ops = {
    .owner   = THIS_MODULE,
    .open    = dcard_open,
    .release = dcard_release,
    .read    = dcard_read,
    .write   = dcard_write,
    .ioctl   = dcard_ioctl
};

static unsigned int dcard_major = 0;  /* use dynamic allocation */

unsigned long base0 = 0x42000000;
unsigned long size0 = 0x00010000;    /*  64 Kbytes  */
unsigned long *base0_mapped;


/*
 * Open the device
*/

static int dcard_open (struct inode *inode, struct file *filp)
{

/*    claim all resources  */

    DPRINTF("dcard_open:\n");
        
    return 0;
}

/*
 * Close the device
*/

static int dcard_release (struct inode *inode, struct file *filp)
{

/*    free all resources  */

    DPRINTF("dcard_release:\n");
       
    return 0;
}

/*
 *  device ioctl
*/

static int dcard_ioctl (struct inode *inode, struct file *filp, unsigned int cmd,
     unsigned long arg)
{

    DPRINTF("dcard_ioctl: called with command %d\n", cmd);
    
    switch (cmd)  {
    
       default:
         return -ENOTTY;
    }
    
    return(0);
}


/*
 *  device read
*/

static ssize_t dcard_read (struct file *filp,
     char *buffer,     /* the buffer to fill with data  */
     size_t length,    /* the length of the buffer      */
     loff_t *offset)   /* offset in the file            */
{
    int count;
    unsigned long *srcptr;          /* pointer in data buffer  */
    unsigned long *destptr;         /* address of hardware */
    int rc = length;

    unsigned char * kbuf = kmalloc(length, GFP_KERNEL);
    srcptr = (unsigned long *)kbuf;
    
    count = length >> 2;    /* number of 32 bit words to read */

    destptr = (unsigned long*) ((unsigned char*)base0_mapped + *offset);
    while (count--) {*srcptr++ = *destptr++;}

    if (copy_to_user(buffer, kbuf, length)) rc = -EFAULT;            

    kfree(kbuf);

    return rc;
}

/*
 *  device write
*/

static ssize_t dcard_write (struct file *filp,
     const char *buffer,     /* the data buffer */
     size_t length,    /* the length of the buffer      */
     loff_t *offset)   /* offset in the file            */
{
    int count;
    unsigned long *srcptr;          /* pointer in data buffer  */
    unsigned long *destptr;         /* address of hardware */
    int rc = length;
    
    unsigned char * kbuf = kmalloc(length, GFP_KERNEL);
    
    if (copy_from_user(kbuf, buffer, length)) return -EFAULT;
    srcptr = (unsigned long *)kbuf;
        
    count = length >> 2;    /* number of 32 bit words to write */

    destptr = (unsigned long*) ((unsigned char*)base0_mapped + *offset);
    while (count--) {*destptr++ = *srcptr++;}
    
    kfree(kbuf);
            
    return rc;
}


/*
 * Module housekeeping.
*/
 
/*
     dcard_init  -  module initialisation
     entered by the kernel when the module is loaded
*/
 
static int __init dcard_init(void)
{
    struct resource* res;

//    release in case of previous error

    release_mem_region(base0,size0);   /*** should fail *****/

//    obtain region of address space required

    res = request_mem_region(base0, size0, DEVICE_NAME);  /* the return pointer is meaningless */
    if (res == NULL) {
            printk(KERN_INFO "dcard: can't get I/O mem address 0x%lx\n",base0);
            return -ENODEV;
    }

//     and ioremap it

    base0_mapped = (unsigned long*) ioremap_nocache(base0, size0);
    
    DPRINTF("dcard: mem region start 0x%lx for 0x%lx mapped at 0x%lx\n", base0, size0,(unsigned long) base0_mapped);
            
//    register the device

    dcard_major = register_chrdev(dcard_major, DEVICE_NAME, &dcard_ops);
    if (dcard_major < 0) {
        printk(KERN_WARNING "dcard: registering the device failed with %d\n", dcard_major);
        return dcard_major;
    }
    
    DPRINTF("dcard: driver assigned major number %d\n", dcard_major);
    
    return 0;
}

/*
    dcard_cleanup  -  module cleanup
    entered by the kernel when the module is unloaded
*/

static void __exit dcard_cleanup(void)
{
    int rc;

    DPRINTF("dcard module is exiting\n");
    
    rc = unregister_chrdev(dcard_major, DEVICE_NAME);
    if (rc < 0) DPRINTF("dcard: Error in unregister_chrdev: %d\n", rc);
    
    iounmap((void *) base0_mapped);
    release_mem_region(base0,size0);
    
}


//    define module initialisation and termination procedures

module_init(dcard_init);
module_exit(dcard_cleanup);



