/*
   Simple device driver for access to the GPIO resources on the Xilinx ML403 evaluation board
*/

#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>   /* printk() */
#include <linux/fs.h>       /* everything... */
#include <linux/ioport.h>
#include <linux/slab.h>

#include <asm/io.h>
#include <asm/uaccess.h>

#include "gpio.h"

MODULE_AUTHOR("CCLRC Daresbury Laboratory");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Xilinx ML40X GPIO Device Driver");

MODULE_PARM(debug_level, "i");

#define DEVICE_NAME "gpio"

static struct file_operations gpio_ops = {
    .owner   = THIS_MODULE,
    .open    = gpio_open,
    .release = gpio_release,
    .read    = gpio_read,
    .write   = gpio_write,
    .ioctl   = gpio_ioctl
};

static unsigned int gpio_major = 0;  /* use dynamic allocation */

unsigned long base0 = 0x90000000;
unsigned long size0 = 0x200;
unsigned long *base0_mapped;

unsigned long base1 = 0x90001000;
unsigned long size1 = 0x200;
unsigned long *base1_mapped;

unsigned long base2 = 0x90002000;
unsigned long size2 = 0x200;
unsigned long *base2_mapped;


/*
 * Open the device
*/

static int gpio_open (struct inode *inode, struct file *filp)
{

/*    claim all resources  */

    DPRINTF("gpio_open:\n");
        
    return 0;
}

/*
 * Close the device
*/

static int gpio_release (struct inode *inode, struct file *filp)
{

/*    free all resources  */

    DPRINTF("gpio_release:\n");
       
    return 0;
}

/*
 *  device ioctl
*/

static int gpio_ioctl (struct inode *inode, struct file *filp, unsigned int cmd,
     unsigned long arg)
{

    DPRINTF("gpio_ioctl: called with command %d\n", cmd);
    
    switch (cmd)  {
    
       default:
         return -ENOTTY;
    }
    
    return(0);
}


/*
 *  device read
*/

static ssize_t gpio_read (struct file *filp,
     char *buffer,     /* the buffer to fill with data  */
     size_t length,    /* the length of the buffer      */
     loff_t *offset)   /* offset in the file            */
{
    struct inode *inode;
    int count;
    int minor;
    unsigned long *srcptr;          /* pointer in data buffer  */
    unsigned long *destptr;         /* address of hardware */
    int rc = length;

//    DPRINTF("gpio_read: %d %d\n", (int)length,(int)*offset);

    unsigned char * kbuf = kmalloc(length, GFP_KERNEL);
    srcptr = (unsigned long *)kbuf;

    inode = filp->f_dentry->d_inode;
    minor = MINOR(inode->i_rdev);     /* the 2.4 way */
    
    count = length >> 2;    /* number of 32 bit words to read */

    switch (minor&3) {
        case 0:
            destptr = (unsigned long*) ((unsigned char*)base0_mapped + *offset);
            while (count--) {*srcptr++ = readl(destptr++); rmb();}
            if (copy_to_user(buffer, kbuf, length)) rc = -EFAULT;
            break;
        case 1:
            destptr = (unsigned long*) ((unsigned char*)base1_mapped + *offset);
            while (count--) {*srcptr++ = readl(destptr++); rmb();}
            if (copy_to_user(buffer, kbuf, length)) rc = -EFAULT;
            break;
        case 2:
            destptr = (unsigned long*) ((unsigned char*)base2_mapped + *offset);
            while (count--) {*srcptr++ = readl(destptr++); rmb();}
            if (copy_to_user(buffer, kbuf, length)) rc = -EFAULT;
            break;
            
        default:
            rc = -EFAULT;
            break;
    }

    kfree(kbuf);

    return rc;
}

/*
 *  device write
*/

static ssize_t gpio_write (struct file *filp,
     const char *buffer,     /* the data buffer */
     size_t length,    /* the length of the buffer      */
     loff_t *offset)   /* offset in the file            */
{
    struct inode *inode;
    int count;
    int minor;
    unsigned long *srcptr;          /* pointer in data buffer  */
    unsigned long *destptr;         /* address of hardware */
    int rc = length;

//    DPRINTF("gpio_write: %d %d\n", (int)length,(int)*offset);
    
    unsigned char * kbuf = kmalloc(length, GFP_KERNEL);
    
    if (copy_from_user(kbuf, buffer, length)) return -EFAULT;
    srcptr = (unsigned long *)kbuf;
    
    inode = filp->f_dentry->d_inode;
    minor = MINOR(inode->i_rdev);     /* the 2.4 way */
    
    count = length >> 2;    /* number of 32 bit words to write */
    
    switch (minor&3) {
        case 0:
            destptr = (unsigned long*) ((unsigned char*)base0_mapped + *offset);
            while (count--) {writel(*srcptr++, destptr++); wmb();}
            break;
        case 1:
            destptr = (unsigned long*) ((unsigned char*)base1_mapped + *offset);
            while (count--) {writel(*srcptr++, destptr++); wmb();}
            break;
        case 2:
            destptr = (unsigned long*) ((unsigned char*)base2_mapped + *offset);
            while (count--) {writel(*srcptr++, destptr++); wmb();}
            break;
            
        default:
            rc = -EFAULT;
            break;
    }
    
    kfree(kbuf);
            
    return rc;
}


/*
 * Module housekeeping.
*/
 
/*
     gpio_init  -  module initialisation
     entered by the kernel when the module is loaded
*/
 
static int __init gpio_init(void)
{
    struct resource* res;

//    release in case of previous error

    release_mem_region(base0,size0);   /*** should fail *****/
    release_mem_region(base1,size1);   /*** should fail *****/
    release_mem_region(base2,size2);   /*** should fail *****/

//    obtain region of address space required

    res = request_mem_region(base0, size0, DEVICE_NAME);  /* the return pointer is meaningless */
    if (res == NULL) {
            printk(KERN_INFO "gpio: can't get I/O mem address 0x%lx\n",base0);
            return -ENODEV;
    }

//     and ioremap it

    base0_mapped = (unsigned long*) ioremap_nocache(base0, size0);
    
    DPRINTF("gpio: mem region start 0x%lx for 0x%lx mapped at 0x%lx\n", base0, size0,(unsigned long) base0_mapped);
    
//    obtain region of address space required

    res = request_mem_region(base1, size1, DEVICE_NAME);  /* the return pointer is meaningless */
    if (res == NULL) {
            printk(KERN_INFO "gpio: can't get I/O mem address 0x%lx\n",base0);
            return -ENODEV;
    }

//     and ioremap it

    base1_mapped = (unsigned long*) ioremap_nocache(base1, size1);
    
    DPRINTF("gpio: mem region start 0x%lx for 0x%lx mapped at 0x%lx\n", base1, size1,(unsigned long) base1_mapped);

//    obtain region of address space required

    res = request_mem_region(base2, size2, DEVICE_NAME);  /* the return pointer is meaningless */
    if (res == NULL) {
            printk(KERN_INFO "gpio: can't get I/O mem address 0x%lx\n",base0);
            return -ENODEV;
    }

//     and ioremap it

    base2_mapped = (unsigned long*) ioremap_nocache(base2, size2);
    
    DPRINTF("gpio: mem region start 0x%lx for 0x%lx mapped at 0x%lx\n", base2, size2,(unsigned long) base2_mapped);
        
//    register the device

    gpio_major = register_chrdev(gpio_major, DEVICE_NAME, &gpio_ops);
    if (gpio_major < 0) {
        printk(KERN_WARNING "gpio: registering the device failed with %d\n", gpio_major);
        return gpio_major;
    }
    
    DPRINTF("gpio: driver assigned major number %d\n", gpio_major);
    
    return 0;
}

/*
    gpio_cleanup  -  module cleanup
    entered by the kernel when the module is unloaded
*/

static void __exit gpio_cleanup(void)
{
    int rc;

    DPRINTF("gpio module is exiting\n");
    
    rc = unregister_chrdev(gpio_major, DEVICE_NAME);
    if (rc < 0) DPRINTF("gpio: Error in unregister_chrdev: %d\n", rc);
    
    iounmap((void *) base0_mapped);
    release_mem_region(base0,size0);
    
    iounmap((void *) base1_mapped);
    release_mem_region(base1,size1);
    
    iounmap((void *) base2_mapped);
    release_mem_region(base2,size2);
}


//    define module initialisation and termination procedures

module_init(gpio_init);
module_exit(gpio_cleanup);



