////////////////////////////////////////////////////////////////////
// *****************************************************************
// *                                                               *
// * CONFIDENTIAL AND PROPRIETARY INFORMATION OF ATMEL CORPORATION *
// *                                                               *
// *                             AT24CXXX                          *
// *                                                               *
// *              Copyright (c) Atmel Corporation 1997             *
// *                                                               *
// * The scope of use of the material contained herein is strictly *
// *  limited to the Confidentiality and Non-Disclosure Agreement  *
// *  ("NDA") between the parties.  Any use inconsistent with the  *
// *   terms of the NDA is strictly forbidden without the prior    *
// *         written consent of Atmel Corporation.                 *
// *                                                               *
// *****************************************************************

////////////////////////////////////////////////////////////////////
// File:  AT24CXXX_body.vhd
//
// Architecture VhdlBehavioral definition for 2 Wire Serial CMOS
//                                                       E2PROMs
//          AT24C01A   1K   (128 x 8)
//          AT24C02    2K   (256 x 8)
//          AT24C04    4K   (512 x 8)
//          AT24C08    8K  (1024 x 8)
//          AT24C16   16K  (2048 x 8)
//          AT24C32   32K  (4096 x 8)
//          AT24C64   64K  (8192 x 8)
//          AT24C128 128K (16384 x 8)
//          AT24C256 256K (32768 x 8)
//
// Original author:  Sanjay Churiwala
//                   CADWorx Consulting Inc.
//                   Milpitas, CA
//                   http://www.cadworx.com
//
///////////////////////////////////////////////////////////////////
// REVISION HISTORY
//
// May 27, 1997
//     Initial revision.
// April 5, 1999
//     Changed the STOP condition for Write, based on 
//          behavior confirmed by Philip Ng
//
// June 19, 2001
//     Converted model to Verilog, instead of driving 
//          signals high, signals are set to 1'bz relying
//          on the pullup to pull the signals high.
//          Changes made by Brian Forsse
//
///////////////////////////////////////////////////////////////////


`timescale 1ns / 1ps

module AT24CXXX(WP, A2, A1, A0, GND, VCC, SCL, SDA);
input WP;
input A2;
input A1;
input A0;
input GND;
input VCC;
input SCL;
inout SDA;

// Constants

parameter WordWidth = 8;                         
parameter DefaultInitMemory = 1;


// Parameters

parameter InstancePath = "AT24CXXX instance";       
parameter XOn = 1;
parameter MsgOn = 1;
parameter AddrWidth = 13;                           // total no. of addr bits
parameter PageWidth = 5;                            // number of toggled bits during PAGE WRITE
parameter InitialMemFile = "default.mem";           // memory init file

// Setup and Hold times
parameter tHDDAT = 300;  // Data hold time 300 ns



// -------------------------------------------------------------
// Main Memory
// -------------------------------------------------------------
reg [WordWidth - 1 : 0] Memory [0 : (1 << AddrWidth) - 1];

// -------------------------------------------------------------
// Page buffer for WRITE
// -------------------------------------------------------------
reg [WordWidth - 1 : 0] Page [ 0 : (1 << PageWidth) - 1];
integer PageAddr; 
integer FirstPageAddr; 
integer HowManyPageWords;

// -------------------------------------------------------------
// Auxiliary variables
// -------------------------------------------------------------
reg SDA_reg;
reg InternalWriteInProgress;
reg InternalWriteInProgressFlag;
reg temp;
reg RandomRead;
reg Stop;
reg uicusp;
reg SawHighWP;
reg Protected;
reg [WordWidth - 1 : 0] TempData; 
reg SDA_event;
reg SCL_event;
reg [AddrWidth : 1] TempAddress;
integer MSB;
reg [AddrWidth:1] Counter;
integer IntCounter; 
integer tempint;
reg Ambiguous;

initial RandomRead = 0;

   // /////////////////////////////////////////////////////////////////
   // /////////////////////////////////////////////////////////////////
   // Function:  PrintMsg
   //
   // Print diagnostic message.  Message is enabled only if
   //                                              MsgOn is TRUE.
   //
   // This is a simple wrapper around $display that checks
   // MsgOn and prints Msg which can only be 20 characters.
   //
   // I/O:
   //
   //     Msg            : Message to be printed.  
   //
   // Non-local references:
   //
   //     MsgOn               : Parameter controlling whether message is


function PrintMsg;
input [8*80:1] Msg;

  if (MsgOn)
    begin
      $display("At Time: ",$time,"ns %s\n",Msg);
      PrintMsg = 1;
    end
  else
    PrintMsg = 0;
endfunction


   // /////////////////////////////////////////////////////////////////
   // /////////////////////////////////////////////////////////////////
   // Function:  InitMemory
   //
   // Initialize memory.  This procedure will test if an initialization
   // file exists.  If so it uses the file's contents to fill the memory.
   // If not it uses the InitialValue input as the initial data.
   //
   // I/O:
   //
   //      Initial Value        : The value to use if a initialization 
   //                            file doesn't exist
   // Non-local reference:
   // 
   //      Memory              : The memory to be initialized.
   //      InitialMemFile      : The filename to read memory from.
   // 


function InitMemory;
  input InitialValue;
  integer i;

  if (InitialMemFile == 8'b0)
    begin
	   if (MsgOn)
	     $display("IIC AT24Cxxx EEPROM: No .mem file used, filling all bits of memory with %b ", InitialValue);
       for (i = 0; i < (1 << AddrWidth) ; i = i + 1)
	      Memory[i] = {8{InitialValue}};
	   InitMemory = 1;
    end
  else
    begin
	  $readmemb(InitialMemFile,Memory);
    end

endfunction


   // /////////////////////////////////////////////////////////////////
   // /////////////////////////////////////////////////////////////////
   // Function:  GetWI2Word
   //
   //
   // I/O:
   //
   //      ErrorMsg            : The error message is function doesn't 
   //                            succeed
   // 
   //
   // Non-local reference: 
   //
   //      WP                  : Write Protection input
   //
   //      SawHighWP           : internal write protection flag
   //      
   //      RandomRead          : asserted if random read happens
   //      
   //      Stop                : asserted if STOP condition happens
   //          
   // 

task GetWI2Word;
  input [20*8:0] ErrorMsg;
  output [WordWidth - 1 : 0] Data;
  integer i;
  integer NextIx;

  begin : ACQUIRE_LOOP
      // Set default return values
      for(i=0;i<WordWidth;i = i+1)
        Data[i] <= 1'bx;

      // Get the bits serially, also watching for other special
      // conditions.

      RandomRead <= 0;
      Stop <= 0;

      NextIx = WordWidth-1;

      if ( WP != 0 )
        SawHighWP <= 1;

      for(i=0;i==0;i=0)
         begin 

            @(SCL or WP or posedge SDA_event);

            // Check for high WP.  (Be pessimistic in case of
            //                                       unknown value.)

            if ( WP  != 0 )
               SawHighWP = 1;       

            // Check for activity on SDA line, while SCL is
            // High. This is used to detect a START condition.
            if (SDA_event && (SCL == 1))
 		    begin
			     if(SDA != 1)
                    RandomRead = 1;
                 else
                    Stop = 1; 
                 disable ACQUIRE_LOOP;

            end

            // Check for rising SCL.

            if ( SCL ) 
			begin
                Data[NextIx] = SDA;  
                if ( NextIx == 0 ) 
				   disable ACQUIRE_LOOP;  // Wait for end of SCL pulse
                NextIx       = NextIx - 1;
            end
         end     // For loop			
	  end  // ACQUIRE_LOOP
endtask  // task GetWI2Word

   // /////////////////////////////////////////////////////////////////
   // /////////////////////////////////////////////////////////////////
   // Function:  PutWI2Word
   //
   //
   // I/O:
   //     
   //      ErrorMsg            : The error message is function doesn't 
   //                            succeed
   //
   //      Data                : Data to put on to the bus
   //
   // Non-local reference: 
   //
   //                     
   //      #THDDAT             : Parameter for data hold time
   //      
   //      SDA_reg             : registered output for SDA bus
   // 
   //      
   // 

task PutWI2Word;
  input [20*8:0] ErrorMsg;
  input [WordWidth - 1 : 0] Data;
  integer NextIx;
  integer i;

  begin : SEND_LOOP
    // Put the bits serially, also watching for other special
    // conditions.

    NextIx = WordWidth - 1;

    for(i=0;i==0;i=0)
      begin
        @(negedge SCL);

          if (Data[NextIx] == 1)
 		  begin
             #tHDDAT;           // Wait for Data Hold time
             SDA_reg = 1'bz;                   
		  end
	      else
		  begin
             #tHDDAT;           // Wait for Data Hold time
		     SDA_reg = 1'b0;
		  end

          if ( NextIx == 0 ) disable SEND_LOOP;
      
	      NextIx = NextIx - 1;
      end // for loop
   end
  endtask  // PutWI2Word

   // /////////////////////////////////////////////////////////////////
   // /////////////////////////////////////////////////////////////////
   // Procedure:  BitsToInt
   //
   // Translate a word into an integer (natural).  If any 'X' bits
   // are present, they are translated as '0', but a return flag
   // Ambiguous is set.
   //
   // Parameters:
   //
   //     BitValue              : Address word to be translated to an
   //                             integer.
   //     IntValue              : Returns the integer value obtained
   //                             from BitValue.
   //     Ambiguous             : Returns TRUE if BitValue has 'X'
   //                             value.
   //
   // Non-local references:   None.

  task BitsToInt;
  input [AddrWidth:1] BitValue;
  output [31:0] IntValue;
  output Ambiguous;
  reg [AddrWidth:1] LocalIntValue;
  integer i;
   begin
        Ambiguous     = 0;
        LocalIntValue = 0;

        for (i = AddrWidth; i > 0; i = i-1)
		begin
            if ( BitValue[i] == 0 ) 
                LocalIntValue = 2 * LocalIntValue;
            else if ( BitValue[i] == 1 ) 
                LocalIntValue = 2 * LocalIntValue + 1;
            else // 'X' or 'U'
			begin
                LocalIntValue = 2 * LocalIntValue;
                Ambiguous     = 1;
            end
        end

        IntValue = LocalIntValue;
    end
  endtask

   // /////////////////////////////////////////////////////////////////
   // /////////////////////////////////////////////////////////////////
   // Main functionality of module AT24CXXX
   // /////////////////////////////////////////////////////////////////
   // /////////////////////////////////////////////////////////////////

initial
begin
   temp <= InitMemory(DefaultInitMemory);
   InternalWriteInProgress <= 0;
   InternalWriteInProgressFlag <= 0;
   RandomRead <= 0;
   Stop       <= 0;
   uicusp     <= 0;
   SDA_reg <= 1'bz;
   TempAddress <= 0;
   Counter <= 0;
end

assign SDA = SDA_reg;

always @ (SDA) 
begin
   SDA_event <= 1;
   #1 SDA_event <= 0;
end

always @ (SCL)
begin
   SCL_event <= 1;
   #1 SCL_event <= 0;
end

always @ (posedge InternalWriteInProgressFlag) 
begin
  InternalWriteInProgress = 1;
  #1 InternalWriteInProgress = 0;  // tWR is 10 ms 
  InternalWriteInProgressFlag = 0;
end

always
begin : START_LOOP

  if (!RandomRead)
  begin   // For Random Read, START condition
                      // is already detected, so, we skip
                      // START detection portion through
                      // the above if statement.
     @(negedge SDA);
                     // Falling SDA when SCL is High is START condition
     if ( (SDA != 1'b0) || (SCL == 1'b0 ) )  
        disable START_LOOP;                  // It is not a START condition. So,
                                        // stay in "standby" mode.
        
     uicusp  = 1;   
  end
  RandomRead = 0;
  Stop       = 0;
  SawHighWP  = 0;
  Protected  = 0;

  // /////////////////////////////////////////////////////////
  // Get the Device and/or Page Address
  // /////////////////////////////////////////////////////////
    if (uicusp) 
    begin
      uicusp = 0;
	  @(SDA_event);  // to be sure that SCA_event doesn't interfere with START condition
      GetWI2Word("Bad Address Fetch",TempData);
    end

    MSB = WordWidth-1;

    if ( (TempData[MSB]   != 1) ||  // Check for Incorrect  
         (TempData[MSB-1] != 0) ||  // sequence for device selection.
         (TempData[MSB-2] != 1) ||  // Need 1010
         (TempData[MSB-3] != 0) ) 
            disable START_LOOP;

    case (AddrWidth)

       7:  if ( (TempData[MSB-4] != A2) ||
                (TempData[MSB-5] != A1) ||
                (TempData[MSB-6] != A0))      
                   disable START_LOOP;  // This 1K memory is not selected

       8:  if ( (TempData[MSB-4] != A2) ||
                (TempData[MSB-5] != A1) ||
                (TempData[MSB-6] != A0))      
                   disable START_LOOP;  // This 2K memory is not selected

       9:  if ( (TempData[MSB-4] != A2) ||
                (TempData[MSB-5] != A1))     
                   disable START_LOOP;  // This 4K memory is not selected
		   else
			     TempAddress[AddrWidth] = TempData[MSB-6];

       10:  if  (TempData[MSB-4] != A2)     
                   disable START_LOOP;  // This 8K memory is not selected
			else
			    begin
			      TempAddress[AddrWidth] = TempData[MSB-5];
				  TempAddress[AddrWidth-1] = TempData[MSB-6];
                end
    
	   11:  begin
	           TempAddress[AddrWidth] = TempData[MSB-4];   // No HardWired                    
			   TempAddress[AddrWidth-1] = TempData[MSB-5]; // Device Pin Comparison needed
			   TempAddress[AddrWidth-2] = TempData[MSB-6]; // For 16K Memory
			end

	   12:   if ( (TempData[MSB-4] != A2) ||
                (TempData[MSB-5] != A1) ||
                (TempData[MSB-6] != A0))      
                   disable START_LOOP;  // This 32K memory is not selected 

	   13:   if ( (TempData[MSB-4] != A2) ||
                (TempData[MSB-5] != A1) ||
                (TempData[MSB-6] != A0))      
                   disable START_LOOP;  // This 64K memory is not selected 

	   14:   if ( (TempData[MSB-4] != A2) ||
                (TempData[MSB-5] != A1) ||
                (TempData[MSB-6] != A0))      
                   disable START_LOOP;  // This 128K memory is not selected 

	   15:   if ( (TempData[MSB-4] != A2) ||
                (TempData[MSB-5] != A1) ||
                (TempData[MSB-6] != A0))      
                   disable START_LOOP;  // This 128K memory is not selected 

	   default:  temp = PrintMsg("Incorrect Address Width");

     endcase

     // If reached till here, means this device is selected
     // and it should acknowledge

     // The acknowledge should be sent only if the
     // device is not in WriteCycle.

     if (InternalWriteInProgress) 
	   begin
         SDA_reg = 1'bz;                   // Send No Acknowledge
         @(negedge InternalWriteInProgress);
       end

     @ (negedge SCL);
     SDA_reg = 1'b0;                      // Send Acknowledge


     if (TempData[MSB-7] == 0 ) 
     begin                           // This is for Write.              
         @(posedge SCL);
	     // At the next falling edge, the external
           // device shall start putting data,
           // so, this device should loose control
           // over the SDA bus
           @(negedge SCL);
           #tHDDAT;        // Wait for Data Hold time
           SDA_reg = 1'bz;                    // Memory has left the
                                              // control over the bus

        // Get Address now.
        GetWI2Word("Bad Address Fetch",TempData);

        case (AddrWidth)
           7:   begin
                  TempAddress[7] = TempData[MSB-1];
                  TempAddress[6] = TempData[MSB-2];
                  TempAddress[5] = TempData[MSB-3];
                  TempAddress[4] = TempData[MSB-4];
                  TempAddress[3] = TempData[MSB-5];
                  TempAddress[2] = TempData[MSB-6];
                  TempAddress[1] = TempData[MSB-7];
                end
           8:   begin
                  TempAddress[8] = TempData[MSB];
                  TempAddress[7] = TempData[MSB-1];
                  TempAddress[6] = TempData[MSB-2];
                  TempAddress[5] = TempData[MSB-3];
                  TempAddress[4] = TempData[MSB-4];
                  TempAddress[3] = TempData[MSB-5];
                  TempAddress[2] = TempData[MSB-6];
                  TempAddress[1] = TempData[MSB-7];
               end
           // For the next 3 memory sizes, a part of the
           // Word Address has been obtained along with
           // the Device Address.

           9, 10, 11:   
               begin
                  TempAddress[8] = TempData[MSB];
                  TempAddress[7] = TempData[MSB-1];
                  TempAddress[6] = TempData[MSB-2];
                  TempAddress[5] = TempData[MSB-3];
                  TempAddress[4] = TempData[MSB-4];
                  TempAddress[3] = TempData[MSB-5];
                  TempAddress[2] = TempData[MSB-6];
                  TempAddress[1] = TempData[MSB-7];
               end
 
           // The following 4 memories are Large Devices, and
           // shall require another word to get the Word Address.

           12:  begin
                  TempAddress[12] = TempData[MSB-4];
                  TempAddress[11] = TempData[MSB-5];
                  TempAddress[10] = TempData[MSB-6];
                  TempAddress[9] = TempData[MSB-7];
                end
           13:  begin
                  TempAddress[13] = TempData[MSB-3];
                  TempAddress[12] = TempData[MSB-4];
                  TempAddress[11] = TempData[MSB-5];
                  TempAddress[10] = TempData[MSB-6];
                  TempAddress[ 9] = TempData[MSB-7];
                end
           14:  begin
                  TempAddress[14] = TempData[MSB-2];
                  TempAddress[13] = TempData[MSB-3];
                  TempAddress[12] = TempData[MSB-4];
                  TempAddress[11] = TempData[MSB-5];
                  TempAddress[10] = TempData[MSB-6];
                  TempAddress[9] = TempData[MSB-7];
                end
           15:  begin
                  TempAddress[15] = TempData[MSB-1];
                  TempAddress[14] = TempData[MSB-2];
                  TempAddress[13] = TempData[MSB-3];
                  TempAddress[12] = TempData[MSB-4];
                  TempAddress[11] = TempData[MSB-5];
                  TempAddress[10] = TempData[MSB-6];
                  TempAddress[9] = TempData[MSB-7];
		        end
           default:
                  temp = PrintMsg ("Incorrect Address Width");

         endcase

         // Send an acknowledgement.
         @(SCL);
         if (SCL == 0)   
           SDA_reg = 0;
         else
           temp = PrintMsg ("Execution should never reach here");


         if (AddrWidth >= 12)
		 begin
         // Large Memory Device. Shall Need another Word
         // for Address
           @(SCL);
           if (SCL == 1) 
           // Nothing. Allow the external device to
           // sample the acknowledge
		     temp = 1'b0;
           else
             temp = PrintMsg ("Execution should never reach here");


           // At the faling edge, the external
           // device shall be sending another
           // word. So, this device should
           // loose its control over the
           // bus
           @(SCL);
           #tHDDAT;        // Wait for Data Hold time
           SDA_reg = 1'bz; // Memory has left
                       // control over the bus

           // Get the second byte of Address now.
           GetWI2Word("Aborting Address Fetch",TempData);

           TempAddress[8] = TempData[MSB];
           TempAddress[7] = TempData[MSB-1];
           TempAddress[6] = TempData[MSB-2];
           TempAddress[5] = TempData[MSB-3];
           TempAddress[4] = TempData[MSB-4];
           TempAddress[3] = TempData[MSB-5];
           TempAddress[2] = TempData[MSB-6];
           TempAddress[1] = TempData[MSB-7];

           @(SCL);
           if (SCL == 0)
             SDA_reg = 0;                       // Send Acknowledge
           else
		     temp = PrintMsg("Execution should never reach here");
         end  //  if address >= 12

         Counter = TempAddress;

         BitsToInt( Counter,
                    IntCounter,
                    Ambiguous);

         // Now, we can either have a "Data Word" to
         // be written, or a START signal for
         // Random Read.

         // START signal can come after a STOP,
         // Or in the next clock cycle.
         @(SCL);

         if (SCL == 1)
         // Nothing. The external device must
         // be sampling the acknowledge.
		   temp = 1'b0;
         else
           temp = PrintMsg ("Execution should never reach here");

         @(SDA or SCL);

         if (SDA_event) 
         // This is a STOP condition.
         // The SDA was already Low, and there
         // has been an event on it.
           disable START_LOOP;
         else
         // SCL has fallen Low.
         // External device is going to send
         // data. This device should loose
         // control over the bus.
           #tHDDAT;        // Wait for Data Hold time
           SDA_reg = 1'bz; // Memory has left
                       // control over the bus

         // If reached till here, it means, writing is desired.


         PageAddr         = IntCounter % (1 << PageWidth);
         IntCounter          = IntCounter - PageAddr;
         FirstPageAddr    = PageAddr;
         HowManyPageWords = 0;


         for (tempint=0;tempint==0;tempint=0)
         begin : WRITE_LOOP  // Write onto a PAGE
            // Writing into actual memory takes
            // place later, after the STOP
            // condition is recvd.
     
           GetWI2Word("Aborting Word Fetch",TempData);

           if (RandomRead)
           begin
             // Recvd. a START condition in between Data
             // Fetch
             // Put Back IntCounter at its proper value
             uicusp   = 1;
             IntCounter  = IntCounter + PageAddr;
             Stop     = 0;
             disable START_LOOP;
           end

         // Otherwise, the whole data byte has been obtained.

           if ( !Stop )
           begin
             @(SCL); // Sending Acknowledge
             if (SCL == 0) 
               SDA_reg = 0;  
             else
               temp = PrintMsg ("Execution should never reach here");

             Page[PageAddr]   = TempData;
             HowManyPageWords = HowManyPageWords + 1;
             if ( PageAddr >= (1 << PageWidth) )
               PageAddr = 0;
             else
               PageAddr = PageAddr + 1;

             @(SCL);
             if (SCL == 1) 
               // Nothing. The external Device must be sampling
               // the Acknowledgement
               temp = 1'b0;
             else
               temp = PrintMsg ("Execution should never reach here");
        
             @ (SCL);
             // There is a falling edge on
             // clock. The external device
             // shall send data. This device
             // should loose control over the
             // bus.
             #tHDDAT;        // Wait for Data Hold time
             SDA_reg = 1'bz; // Memory has left control
                           // over the bus
           end
           else
           begin
             // Got a STOP condition
             // Write the Page into Memory now.
             // First Check for Memory Protection.
             if (SawHighWP)
             begin
               case (AddrWidth)
                   7,8,9:  Protected = 1;
			          10:  Protected = 0;
                      11:  if (IntCounter < 1024) Protected = 1;
                      12:  if (IntCounter < 1024) Protected = 1;
                      13:  if (IntCounter < 2048) Protected = 1;
                   14,15:  Protected = 1;
                 default:  temp = PrintMsg ("Incorrect Address Width");
               endcase
               SawHighWP = 0;
             end  // WriteProtect Check Over

             PageAddr = FirstPageAddr;
    
             if (!Protected)
             begin
               // Check for InternalWrite
               if (InternalWriteInProgressFlag)
               begin
               // Execution should never reach
               // here. Still, it is given just
               // for the sake of completeness.
                 temp = PrintMsg("Could not write because internal write is in progress");
                 disable WRITE_LOOP;
               end
               else
                 InternalWriteInProgressFlag = 1;

    
               while ( HowManyPageWords > 0 )
               begin
                 Memory[IntCounter+PageAddr] = Page[PageAddr];
                 if ( PageAddr >= (1 << PageWidth) )
                   PageAddr = 0;
                 else
                   PageAddr = PageAddr + 1;
                 HowManyPageWords = HowManyPageWords - 1;
               end // While loop
             end // Memory updating over
           end // End of IF which detected Stop -vs- complete byte 
         end // WRITE_LOOP for loop

       IntCounter = IntCounter + PageAddr;

     end  //Write Operation

     else // Read Operation
	   for (tempint=0;tempint==0;tempint=0)
       begin : READ_LOOP
         // Reading starts from the last write byte + 1
         if ( IntCounter >= ((1 << AddrWidth)-1) ) 
		   IntCounter = 1'b0;
         else
           IntCounter = IntCounter + 1;
         	   
	     TempData = Memory[IntCounter];

         if (InternalWriteInProgress) 
                             // Execution should never reach
                             // here. Still, it is given just
                             // for the sake of completeness.
           temp = PrintMsg("Could not read because internal write is in progress");
         else
           PutWI2Word("Aborting Putting Out Data",
		               TempData);
       
          // Now, we need to watch for Acknowledge, or Stop

          @(SCL);
          if (SCL == 1) 
            // Nothing. External Device must be reading the
            // last bit.
			temp = 1'b0;
          else
            temp = PrintMsg ("Execution should never reach here");


          @(SCL);
          if (SCL == 0)
		  begin
            // No Operation. The external device should be
            // sending an Acknowledgement,
            // or No  Acnowledgement.
            // The external device has to send
            // Ack/NoAck. So, this device should
            // loose control over the bus.
            #tHDDAT;        // Wait for Data Hold time
            SDA_reg = 1'bz; // Memory has left control
                            // over the bus
		  end
          else
            temp = PrintMsg ("Execution should never reach here ACK/NACK not recieved ");


          @(SCL);
          // Now, we have recvd. a Rising Edge on Clock.
          // If SDA is High, then, there is NO ACKNO, and
          // a STOP shall be following. So, we can go to
          // STANDBY.
          if (SDA == 1) 
            disable START_LOOP;
          // Otherwise, there was Acknowledge, and Read
          // should continue.

        end // For the Read/Write Selection

end // START_LOOP

endmodule
	
