//----------------------------------------------------------------------------
// PPC405 Processor Block Bus Functional Model - Module
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
// 
//----------------------------------------------------------------------------
// Filename:        proc_block_BFM.v
// 
// Description:
//   This module instantiates PLB and DCR BFMs to emulate the CPU.
//
// Design Notes:
//   This model should only be used for the purpose of exercising the PLB 
//   and DCR busses. It is not intended to provide any other behavioral
//   functionality other than PLB and DCR bus operations.
//
//-----------------------------------------------------------------------------
// Structure:   
// 
//            -- proc_block_BFM.v
//                 -- dcr_master.v
//                 -- plb_master3x.v
//
//-----------------------------------------------------------------------------
// Author:      KD
// History:
//   KD        5/03/01 -- EAK2 Release
//
//-----------------------------------------------------------------------------
// Naming Conventions:
//      active low signals:                     "*_n"
//      clock signals:                          "clk", "clk_div#", "clk_#x" 
//      reset signals:                          "rst", "rst_n" 
//      generics/parameters:                    "C_*" 
//      user defined types:                     "*_TYPE" 
//      state machine next state:               "*_ns" 
//      state machine current state:            "*_cs" 
//      combinatorial signals:                  "*_com" 
//      pipelined or register delay signals:    "*_d#" 
//      counter signals:                        "*cnt*"
//      clock enable signals:                   "*_ce" 
//      internal version of output port         "*_i"
//      device pins:                            "*_pin" 
//      ports:                                  - Names begin with Uppercase 
//      processes:                              "*_PROCESS" 
//      component instantiations:               "<ENTITY_>I_<#|FUNC>
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
// Module Declaration
///////////////////////////////////////////////////////////////////////////////

module PPC405 (
   C405JTGPGMOUT,                 // (O)
   MCBCPUCLKEN,                   // (I) 
   MCBJTAGEN,                     // (I) 
   MCBTIMEREN,                    // (I) 
   MCPPCRST,                      // (I) 
   C405TRCODDEXECUTIONSTATUS,     // (O)
   C405TRCEVENEXECUTIONSTATUS,    // (O)

// CLOCK AND POWER MANAGEMENT INTERFACE
   CPMC405CLOCK,                  // (I) MAIN CORE CLOCK
   CPMC405CORECLKINACTIVE,        // (I) CORE CLOCKS DISABLED
   PLBCLK,                        // (I) PLB CLOCK
   CPMC405TIMERTICK,              // (I)
   CPMC405CPUCLKEN,               // (I)
   CPMC405JTAGCLKEN,              // (I)
   CPMC405TIMERCLKEN,             // (I)
   C405CPMMSREE,                  // (O) INDICATES THAT MSR[EE]=1
   C405CPMMSRCE,                  // (O) INDICATES THAT MSR[CE]=1
   C405CPMTIMERIRQ,               // (O) INDICATES THAT A TIMER INTERRUPT REQUEST WAS GENERATED
   C405CPMTIMERRESETREQ,          // (O) INDICATES THAT A WATCHDOG TIMER GENERATED RESET REQUEST
   C405CPMCORESLEEPREQ,           // (O) INDICATES THAT THE CORE IS REQUESTING TO BE PUT INTO SLEEP

// CPU CONTROL INTERFACE
   TIEC405DISOPERANDFWD,          
   TIEC405DETERMINISTICMULT,      
   TIEC405MMUEN,                  // (I) ENABLE THE MMU FUNCTIONAITY OF THE CORE
   C405XXXMACHINECHECK,           // (O) INDICATES A MACHINE CHECK ERROR 
                                  //     HAS BEEN DETECTED

// RESET INTERFACE
   C405RSTCHIPRESETREQ,           // (O) CHIP RESET REQUEST EVENT
   C405RSTCORERESETREQ,           // (O) CORE RESET REQUEST EVENT
   C405RSTSYSRESETREQ,            // (O) SYSTEM RESET REQUEST EVENT
   RSTC405RESETCHIP,              // (I) CHIP RESET INPUT
   RSTC405RESETCORE,              // (I) CORE RESET INPUT (MINIMUM OF 4 CLOCK CYCLES)
   RSTC405RESETSYS,               // (I) SYSTEM RESET INPUT

// INSTRUCTION SIDE PLB INTERFACE
   C405PLBICUREQUEST,             // (O) INSTRUCTION FETCH REQUEST
   C405PLBICUPRIORITY,            // (O) INDICATES THAT THE CURRENT REQUEST IS 
                                  //     HIGH PRIORITY
   C405PLBICUCACHEABLE,           // (O) INDICATES REQUESTED INSTRUCTION IS IN 
                                  //     CACHEABLE STORAGE REGION
   C405PLBICUABUS,                // (O) INSTRUCTION MEMORY ADDRESS BUS
   C405PLBICUSIZE,                // (O) INDICATES SIZE OF TRANSFER
   C405PLBICUABORT,               // (O) INDICATES THAT CURRENT ICU TRANSFER 
                                  //     REQUEST IS BEING ABORTED
   C405PLBICUU0ATTR,              // (O) PLB USER DEFINED ATTRIBUTE
   PLBC405ICUADDRACK,             // (I) INDICATES THAT PLB HAS ACKNOWLEDGED 
                                  //     CURRENT REQUEST
   PLBC405ICUBUSY,                // (I) INDICATES THE PLB IS BUSY PERFORMING 
                                  //     AN ICU OPERATION
   PLBC405ICUERR,                 // (I) INDICATES THAT AN ERROR WAS DETECTED 
                                  //     BY PLB DURING THE TRANSFER
   PLBC405ICURDDACK,              // (I) INDICATES REQUESTED INSTRUCTION WORD 
                                  //     IS PRESENTED ON PLBC405ICURDDBUS
   PLBC405ICURDDBUS,              // (I) 64-BIT MEMORY ALIGNED DATA BUS
   PLBC405ICUSSIZE1,              // (I) INDICATES PLB SLAVE SIZE ('0'=32, '1'=64)
   PLBC405ICURDWDADDR,            // (I) INDICATES WHICH WORD IS BEING 
                                  //     TRANSFERRED DURING 8 WORD TRANSFERS

// DATA SIDE PLB INTERFACE
   C405PLBDCUREQUEST,             // (O) DATA FETCH REQUEST
   C405PLBDCURNW,                 // (O) READ/WRITE INDICATION
   C405PLBDCUABUS,                // (O) DATA MEMORY ADDRESS BUS
   C405PLBDCUBE,                  // (O) INDICATES WHICH BYTES IN A WORD ARE 
                                  //     TO BE TRANSFERRED
   C405PLBDCUCACHEABLE,           // (O) INDICATES REQUESTED DATA IS IN 
                                  //     CACHEABLE STORAGE REGION
   C405PLBDCUGUARDED,             // (O) INDICATES REQUESTED DATA IS IN 
                                  //     GUARDED STORAGE REGION
   C405PLBDCUPRIORITY,            // (O) INDICATES THAT THE CURRENT REQUEST 
                                  //     IS HIGH PRIORITY
   C405PLBDCUSIZE2,               // (O) INDICATES SIZE OF TRANSFER
   C405PLBDCUABORT,               // (O) INDICATES THAT CURRENT DCU TRANSFER 
                                  //     REQUEST IS BEING ABORTED
   C405PLBDCUWRDBUS,              // (O) 64-BIT MEMORY ALIGNED WRITE DATA BUS
   C405PLBDCUU0ATTR,              // (O) PLB USER DEFINED ATTRIBUTE
   C405PLBDCUWRITETHRU,           // (O)
   PLBC405DCUADDRACK,             // (I) INDICATES THAT PLB HAS ACKNOWLEDGED 
                                  //     CURRENT REQUEST
   PLBC405DCUBUSY,                // (I) INDICATES THE PLB IS BUSY PERFORMING 
                                  //     A DCU OPERATION
   PLBC405DCUERR,                 // (I) INDICATES THAT AN ERROR WAS DETECTED 
                                  //     BY PLB DURING THE TRANSFER
   PLBC405DCURDDACK,              // (I) INDICATES REQUESTED DATA WORD IS 
                                  //     PRESENTED ON PLBC405ICURDDBUS
   PLBC405DCURDDBUS,              // (I) 64-BIT MEMORY ALIGNED READ DATA BUS
   PLBC405DCURDWDADDR,            // (I) INDICATES WHICH WORD IS BEING 
                                  //     TRANSFERRED DURING 8 WORD TRANSFERS
   PLBC405DCUSSIZE1,              // (I) INDICATES PLB SLAVE SIZE ('0'=32, '1'=64)
   PLBC405DCUWRDACK,              // (I) INDICATES REQUESTED DATA WORD IS 
                                  //     ACCEPTED BY PLB ON PLBC405DCUWRDBUS

//  INSTRUCTION-SIDE ON-CHIP MEMORY INTERFACE
   BRAMISOCMCLK,                  // (I)
   ISOCMBRAMRDABUS,               // (O) BRAM ADDR - READ ADDR
   ISOCMBRAMWRABUS,               // (O) BRAM INITIALIZATION ADDR - WRITE ADDR
   ISOCMBRAMEN,                   // (O) BRAM ENABLE
   ISOCMBRAMODDWRITEEN,           // (O) BRAM ODD WORD WRITE ENABLE
   ISOCMBRAMEVENWRITEEN,          // (O) BRAM EVEN WORD WRITE ENABLE
   ISOCMBRAMWRDBUS,               // (O) STORE DATA BUS TO BRAM
   BRAMISOCMRDDBUS,               // (I) LOAD DATA BUS FROM BRAM
   TIEISOCMDCRADDR,               // (I) ISOCM DCR REG ADDRESS
   ISARCVALUE,                    // (I) ISOCM IS MAPPED TO F200,0000
   ISCNTLVALUE,                   // (I) DEFAULT ISCNTL REG VALUE WHEN RESET

//  DATA-SIDE ON-CHIP MEMORY INTERFACE
   BRAMDSOCMCLK,                  // (I)
   DSOCMBRAMABUS,                 // (O) BRAM ADDR
   DSOCMBRAMBYTEWRITE,            // (O) BRAM BYTE ENABLE
   DSOCMBRAMEN,                   // (O) BRAM ENABLE
   DSOCMBRAMWRDBUS,               // (O) STORE DATA BUS TO BRAM
   BRAMDSOCMRDDBUS,               // (I) LOAD DATA BUS FROM BRAM
   DSOCMBUSY,                     // (O) BRAM TRANSACTIONS IN PROGRESS
   TIEDSOCMDCRADDR,               // (I) DSOCM DCR REG ADDRESS
   DSARCVALUE,                    // (I) DEFAULT DSARC REG VALUE WHEN RESET
   DSCNTLVALUE,                   // (I) DEFAULT DSCNTL REG VALUE WHEN RESET

// DEVICE CONTROL REGISTER (DCR) INTERFACE
   C405DCRREAD,                   // (O) CPU IS REQUESTING TO READ DATA FROM 
                                  //     A DCR INTO CPU
   C405DCRWRITE,                  // (O) CPU IS REQUESTING TO WRITE FROM CPU TO A DCR
   C405DCRABUS,                   // (O) DCR ADDRESS
   C405DCRDBUSOUT,                // (O) DCR WRITE DATA BUS FROM CPU TO DCR
   DCRC405ACK,                    // (I) INDICATES THAT THE TARGET DCR DATA 
                                  //     IS AVAILABLE FOR READ/WRITE
   DCRC405DBUSIN,                 // (I) DCR READ DATA BUS FROM DCR TO CPU

// EXTERNAL INTERRUPT CONTROLLER INTERFACE
   EICC405EXTINPUTIRQ,            // (I) EXTERNAL NON-CRITICAL INTERRUPT OCCURRED
   EICC405CRITINPUTIRQ,           // (I) CRITICAL INTERRUPT OCCURRED

// JTAG INTERFACE
   JTGC405BNDSCANTDO,             // (I) INPUT FED FROM JTAG BOUNDARY SCAN 
   JTGC405TCK,                    // (I) JTAG TEST CLOCK
   JTGC405TDI,                    // (I) JTAG TEST DATA IN
   JTGC405TMS,                    // (I) JTAG TEST MODE SELECT
   JTGC405TRSTNEG,                // (I) JTAG TEST RESET (MUST BE ASSERTED AT POR)
   C405JTGTDO,                    // (O) JTAG TEST DATA OUT
   C405JTGTDOEN,                  // (O) 405 CORE'S DRIVER ENABLE SIGNAL FOR TDO
   C405JTGEXTEST,                 // (O) INDICATES JTAG EXTEST INSTRUCTION IS SELECTED
   C405JTGCAPTUREDR,              // (O) INDICATES JTAG CAPTURE-DR CONTROLLER 
                                  //     STATE IS SELECTED
   C405JTGSHIFTDR,                // (O) INDICATES JTAG SHIFT-DR CONTROLLER 
                                  //     STATE IS SELECTED
   C405JTGUPDATEDR,               // (O) INDICATES JTAG UPDATE-DR CONTROLLER 
                                  //     STATE IS SELECTED

// DEBUG INTERFACE
   DBGC405DEBUGHALT,              // (I) INDICATES THAT AN EXTERNAL SOURCE IS 
                                  //     REQUESTING TO STOP THE PROCESSOR
   DBGC405UNCONDDEBUGEVENT,       // (I) FEEDS THE UDE BIT OF THE DBSR.  ALLOWS 
                                  //     CUSTOMER LOGIC TO INTERRUPT NORMAL CPU FLOW
   DBGC405EXTBUSHOLDACK,          // (I) INDICATES THAT THE BIU/PLB HAS GIVEN 
                                  //     ITS EXTERNAL BUSSES TO AN EXTERNAL MASTER
                                  //     (THUS THE BUS IS UNAVAIL. FOR DEBUG)
   C405DBGMSRWE,                  // (O) INDICATES THE PROCESSOR IS IN THE WAIT STATE
   C405DBGSTOPACK,                // (O) INDICATES THE PROCESSOR IS STOPPED
   C405DBGWBCOMPLETE,             // (O)
   C405DBGWBFULL,                 // (O)
   C405DBGWBIAR,                  // (O)

// TRACE INTERFACE
   C405TRCTRIGGEREVENTOUT,        // (O) CPU TRIGGER EVENT INDICATION FOR DEBUG LOGIC
   C405TRCTRIGGEREVENTTYPE,       // (O) IDENTIFIES THE DEBUG EVENT THAT CAUSED 
                                  //     TRIGGEREVENTOUT
   C405TRCCYCLE,                  // (O)
   C405TRCTRACESTATUS,            // (O) ENCODED TRACE STATUS BUS
   TRCC405TRACEDISABLE,           // (I)
   TRCC405TRIGGEREVENTIN          // (I) TRIGGER EVENT INPUT TO TRACE LOGIC
);


output    
     C405CPMCORESLEEPREQ,
     C405CPMMSRCE, C405CPMMSREE, C405CPMTIMERIRQ, C405CPMTIMERRESETREQ,
     C405DBGMSRWE, C405DBGSTOPACK,
     C405DBGWBCOMPLETE, C405DBGWBFULL, C405DCRREAD, C405DCRWRITE,
     C405JTGCAPTUREDR, C405JTGEXTEST, C405JTGPGMOUT, C405JTGSHIFTDR,
     C405JTGTDO, C405JTGTDOEN, C405JTGUPDATEDR, 
     C405PLBDCUABORT, C405PLBDCUCACHEABLE,
     C405PLBDCUGUARDED, C405PLBDCUREQUEST, C405PLBDCURNW,
     C405PLBDCUSIZE2, C405PLBDCUU0ATTR, C405PLBDCUWRITETHRU,
     C405PLBICUABORT, C405PLBICUCACHEABLE, C405PLBICUREQUEST,
     C405PLBICUU0ATTR, C405RSTCHIPRESETREQ, C405RSTCORERESETREQ,
     C405RSTSYSRESETREQ, C405TRCCYCLE, C405TRCTRIGGEREVENTOUT,
     C405XXXMACHINECHECK, DSOCMBRAMEN, DSOCMBUSY, ISOCMBRAMEVENWRITEEN,
     ISOCMBRAMODDWRITEEN, ISOCMBRAMEN;

input    
     CPMC405CLOCK, CPMC405CORECLKINACTIVE, 
     CPMC405CPUCLKEN, CPMC405JTAGCLKEN, CPMC405TIMERCLKEN,
     CPMC405TIMERTICK, DBGC405DEBUGHALT, DBGC405EXTBUSHOLDACK,
     DBGC405UNCONDDEBUGEVENT, DCRC405ACK, EICC405CRITINPUTIRQ,
     EICC405EXTINPUTIRQ, JTGC405BNDSCANTDO, JTGC405TCK,
     JTGC405TDI, JTGC405TMS, JTGC405TRSTNEG, 
     MCBCPUCLKEN, MCBJTAGEN,
     MCBTIMEREN, MCPPCRST, PLBC405DCUADDRACK, PLBC405DCUBUSY,
     PLBC405DCUERR, PLBC405DCURDDACK, PLBC405DCUSSIZE1,
     PLBC405DCUWRDACK, PLBC405ICUADDRACK, PLBC405ICUBUSY,
     PLBC405ICUERR, PLBC405ICURDDACK, PLBC405ICUSSIZE1,
     RSTC405RESETCHIP, RSTC405RESETCORE, RSTC405RESETSYS, PLBCLK,
     BRAMDSOCMCLK, BRAMISOCMCLK, 
     TIEC405DETERMINISTICMULT, TIEC405DISOPERANDFWD, TIEC405MMUEN,
     TRCC405TRACEDISABLE, TRCC405TRIGGEREVENTIN;


output [0:31] C405PLBDCUABUS;
output [0:1]  C405PLBDCUPRIORITY;
output [8:28] ISOCMBRAMRDABUS;
output [0:3]  DSOCMBRAMBYTEWRITE;
output [0:1]  C405TRCEVENEXECUTIONSTATUS;
output [0:31] ISOCMBRAMWRDBUS;
output [0:29] C405DBGWBIAR;
output [0:1]  C405PLBICUPRIORITY;
output [0:31] C405DCRDBUSOUT;
output [0:31] DSOCMBRAMWRDBUS;
output [0:29] C405PLBICUABUS;
output [0:9]  C405DCRABUS;
output [2:3]  C405PLBICUSIZE;
output [0:1]  C405TRCODDEXECUTIONSTATUS;
output [0:10] C405TRCTRIGGEREVENTTYPE;
output [0:63] C405PLBDCUWRDBUS;
output [0:7]  C405PLBDCUBE;
output [0:3]  C405TRCTRACESTATUS;
output [8:28] ISOCMBRAMWRABUS;
output [8:29] DSOCMBRAMABUS;


input [0:63] PLBC405DCURDDBUS;
input [1:3]  PLBC405ICURDWDADDR;
input [0:31] BRAMDSOCMRDDBUS;
input [0:63] BRAMISOCMRDDBUS;
input [0:7]  ISARCVALUE;
input [0:31] DCRC405DBUSIN;
input [0:7]  TIEISOCMDCRADDR;
input [0:7]  DSCNTLVALUE;
input [1:3]  PLBC405DCURDWDADDR;
input [0:7]  TIEDSOCMDCRADDR;
input [0:7]  ISCNTLVALUE;
input [0:7]  DSARCVALUE;
input [0:63] PLBC405ICURDDBUS;

wire         bfm_error_detect;
wire         dcu_master_error;
wire  [0:31] dcr_synch_out;
wire  [0:31] dcu_synch_out;
wire         icu_master_error;
wire  [0:31] icu_synch_out;
wire  [0:31] nc;
wire         sim_reset;
wire  [0:31] synch_in;
wire  [0:31] synch_in_rst_key;
wire  [0:31] synch_out;
    
assign synch_out = dcu_synch_out | icu_synch_out | dcr_synch_out;

assign bfm_error_detect = icu_master_error | dcu_master_error;

assign sim_reset = RSTC405RESETCHIP | RSTC405RESETCORE | RSTC405RESETSYS;

// During reset, pretend to the PLB and DCR masters that a PLB/DCR bus
// monitor is instantiated. This prevents problems with the BFM masters
// when a monitor is not used
assign synch_in_rst_key = (sim_reset)? 32'h0100_0300 : synch_in;

// Instantiate Testbench Master to Emulate CPU for ICU interface
plb_master3x #(2'b01,        // plb_master_size
               0,            // plb_master_num
               32'h00000000, // plb_master_addr_lo_0
               32'hffffffff, // plb_master_addr_hi_0
               32'h00000000, // plb_master_addr_lo_1
               32'hffffffff  // plb_master_addr_hi_1
              ) M_ICU
  (
    .synch_out       (icu_synch_out              ),
    .synch_in        (synch_in_rst_key           ),
    .plb_master_error(icu_master_error           ),
    .plb_clk         (PLBCLK                     ),
    .plb_reset       (sim_reset                  ),
    .plb_maddrack    (PLBC405ICUADDRACK          ),
    .plb_mssize      ({1'b0, PLBC405ICUSSIZE1}   ),
    .plb_mrearbitrate(1'b0                       ),
    .plb_mbusy       (PLBC405ICUBUSY             ),
    .plb_merr        (PLBC405ICUERR              ),
    .plb_mwrdack     (1'b0                       ),
    .plb_mrddbus     (PLBC405ICURDDBUS           ),
    .plb_mrdwdaddr   ({1'b1, PLBC405ICURDWDADDR} ),
    .plb_mrddack     (PLBC405ICURDDACK           ),
    .plb_mrdbterm    (1'b0                       ),
    .plb_mwrbterm    (1'b0                       ),
    .m_request       (C405PLBICUREQUEST          ),
    .m_priority      (C405PLBICUPRIORITY         ),
    .m_buslock       (                           ),
    .m_rnw           (                           ),
    .m_be            (                           ),
    .m_msize         (                           ),
    .m_size          ({nc[0:1], C405PLBICUSIZE}  ),
    .m_type          (                           ),
    .m_compress      (C405PLBICUU0ATTR           ),
    .m_guarded       (                           ),
    .m_ordered       (                           ),
    .m_lockerr       (                           ),
    .m_abort         (C405PLBICUABORT            ),
    .m_uabus         (                           ),
    .m_abus          ({C405PLBICUABUS, nc[2:3]}  ),
    .m_wrdbus        (                           ),
    .m_wrburst       (                           ),
    .m_rdburst       (                           )
  );

  assign C405PLBICUCACHEABLE  = 1'b0;

// Instantiate PLB Testbench Master to Emulate CPU for DCU interface
plb_master3x #(2'b01,        // plb_master_size
                1,            // plb_master_num
                32'h00000000, // plb_master_addr_lo_0
                32'hffffffff, // plb_master_addr_hi_0
                32'h00000000, // plb_master_addr_lo_1
                32'hffffffff  // plb_master_addr_hi_1
               ) M_DCU
  (
    .synch_out       (dcu_synch_out              ),
    .synch_in        (synch_in_rst_key           ),
    .plb_master_error(dcu_master_error           ),
    .plb_clk         (PLBCLK                     ),
    .plb_reset       (sim_reset                  ),
    .plb_maddrack    (PLBC405DCUADDRACK          ),
    .plb_mssize      ({1'b0, PLBC405DCUSSIZE1}   ),
    .plb_mrearbitrate(1'b0                       ),
    .plb_mbusy       (PLBC405DCUBUSY             ),
    .plb_merr        (PLBC405DCUERR              ),
    .plb_mwrdack     (PLBC405DCUWRDACK           ),
    .plb_mrddbus     (PLBC405DCURDDBUS           ),
    .plb_mrdwdaddr   ({1'b1, PLBC405DCURDWDADDR} ),
    .plb_mrddack     (PLBC405DCURDDACK           ),
    .plb_mrdbterm    (1'b0                       ),
    .plb_mwrbterm    (1'b0                       ),
    .m_request       (C405PLBDCUREQUEST          ),
    .m_priority      (C405PLBDCUPRIORITY         ),
    .m_buslock       (                           ),
    .m_rnw           (C405PLBDCURNW              ),
    .m_be            (C405PLBDCUBE               ),
    .m_msize         (                           ),
    .m_size          ({nc[4:5], C405PLBDCUSIZE2, nc[6]} ),
    .m_type          (                           ),
    .m_compress      (C405PLBDCUU0ATTR           ),
    .m_guarded       (C405PLBDCUGUARDED          ),
    .m_ordered       (                           ),
    .m_lockerr       (                           ),
    .m_abort         (C405PLBDCUABORT            ),
    .m_uabus         (                           ),
    .m_abus          (C405PLBDCUABUS             ),
    .m_wrdbus        (C405PLBDCUWRDBUS           ),
    .m_wrburst       (                           ),
    .m_rdburst       (                           )
  );

  assign  C405PLBDCUCACHEABLE = 1'b0;
  assign  C405PLBDCUWRITETHRU = 1'b0;

// Instantiate DCR Master to allow emulation of DCR accesses from CPU
dcr_master #(4'h0,
             10'b0000000000,
             10'b1111111111,
             10'b0000000000,
             10'b1111111111
            ) M_DCR
  (
    .synch_out(dcr_synch_out),
    .synch_in(synch_in_rst_key),
    .dcr_clk(PLBCLK),
    .dcr_reset(sim_reset),
    .dcr_cpuack(DCRC405ACK),
    .dcr_cpudbusin(DCRC405DBUSIN),
    .cpu_dcrwrite(C405DCRWRITE),
    .cpu_dcrread(C405DCRREAD),
    .cpu_dcrabus(C405DCRABUS),
    .cpu_dcrdbusout(C405DCRDBUSOUT)
  );

// Assign all other outputs to idle state

   assign C405CPMCORESLEEPREQ = 0;
   assign C405CPMMSRCE = 0;
   assign C405CPMMSREE = 0;
   assign C405CPMTIMERIRQ = 0;
   assign C405CPMTIMERRESETREQ = 0;
   assign C405DBGMSRWE = 0;
   assign C405DBGSTOPACK = 0;
   assign C405DBGWBCOMPLETE = 0;
   assign C405DBGWBFULL = 0;
   assign C405JTGCAPTUREDR = 0;
   assign C405JTGEXTEST = 0;
   assign C405JTGPGMOUT = 0;
   assign C405JTGSHIFTDR = 0;
   assign C405JTGTDO = 0;
   assign C405JTGTDOEN = 0;
   assign C405JTGUPDATEDR = 0;
   assign C405RSTCHIPRESETREQ = 0;
   assign C405RSTCORERESETREQ = 0;
   assign C405RSTSYSRESETREQ = 0;
   assign C405TRCCYCLE = 0;
   assign C405TRCTRIGGEREVENTOUT = 0;
   assign C405XXXMACHINECHECK = 0;
   assign DSOCMBRAMEN = 0;
   assign DSOCMBUSY = 0;
   assign ISOCMBRAMEVENWRITEEN = 0;
   assign ISOCMBRAMODDWRITEEN = 0;
   assign ISOCMBRAMEN = 0;

   assign ISOCMBRAMRDABUS[8:28] = 21'b0;
   assign DSOCMBRAMBYTEWRITE[0:3] = 4'b0;
   assign C405TRCEVENEXECUTIONSTATUS[0:1] = 2'b0;
   assign ISOCMBRAMWRDBUS[0:31] = 32'b0;
   assign C405DBGWBIAR[0:29] = 30'b0;
   assign DSOCMBRAMWRDBUS[0:31] = 32'b0;
   assign C405TRCODDEXECUTIONSTATUS[0:1] = 2'b0;
   assign C405TRCTRIGGEREVENTTYPE[0:10] = 11'b0;
   assign C405TRCTRACESTATUS[0:3] = 4'b0;
   assign ISOCMBRAMWRABUS[8:28] = 21'b0;
   assign DSOCMBRAMABUS[8:29] = 22'b0;
   
endmodule
