/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2002 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/******************************************************************************
*
* FILENAME:
*
* xpci_example_level_0.c
*
* DESCRIPTION:
*
* This file contains design examples using level 0 of the PCI bridge
* driver XPci.  
*
* NOTES:
*
* These code fragments will illustrate how the XPci component can be used to:
*   - Initialize a pci bridge core
*   - Initialize a remote device on the pci bus
*   - Routine that prints messages describing the bridge and memory mappings
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a rmm  04/11/03 First release
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "xparameters.h"    /* defines XPAR constants */
#include "xpci_l.h"         /* XPci level 0 interface */
#include "stdio.h"

/************************** Constant Definitions *****************************/

/* bus/sub-bus number segment to use */
#define PCI_BUSNO               0  
#define PCI_SUBBUSNO            0

/*
 * Define hard-coded configuration address constants
 */
#define CAR_BUS0_DEV0_VENDOR   0x80000000  /* local vendor/device id */
#define CAR_BUS0_DEV0_STATCMD  0x80000004  /* local status/cmd register */
#define CAR_BUS0_DEV0_LATENCY  0x8000000C  /* local latency/cache register */
#define CAR_BUS0_DEV0_BAR0     0x80000010  /* local BAR0 register */
#define CAR_BUS0_DEV0_BAR1     0x80000014  /* local BAR1 register */
#define CAR_BUS0_DEV0_BAR2     0x80000018  /* local BAR2 register */
#define CAR_BUS0_DEV1_VENDOR   0x80000800  /* remote vendor/device id */
#define CAR_BUS0_DEV1_STATCMD  0x80000804  /* remote status/cmd register */
#define CAR_BUS0_DEV1_LATENCY  0x8000080C  /* remote latency/cache register */
#define CAR_BUS0_DEV1_BAR0     0x80000810  /* remote BAR1 register */
#define CAR_BUS0_DEV1_BAR1     0x80000814  /* remote BAR1 register */
#define CAR_BUS0_DEV1_BAR2     0x80000818  /* remote BAR2 register */
#define CAR_BUS0_DEV1_BAR3     0x8000081C  /* remote BAR3 register */
#define CAR_BUS0_DEV1_BAR4     0x80000820  /* remote BAR4 register */
#define CAR_BUS0_DEV1_BAR5     0x80000824  /* remote BAR5 register */

/*
 * Define BAR register sizes from XPARs
 */
#define IPIF_BAR0_SZ   (mParPci(IPIF_HIGHADDR_0) - mParPci(IPIFBAR_0 + 1))
#define IPIF_BAR1_SZ   (mParPci(IPIF_HIGHADDR_1) - mParPci(IPIFBAR_1 + 1))
#define IPIF_BAR2_SZ   (mParPci(IPIF_HIGHADDR_2) - mParPci(IPIFBAR_2 + 1))
#define IPIF_BAR3_SZ   (mParPci(IPIF_HIGHADDR_3) - mParPci(IPIFBAR_3 + 1))
#define IPIF_BAR4_SZ   (mParPci(IPIF_HIGHADDR_4) - mParPci(IPIFBAR_4 + 1))
#define IPIF_BAR5_SZ   (mParPci(IPIF_HIGHADDR_5) - mParPci(IPIFBAR_5 + 1))
#define PCI_BAR0_SZ    ((Xuint32)(1 << mParPci(PCIBAR_LEN_0)))
#define PCI_BAR1_SZ    ((Xuint32)(1 << mParPci(PCIBAR_LEN_1)))
#define PCI_BAR2_SZ    ((Xuint32)(1 << mParPci(PCIBAR_LEN_2)))


/**************************** Type Definitions *******************************/


/***************** Macros (Inline Functions) Definitions *********************/

/*
 * mParPci usage
 *
 * PCI_NAME is the prefix to the name of the PCI component defined in
 * xparameters.h The following example explains how assigning constant names
 * works during preprocessing:
 *
 *    #define PCI_NAME   XPAR_OPB_PCI_1_
 *      ...
 *    #define BAR0       mParPci(IPIFBAR_0)
 *
 * The constant BAR0 will be set to the value of XPAR_OPB_PCI_1_IPIFBAR_0
 * This construct is useful if this code is used in a different system with
 * a different name given to the pci component. All that needs to be changed
 * is the definition of PCI_NAME.
 */
#define PCI_NAME           XPAR_OPB_PCI_1_
#define mCat(prefix,value) prefix ## value
#define mXcat(x,y)         mCat(x,y)
#define mParPci(x)         mXcat(PCI_NAME,x)

/************************** Function Prototypes ******************************/


/************************** Variable Definitions *****************************/


/************************** Function Prototypes ******************************/
void PciInitLevel_0(void);
void ShowPci(void);
void StuffBar(Xuint32 Car, Xuint32 Bar, Xuint8 Prefetch, Xuint8 Space);

/************************** Variable Definitions *****************************/


/****************************************************************************
 *
 * PciInitLevel_0 - initialize host bridge and an external device on the PCI bus
 *
 * This function is an example of how to initialize the PCI IP core with
 * level 0 of the XPci component driver.
 *
 * The example assumes there is a host bridge and one other device on the
 * PCI bus.
 *
 * Parameters:
 *   none
 *
 * Returns:
 *   nothing
 *
 ****************************************************************************/
void PciInitLevel_0(void)
{
    /**********************************************************************
     * Step 1, initialize the IP core IPIF registers
     *********************************************************************/
    
    /*
     * Place the core in a quiet state by resetting it
     * Set the bus and sub-bus number
     */
    XPci_mReset(mParPci(BASEADDR));
    XPci_mWriteReg(mParPci(BASEADDR), XPCI_BUSNO_OFFSET,
                   (PCI_BUSNO << 16) | PCI_SUBBUSNO);

    /**********************************************************************
     * Step 2, perform configuration writes to the PCI header to setup the
     * bridge
     *********************************************************************/
    
    /*
     * Clear error bits in PCI header status register, set bridge as a bus
     * master and allow memory transactions
     */
    XPci_mConfigOut(mParPci(BASEADDR), CAR_BUS0_DEV0_STATCMD,
                    XPCI_STATCMD_ERR_MASK |
                    XPCI_STATCMD_MEM_EN |
                    XPCI_STATCMD_BUSM_EN);

    /*
     * Set latency timer to maximum
     */
    XPci_mConfigOut(mParPci(BASEADDR), CAR_BUS0_DEV0_LATENCY, 0x0000FF00);
    
    /*
     * Stuff the bridge's PCI bar registers
     * The data we use to write to the BARs originates from the XPARs in
     * xparameters.h which originates from parameters assigned by the designer
     * in XPS or an equivalent design tool that instantiates the core
     */
    StuffBar(CAR_BUS0_DEV0_BAR0, mParPci(PCIBAR_0),
             mParPci(PCI_PREFETCH_0), mParPci(PCI_SPACETYPE_0));
             
    StuffBar(CAR_BUS0_DEV0_BAR1, mParPci(PCIBAR_1),
             mParPci(PCI_PREFETCH_1), mParPci(PCI_SPACETYPE_1));
             
    StuffBar(CAR_BUS0_DEV0_BAR2, mParPci(PCIBAR_2),
             mParPci(PCI_PREFETCH_2), mParPci(PCI_SPACETYPE_2));

    /**********************************************************************
     * Step 3, perform configuration writes to the PCI header to setup a
     * remote device on the pci bus
     *********************************************************************/
    
    /*
     * Clear error bits in PCI header status register, allow memory transactions
     */
    XPci_mConfigOut(mParPci(BASEADDR), CAR_BUS0_DEV1_STATCMD,
                    XPCI_STATCMD_ERR_MASK | XPCI_STATCMD_MEM_EN);

    /*
     * Set latency timer to maximum
     */
    XPci_mConfigOut(mParPci(BASEADDR), CAR_BUS0_DEV1_LATENCY, 0x0000FF00);
    
    /*
     * Set all 6 BARs. The values used in this example are arbitrary
     */
    XPci_mConfigOut(mParPci(BASEADDR), CAR_BUS0_DEV1_BAR0,
                    0x70000000 |
                    XPCI_HDR_BAR_PREFETCH_YES |
                    XPCI_HDR_BAR_SPACE_MEMORY);

    XPci_mConfigOut(mParPci(BASEADDR), CAR_BUS0_DEV1_BAR1,
                    0x80000000 |
                    XPCI_HDR_BAR_PREFETCH_NO |
                    XPCI_HDR_BAR_SPACE_MEMORY);
    
    XPci_mConfigOut(mParPci(BASEADDR), CAR_BUS0_DEV1_BAR2,
                    0x90000000 |
                    XPCI_HDR_BAR_PREFETCH_NO |
                    XPCI_HDR_BAR_SPACE_MEMORY);
    
    XPci_mConfigOut(mParPci(BASEADDR), CAR_BUS0_DEV1_BAR3,
                    0xA0000000 |
                    XPCI_HDR_BAR_PREFETCH_NO |
                    XPCI_HDR_BAR_SPACE_MEMORY);
    
    XPci_mConfigOut(mParPci(BASEADDR), CAR_BUS0_DEV1_BAR4,
                    0xB0000000 |
                    XPCI_HDR_BAR_PREFETCH_YES |
                    XPCI_HDR_BAR_SPACE_MEMORY);
    
    XPci_mConfigOut(mParPci(BASEADDR), CAR_BUS0_DEV1_BAR5,
                    0xC0000000 |
                    XPCI_HDR_BAR_PREFETCH_YES |
                    XPCI_HDR_BAR_SPACE_MEMORY);
    

    /**********************************************************************
     * Step 4, enable interrupts when a master abort write occurs
     *********************************************************************/

    /* enable pci interrupts to occur */
    XPci_mIntrEnable(mParPci(BASEADDR), XPCI_IPIF_INT_PCI);
    
    /* enable specific master abort interrupt in PCI core */
    XPci_mIntrPciEnable(mParPci(BASEADDR), XPCI_IR_LM_MA_W);

    /* enable interrupt signal to an external interrupt controller */
    XPci_mIntrGlobalEnable(mParPci(BASEADDR));

}
    

/****************************************************************************
 *
 * ShowPci - print some information about the PCI bus
 *
 * Parameters:
 *   none
 *
 * Returns:
 *   nothing
 *
 ****************************************************************************/
void ShowPci(void)
{
    Xuint32 Data;
    
    /*
     * print vendor/device id and status/command register contents
     * first for the local bridge then the remote device
     */
    XPci_mConfigIn(mParPci(BASEADDR), CAR_BUS0_DEV0_VENDOR, Data);
    printf("Local Vendor/Device ID  = %08X\n", Data);
    
    XPci_mConfigIn(mParPci(BASEADDR), CAR_BUS0_DEV0_STATCMD, Data);
    printf("Local Status/Command    = %08X\n", Data);
    
    XPci_mConfigIn(mParPci(BASEADDR), CAR_BUS0_DEV1_VENDOR, Data);
    printf("Remote Vendor/Device ID = %08X\n", Data);

    XPci_mConfigIn(mParPci(BASEADDR), CAR_BUS0_DEV1_STATCMD, Data);
    printf("Remote Status/Command   = %08X\n", Data);

    /*
     * Using the XPARs defined in xparameters.h, print out a table of
     * how the bridge is mapping local memory space to the pci bus
     *
     * Output will look like:
     *
     *   Master local to PCI address mappings
     *      Local: 80000000 - 81FFFFFF     PCI: 70000000 - 71FFFFFF
     *      Local: 82000000 - 820007FF     PCI: 50000000 - 500007FF
     *      Local: 82320000 - 823207FF     PCI: 82330000 - 8233FFFF
     *      Local: 82330000 - 8233FFFF     PCI: 82340000 - 8234FFFF
     *      Local: 82340000 - 8234FFFF     PCI: 82350000 - 8235FFFF
     *      Local: 82350000 - 8235FFFF     PCI: 82360000 - 8236FFFF
     *
     *   Slave local to PCI address mappings
     *      Local: 00000000 - 07FFFFFF     PCI: 10000000 - 17FFFFFF
     *      Local: FFFF8000 - FFFFFFFF     PCI: 3F000000 - 3F007FFF
     *      Local: FFFF8000 - FFFFFFFF     PCI: 3F000000 - 3F007FFF
     *
     */
    printf("\n\nMaster local to PCI address mappings\n");

    /* ipif bar 0 - IPIF local address range */
    printf("   Local: %08X - %08X     ", mParPci(IPIFBAR_0),
           mParPci(IPIFBAR_0) + IPIF_BAR0_SZ - 1);

    /* ipif bar 0 - PCI address range */
    Data = XPci_mLocal2Pci(mParPci(IPIFBAR_0), mParPci(IPIFBAR2PCI_0));
    printf("PCI: %08X - %08X\n", Data, Data + IPIF_BAR0_SZ - 1);

    /* ipif bar 1 - IPIF local address range */
    printf("   Local: %08X - %08X     ", mParPci(IPIFBAR_1),
           mParPci(IPIFBAR_1) + IPIF_BAR1_SZ - 1);

    /* ipif bar 1 - PCI address range */
    Data = XPci_mLocal2Pci(mParPci(IPIFBAR_1), mParPci(IPIFBAR2PCI_1));
    printf("PCI: %08X - %08X\n", Data, Data + IPIF_BAR1_SZ - 1);

    /* ipif bar 2 - IPIF local address range */
    printf("   Local: %08X - %08X     ", mParPci(IPIFBAR_2),
           mParPci(IPIFBAR_2) + IPIF_BAR1_SZ - 1);

    /* ipif bar 2 - PCI address range */
    Data = XPci_mLocal2Pci(mParPci(IPIFBAR_2), mParPci(IPIFBAR2PCI_2));
    printf("PCI: %08X - %08X\n", Data, Data + IPIF_BAR2_SZ - 1);

    /* ipif bar 3 - IPIF local address range */
    printf("   Local: %08X - %08X     ", mParPci(IPIFBAR_3),
           mParPci(IPIFBAR_3) + IPIF_BAR3_SZ - 1);

    /* ipif bar 3 - PCI address range */
    Data = XPci_mLocal2Pci(mParPci(IPIFBAR_3), mParPci(IPIFBAR2PCI_3));
    printf("PCI: %08X - %08X\n", Data, Data + IPIF_BAR3_SZ - 1);

    /* ipif bar 4 - IPIF local address range */
    printf("   Local: %08X - %08X     ", mParPci(IPIFBAR_4),
           mParPci(IPIFBAR_4) + IPIF_BAR4_SZ - 1);

    /* ipif bar 4 - PCI address range */
    Data = XPci_mLocal2Pci(mParPci(IPIFBAR_4), mParPci(IPIFBAR2PCI_4));
    printf("PCI: %08X - %08X\n", Data, Data + IPIF_BAR4_SZ - 1);

    /* ipif bar 5 - IPIF local address range */
    printf("   Local: %08X - %08X     ", mParPci(IPIFBAR_5),
           mParPci(IPIFBAR_5) + IPIF_BAR5_SZ - 1);

    /* ipif bar 5 - PCI address range */
    Data = XPci_mLocal2Pci(mParPci(IPIFBAR_5), mParPci(IPIFBAR2PCI_5));
    printf("PCI: %08X - %08X\n", Data, Data + IPIF_BAR5_SZ - 1);

    
    printf("\n\nSlave local to PCI address mappings\n");

    /* pci bar 0 - IPIF local address range */
    Data = XPci_mPci2Local(mParPci(PCIBAR_0), mParPci(PCIBAR2IPIF_0));
    printf("   Local: %08X - %08X     ", Data, Data + PCI_BAR0_SZ - 1);

    /* pci bar 0 - PCI address range */
    printf("PCI: %08X - %08X\n", mParPci(PCIBAR_0),
           mParPci(PCIBAR_0) + PCI_BAR0_SZ - 1);

    /* pci bar 1 - IPIF local address range */
    Data = XPci_mPci2Local(mParPci(PCIBAR_1), mParPci(PCIBAR2IPIF_1));
    printf("   Local: %08X - %08X     ", Data, Data + PCI_BAR1_SZ - 1);

    /* pci bar 1 - PCI address range */
    printf("PCI: %08X - %08X\n", mParPci(PCIBAR_1),
           mParPci(PCIBAR_1) + PCI_BAR1_SZ - 1);

    /* pci bar 2 - IPIF local address range */
    Data = XPci_mPci2Local(mParPci(PCIBAR_2), mParPci(PCIBAR2IPIF_2));
    printf("   Local: %08X - %08X     ", Data, Data + PCI_BAR2_SZ - 1);

    /* pci bar 2 - PCI address range */
    printf("PCI: %08X - %08X\n\n", mParPci(PCIBAR_2),
           mParPci(PCIBAR_2) + PCI_BAR2_SZ - 1);
}

/****************************************************************************
 *
 * StuffBar - set the contents of a BAR register
 *
 * This local function helps write the local BAR registers of the PCI bridge.
 * It uses constants that are derived from xparameters.h in order to determine
 * how the BAR is written.
 *
 * Parameters:
 *   Car - Configuration address of BAR
 *   Bar - Base address component of BAR
 *   Prefetch - Prefetchability of memory space described by BAR
 *   Space - Type of memory space described by BAR
 *
 * Returns:
 *   nothing
 *
 ****************************************************************************/
void StuffBar(Xuint32 Car, Xuint32 Bar, Xuint8 Prefetch, Xuint8 Space)
{
    /* prefetchable? 1=prefetchable, 0=non-prefetchable */
    if (Prefetch == 1)
    {
        Bar |= XPCI_HDR_BAR_PREFETCH_YES;
    }

    /* IO or memory space? 1=memory space, 0=io space */
    if (Space == 0)
    {
        Bar |= XPCI_HDR_BAR_SPACE_IO;
    }

    /* write PCI header BAR register */
    XPci_mConfigOut(mParPci(BASEADDR), Car, Bar);
}


    
/****************************************************************************
 *
 * main - entry point for this example
 *
 * Returns:
 *   nothing
 *
 ****************************************************************************/
int main(int argc, char **argv)
{
    PciInitLevel_0();
    ShowPci();
}
