/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2002 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/******************************************************************************
*
* FILENAME:
*
* xpci_example_level_1.c
*
* DESCRIPTION:
*
* This file contains a design examples using level 1 of the PCI bridge
* driver XPci.
*
* NOTES:
*
* These code fragments will illustrate how the XPci component can be used to:
*   - Initialize a pci bridge core
*   - Initialize a remote device on the pci bus
*   - Extract DMA implementation and initialize it
*   - Interrupt driven DMA and error handling
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a rmm  04/11/03 First release
*
******************************************************************************/

/***************************** Include Files *********************************/
#include "xparameters.h"
#include "xpci.h"
#include "xdma_channel.h"
#include "xexception_l.h"
#include "xintc.h"
#include "stdio.h"

/************************** Constant Definitions *****************************/

/* bus/sub-bus number segment to use */
#define PCI_BUSNO               0  
#define PCI_SUBBUSNO            0

#define DMA_TRANSFER_BYTES      0x100
#define DMA_TRANSFER_LONGS      (DMA_TRANSFER_BYTES / sizeof(Xuint32))

/* interrupt controller xpars */
#define INT_CTRL_DEVICE_ID      XPAR_OPB_INTC_0_DEVICE_ID

/* interrupt vector assigned to the bridge */
#define PCI_CORE_INTERRUPT_VEC  XPAR_OPB_INTC_0_OPB_PCI_1_IP2INTC_IRPT_INTR

/*
 * pci device numbers of devices on the PCI bus in this example
 * these values may require adjustment for your system
 */
#define PCI_BRIDGE_DEVICE       0  /* Host bridge device id */
#define PCI_REMOTE_DEVICE       1  /* device id of another pci device */

/*
 * REMOTE_BUFFER_ADDR is an address on the local bus that maps to PCI bus
 * REMOTE_BUFFER_PCI_ADDR is the address in PCI space of REMOTE_BUFFER_ADDR
 */
#define REMOTE_BUFFER_ADDR      XPAR_OPB_PCI_1_IPIFBAR_0
#define REMOTE_BUFFER_PCI_ADDR  XPci_mLocal2Pci(XPAR_OPB_PCI_1_IPIFBAR_0, \
                                                XPAR_OPB_PCI_1_IPIFBAR2PCI_0)

/*
 * remote PCI bar registers
 * One of these BARS must be able to decode REMOTE_BUFFER_PCI_ADDR
 */
#define PCI_REMOTE_BAR0         0x70000008
#define PCI_REMOTE_BAR1         0x50000008
#define PCI_REMOTE_BAR2         0x4F000008


/**************************** Type Definitions *******************************/

/*
 * PciObjs defines a type used to contain a XPci object and its associated
 * DMA instance
 */
typedef struct {
    XPci        Pci;           /* pci bridge */
    XDmaChannel Dma;           /* dma component of pci bridge */
    Xuint8      DmaSemaphore;  /* binary semaphore */
} PciObjs;


/***************** Macros (Inline Functions) Definitions *********************/


/************************** Function Prototypes ******************************/
void PciInitLevel_1(void);
void PciDmaTransfer(Xuint32 LocalAddress, Xuint32 RemoteAddress, Xuint32 Bytes,
                    int Direction);
void PciIsr(PciObjs *ObjPtr);
void PciHandleError(XPciError *ErrStatusPtr);
void SemaphoreGive(Xuint8 *SemPtr);
void SemaphoreTake(Xuint8 *SemPtr);
XStatus SetupInterruptSystem();

/************************** Variable Definitions *****************************/

/* buffer to use as DMA source and destination */
Xuint32 LocalBuffer[DMA_TRANSFER_LONGS];

/* pci/dma object container setup and used by this example */
PciObjs Bridge;

/* external interrupt controller */
XIntc xintc;


/******************************************************************************
 * PciInitLevel_1 - initialize host bridge and an external device on the PCI bus
 *
 * Parameters:
 *   none
 *
 * Returns:
 *   nothing
 *
 *****************************************************************************/
void PciInitLevel_1(void)
{
    XStatus status;
    Xuint32 DmaBaseAddr;
    Xuint8 DmaType;
    XPciError ErrStatus;

    /*******************************************************************/
    /* Initialize driver component for the host bridge                 */
    /*******************************************************************/
    status = XPci_Initialize(&Bridge.Pci, XPAR_OPB_PCI_1_DEVICE_ID,
                             PCI_BUSNO, PCI_SUBBUSNO);
    if (status != XST_SUCCESS)
    {
        printf("Failed to initialize pci\n");
    }

    /* clear any current errors */
    XPci_ErrorClear(&Bridge.Pci, XPCI_CLEAR_ALL_ERRORS);

    /*******************************************************************/
    /* Initialize driver component for the DMA engine                  */
    /*******************************************************************/
    XPci_GetDmaImplementation(&Bridge.Pci, &DmaBaseAddr, &DmaType);
    
    switch(DmaType)
    {
        case XPCI_DMA_TYPE_NONE:
            printf("No DMA engine implemented in this PCI core\n");
            break;

        case XPCI_DMA_TYPE_SG:
            printf("Scatter-gather DMA engine not supported by this example\n");
            break;

        case XPCI_DMA_TYPE_SIMPLE:
            status = XDmaChannel_Initialize(&Bridge.Dma, DmaBaseAddr);
            if (status != XST_SUCCESS)
            {
                printf("Failed to initialize dma\n");
            }
            break;

        default:
            printf("Unknown DMA type\n");
    }


    /*******************************************************************/
    /* Configure the host bridge on the PCI bus                        */
    /*******************************************************************/

    /* enable memory I/O, bus mastering, parity & serr reporting  */
    XPci_ConfigOut32(&Bridge.Pci, PCI_BUSNO, PCI_BRIDGE_DEVICE, 0,
                     XPCI_HDR_COMMAND,
                     XPCI_STATCMD_ERR_MASK |
                     XPCI_STATCMD_MEM_EN |
                     XPCI_STATCMD_BUSM_EN |
                     XPCI_STATCMD_PARITY |
                     XPCI_STATCMD_SERR_EN);

    /* set latency timer */
    XPci_ConfigOut8(&Bridge.Pci, PCI_BUSNO, PCI_BRIDGE_DEVICE, 0,
                    XPCI_HDR_LATENCY, 0xFF);
    
    /*******************************************************************/
    /* Configure the remote device on the PCI bus                      */
    /*******************************************************************/

    /* set bar(s) */
    XPci_ConfigOut32(&Bridge.Pci, PCI_BUSNO, PCI_REMOTE_DEVICE, 0, XPCI_HDR_BAR0,
                     PCI_REMOTE_BAR0);
    XPci_ConfigOut32(&Bridge.Pci, PCI_BUSNO, PCI_REMOTE_DEVICE, 0, XPCI_HDR_BAR1,
                     PCI_REMOTE_BAR1);
    XPci_ConfigOut32(&Bridge.Pci, PCI_BUSNO, PCI_REMOTE_DEVICE, 0, XPCI_HDR_BAR2,
                     PCI_REMOTE_BAR2);
    
    /* latency */
    XPci_ConfigOut8(&Bridge.Pci, PCI_BUSNO, PCI_REMOTE_DEVICE, 0,
                    XPCI_HDR_LATENCY, 0xFF);
    
    /* clear any current errors */
    /* enable memory I/O, bus mastering  */
    XPci_ConfigOut32(&Bridge.Pci, PCI_BUSNO, PCI_REMOTE_DEVICE, 0,
                     XPCI_HDR_COMMAND,
                     XPCI_STATCMD_ERR_MASK |
                     XPCI_STATCMD_MEM_EN |
                     XPCI_STATCMD_BUSM_EN);

    /* after all this configuration access, check for errors */
    XPci_ErrorGet(&Bridge.Pci, &ErrStatus);
    if (ErrStatus.IsError)
    {
        PciHandleError(&ErrStatus);
        XPci_ErrorClear(&Bridge.Pci, XPCI_CLEAR_ALL_ERRORS);
    }

    /*******************************************************************/
    /* Configure PCI & DMA engine for interrupt driven operation       */
    /*******************************************************************/

    /* setup semaphore */
    Bridge.DmaSemaphore = 0;
    
    /* hook up interrupt service routine */
    XIntc_Disconnect(&xintc, PCI_CORE_INTERRUPT_VEC);
    XIntc_Connect(&xintc, PCI_CORE_INTERRUPT_VEC,
                  (XInterruptHandler)PciIsr, &Bridge);
    
    /* enable all possible interrupt sources from the PCI component */
    XPci_InterruptPciEnable(&Bridge.Pci, XPCI_IR_MASK);

    /* enable selected interrupt sources from the DMA component */
    XDmaChannel_SetIntrEnable(&Bridge.Dma,
                              XDC_IXR_DMA_DONE_MASK | XDC_IXR_DMA_ERROR_MASK);

    /* enable PCI & DMA interrupts at the device level */
    XPci_InterruptEnable(&Bridge.Pci, XPCI_IPIF_INT_DMA | XPCI_IPIF_INT_PCI);

    /* enable interrupts from the PCI core */
    XPci_InterruptGlobalEnable(&Bridge.Pci);

    /* finally, enable the interrupt vector at the interrupt controller */
    XIntc_Enable(&xintc, PCI_CORE_INTERRUPT_VEC);
}


/******************************************************************************
 * PciDmaTransfer - interrupt driven DMA to/from a device on the PCI bus
 *
 * Parameters:
 *   LocalAddress - address of a local memory device such as SRAM,DDR,etc.
 *   RemoteAddress - address local to the processor that maps to the PCI bus
 *   Bytes - bytes to transfer
 *   Direction - direction to transfer, 0 = local to remote, 1 = remote to local
 *
 * Returns:
 *   nothing
 *
 *****************************************************************************/
void PciDmaTransfer(Xuint32 LocalAddress, Xuint32 RemoteAddress, Xuint32 Bytes,
                    int Direction)
{
    Xuint32 DmaSrc;
    Xuint32 DmaDest;
    
    /* setup transfer direction */
    if (Direction == 0)
    {
        /* local to PCI */
        XDmaChannel_SetControl(&Bridge.Dma,
                               XDC_DMACR_SOURCE_INCR_MASK |
                               XDC_DMACR_DEST_INCR_MASK |
                               XDC_DMACR_DEST_LOCAL_MASK);
        DmaSrc = LocalAddress;
        DmaDest = RemoteAddress;
    }
    else if (Direction == 1)
    {
        /* PCI to local */
        XDmaChannel_SetControl(&Bridge.Dma,
                               XDC_DMACR_SOURCE_INCR_MASK |
                               XDC_DMACR_DEST_INCR_MASK |
                               XDC_DMACR_SOURCE_LOCAL_MASK);
        DmaSrc = RemoteAddress;
        DmaDest = LocalAddress;
    }
    else
    {
        printf("Invalid direction argument. Must be 0 or 1\n");
        return;
    }

    /* begin transfer */
    XDmaChannel_Transfer(&Bridge.Dma, (Xuint32*)DmaSrc,
                         (Xuint32*)DmaDest, Bytes);

    /* wait for completion */
    SemaphoreTake(&Bridge.DmaSemaphore);
    printf("Dma Transfer complete\n");
}


/******************************************************************************
 * PciIsr - handle interrupts from the PCI & DMA components of the PCI core
 *
 * Parameters:
 *   InstancePtr - points to the core's container object
 *
 * Returns:
 *   nothing
 *
 *****************************************************************************/
void PciIsr(PciObjs *ObjPtr)
{
    Xuint32 DeviceExcData, DmaExcData, PciExcData;
    XPciError PciErrData;

    /* get device level interrupt status */
    DeviceExcData = XPci_InterruptGetStatus(&ObjPtr->Pci);

    /* DMA interrupt? */
    if (DeviceExcData & XPCI_IPIF_INT_DMA)
    {
        /* Get DMA device specific interrupts */
        DmaExcData = XDmaChannel_GetIntrStatus(&ObjPtr->Dma);
        
        /* DMA transfer complete? */
        if (DmaExcData & XDC_IXR_DMA_DONE_MASK)
        {
            SemaphoreGive(&ObjPtr->DmaSemaphore);
        }

        /* DMA transfer error? */
        if (DmaExcData & XDC_IXR_DMA_ERROR_MASK)
        {
            /* Handle DMA error */
        }
        
        /* clear dma exceptions */
        XDmaChannel_SetIntrStatus(&ObjPtr->Dma, DmaExcData);
    }

    /* PCI interrupt? */
    if (DeviceExcData & XPCI_IPIF_INT_PCI)
    {
        /* get PCI device specific interrupt status */
        PciExcData = XPci_InterruptPciGetStatus(&ObjPtr->Pci);
        
        /* get error data from the bridge */
        XPci_ErrorGet(&ObjPtr->Pci, &PciErrData);
        PciHandleError(&PciErrData);

        /* clear reported errors */
        XPci_ErrorClear(&ObjPtr->Pci, &PciErrData);

        /* clear pci exceptions */
        XPci_InterruptPciClear(&ObjPtr->Pci, PciExcData);
    }

    /* clear device level interrupts */
    XPci_InterruptClear(&ObjPtr->Pci, DeviceExcData);
}


/******************************************************************************
 * PciHandleError - handle error conditions from the PCI device
 *
 * Parameters:
 *   ErrStatusPtr - error status information
 *
 * Returns:
 *   nothing
 *
 *****************************************************************************/
void PciHandleError(XPciError *ErrStatusPtr)
{
    /* was there an error */
    if (!ErrStatusPtr->IsError)
    {
        return;
    }

    /* Did error occur on the local bus */
    if (ErrStatusPtr->LocalBusReason)
    {
        if (ErrStatusPtr->LocalBusReason & XPCI_LMADDR_SERR_R)
        {
            /*
             * Handle "Local bus master read system error"
             * ErrStatusPtr->LocalBusReadAddr contains the error address
             */
        }
        if (ErrStatusPtr->LocalBusReason & XPCI_LMADDR_SERR_W)
        {
            /*
             * Handle "Local bus master write system error"
             * ErrStatusPtr->LocalBusWriteAddr contains the error address
             */
        }
        if (ErrStatusPtr->LocalBusReason & XPCI_LMADDR_PERR_R)
        {
            /*
             * Handle "Local bus master read parity error"
             * ErrStatusPtr->LocalBusReadAddr contains the error address
             */
        }
        if (ErrStatusPtr->LocalBusReason & XPCI_LMADDR_PERR_W)
        {
            /*
             * Handle "Local bus master write parity error"
             * ErrStatusPtr->LocalBusWriteAddr contains the error address
             */
        }
        if (ErrStatusPtr->LocalBusReason & XPCI_LMADDR_TA_R)
        {
            /*
             * Handle "Local bus master read target abort error"
             * ErrStatusPtr->LocalBusReadAddr contains the error address
             */
        }
        if (ErrStatusPtr->LocalBusReason & XPCI_LMADDR_TA_W)
        {
            /*
             * Handle "Local bus master write target abort error"
             * ErrStatusPtr->LocalBusWriteAddr contains the error address
             */
        }
        if (ErrStatusPtr->LocalBusReason & XPCI_LMADDR_MA_W)
        {
            /*
             * Handle "Local bus master write abort error"
             * ErrStatusPtr->LocalBusWriteAddr contains the error address
             */
        }
        if (ErrStatusPtr->LocalBusReason & XPCI_LMADDR_BR_W)
        {
            /*
             * Handle "Local bus master burst write retry error"
             * ErrStatusPtr->LocalBusWriteAddr contains the error address
             */
        }
        if (ErrStatusPtr->LocalBusReason & XPCI_LMADDR_BRD_W)
        {
            /*
             * Handle "Local bus master burst write retry disconnect error"
             * ErrStatusPtr->LocalBusWriteAddr contains the error address
             */
        }
        if (ErrStatusPtr->LocalBusReason & XPCI_LMADDR_BRT_W)
        {
            /*
             * Handle "Local bus master burst write timeout error"
             * ErrStatusPtr->LocalBusWriteAddr contains the error address
             */
        }
        if (ErrStatusPtr->LocalBusReason & XPCI_LMADDR_BRANGE_W)
        {
            /*
             * Handle "Local bus master burst write range error"
             * ErrStatusPtr->LocalBusWriteAddr contains the error address
             */
        }
    }

    /* Did error occur on the PCI bus */
    if (ErrStatusPtr->PciReason)
    {
        if (ErrStatusPtr->PciReason & XPCI_PIADDR_ERRACK_R)
        {
            /*
             * Handle "PCI initiator read ErrAck error"
             * ErrStatusPtr->PciReadAddr contains the error address
             */
        }
        if (ErrStatusPtr->PciReason & XPCI_PIADDR_ERRACK_W)
        {
            /*
             * Handle "PCI initiator write ErrAck error"
             * ErrStatusPtr->PciWriteAddr contains the error address
             */
        }
        if (ErrStatusPtr->PciReason & XPCI_PIADDR_RETRY_W)
        {
            /*
             * Handle "PCI initiator write retry error"
             * ErrStatusPtr->PciWriteAddr contains the error address
             */
        }
        if (ErrStatusPtr->PciReason & XPCI_PIADDR_TIMEOUT_W)
        {
            /*
             * Handle "PCI initiator write timeout error"
             * ErrStatusPtr->PciWriteAddr contains the error address
             */
        }
        if (ErrStatusPtr->PciReason & XPCI_PIADDR_RANGE_W)
        {
            /*
             * Handle "PCI initiator write range error"
             * ErrStatusPtr->PciWriteAddr contains the error address
             */
        }

    }

    /* PCI system error? */
    if (ErrStatusPtr->PciSerrReason)
    {
        if (ErrStatusPtr->PciSerrReason & XPCI_IR_PI_SERR_R)
        {
            /*
             * Handle "PCI initiator read system error"
             * ErrStatusPtr->PciSerrReadAddr contains the error address
             */
        }
        if (ErrStatusPtr->PciSerrReason & XPCI_IR_PI_SERR_W)
        {
            /*
             * Handle "PCI initiator write system error"
             * ErrStatusPtr->PciSerrWriteAddr contains the error address
             */
        }
    }
}


/******************************************************************************
 * SemaphoreGive - simple binary semaphore give
 *
 * Parameters:
 *   SemPtr - points to a semaphore object
 *
 * Returns:
 *   nothing
 *
 *****************************************************************************/
void SemaphoreGive(Xuint8 *SemPtr)
{
    *SemPtr = 1;
}


/******************************************************************************
 * SemaphoreTake - simple binary semaphore take
 *
 * Parameters:
 *   sPtr - points to a semaphore object
 *
 * Returns:
 *   nothing
 *
 *****************************************************************************/
void SemaphoreTake(Xuint8 *SemPtr)
{
    while(*SemPtr == 0);
    SemPtr = 0;
}

/****************************************************************************
*
* SetupInterruptSystem - setup system for interrupt driven operation
*
* This function setups the interrupt system so that external interrupts are
* trapped and handled by XIntc. 
*
* This function is application-specific since the actual system may or may
* not have an interrupt controller. The EMAC could be directly connected to
* a processor without an interrupt controller. The user should modify this
* function to fit the application.
*
****************************************************************************/
XStatus SetupInterruptSystem()
{
    XStatus Result;
    
    /*
     * Initialize the interrupt controller driver so that it's ready to use.
     * specify the device ID that was generated in xparameters.h 
     */
    Result = XIntc_Initialize(&xintc, INT_CTRL_DEVICE_ID);
    if (Result != XST_SUCCESS)
    {
        return(Result);
    }

    /*
     * Initialize the PPC405 exception table
     */
    XExc_Init();

    /*
     * Register the interrupt controller handler with the exception table
     */
    XExc_RegisterHandler(XEXC_ID_NON_CRITICAL_INT,
                         (XExceptionHandler)XIntc_InterruptHandler,&xintc);

    /*
     * Enable non-critical exceptions
     */
    XExc_mEnableExceptions(XEXC_NON_CRITICAL);

    /*
     * Start the interrupt controller. PCI interrupts will be enabled
     * in PciInitLevel_1()
     */
    Result = XIntc_Start(&xintc, XIN_REAL_MODE);
    if (Result != XST_SUCCESS)
    {
        return Result;
    }
    
    return XST_SUCCESS;
}


/****************************************************************************
 *
 * main - entry point for this example
 *
 * Returns:
 *   nothing
 *
 ****************************************************************************/
int main(int argc, char **argv)
{
    int i;
    
    /* init interrupt subsystem */
    SetupInterruptSystem();
    
    /* init pci */
    PciInitLevel_1();

    /* prepare local buffer with data pattern [0, 1, 2, ..., N] */
    for (i=0; i<DMA_TRANSFER_LONGS; i++)
    {
        LocalBuffer[i] = i;
    }

    /* dma transfer to pci bus */
    printf("DMA %d bytes from local address %08X to pci   address %08X\n",
           DMA_TRANSFER_BYTES, (int)&LocalBuffer[0], REMOTE_BUFFER_PCI_ADDR);
    
    PciDmaTransfer((Xuint32)&LocalBuffer[0], REMOTE_BUFFER_ADDR,
                   DMA_TRANSFER_BYTES, 0);
    
    /* dma transfer from pci bus */
    printf("DMA %d bytes from pci   address %08X to local address %08X\n",
           DMA_TRANSFER_BYTES, REMOTE_BUFFER_PCI_ADDR, (int)&LocalBuffer[0]);
    
    PciDmaTransfer((Xuint32)&LocalBuffer[0], REMOTE_BUFFER_ADDR,
                   DMA_TRANSFER_BYTES, 1);

    printf("DMA operation complete\n");
    return(0);
}
