/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2002-2003 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file xpci.h
*
* This file contains the software API definition of the Xilinx PCI bridge 
* (XPci) component. This component bridges between local bus IPIF and the
* Xilinx LogiCORE PCI64 Interface v3.0 core. It provides full bridge
* functionality between the local bus a 32 bit V2.2 compliant PCI
* bus.
*
* <b>Features</b>
*
* This driver allows the user to access the device's registers to perform PCI
* configuration read and write access, error detection and processing, and
* interrupt management.
*
* The Xilinx PCI bridge controller is a soft IP core designed for 
* Xilinx FPGAs and contains the following features:
*   - Supports 32 bit OPB local bus
*   - PCI V2.2 Complient
*   - Robust error reporting and diagnostics
*   - DMA capable
*
* <b>Interrupt Management</b>
*
* The XPci component driver provides interrupt management functions.
* Implementation of callback handlers is left to the user. Refer to the provided
* PCI code fragments in the examples directory.
*
* The PCI bridge IP core uses the IPIF to manage interrupts from devices within
* it. Devices in this core include the PCI bridge itself and an optional DMA
* engine. To manage interrupts from these devices, a three layer approach is
* utilized and is modeled on the IPIF.
* 
* Device specific interrupt control is at the lowest layer. This is where 
* individual sources are managed. For example, PCI Master Abort or DMA complete
* interrupts are enabled/disabled/cleared here. The XPci function API that 
* manages this layer is identified as XPci_InterruptPci<operation>(). DMA
* interrupts at this layer are managed by the XDma_Channel software component.
*
* The middle layer is utilized to manage interrupts at a device level. For
* example, enabling PCI interrupts at this layer allows any PCI device specific 
* interrupt enabled at the lowest layer to be passed up to the highest layer. 
* The XPCI function API that manages this layer is identified as
* XPci_Interrupt<operation>().
*
* The middle layer serves little purpose when there is no DMA engine and can
* largely be ignored. During initialization, use XPci_InterruptEnable(...,
* XPCI_IPIF_INT_PCI) to allow all PCI interrupts enabled at the lowest layer
* to pass through. After this operation, the middle layer can be forgotten.
*
* The highest layer is simply a global interrupt enable/disable switch that
* allows all or none of the enabled interrupts to be passed on to an interrupt
* controller. The XPci function API that manages this level is identified as
* XPci_InterruptGlobal<operation>().
*
* <b>DMA</b>
*
* The PCI bridge can include a DMA engine in HW. The XPci software driver can
* be used to query which type of DMA engine has been implemented and manage
* interrupts. The application is required to initialize an XDma_Channel
* component driver and provide an interrupt service routine to service DMA
* exceptions. Example DMA management code is provided in the examples directory.
*
* @note
* This driver is intended to be used to bridge across multiple types of
* buses (PLB or OPB). While the register set will remain the same for all buses,
* their bit definitions may change slightly from bus to bus. The differences
* that arise out of this are clearly documented in this file.
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a rmm  04/15/03 First release
* 1.00b rpm  11/05/03 Changed to use XIo_InSwap32 and XIo_OutSwap32 swap
*                     routines. This corresponds to the opb_pci core change
*                     that preserves byte address through the bridge.
* </pre>
*
*****************************************************************************/
#ifndef XPCI_H  /* prevent circular inclusions */
#define XPCI_H  /* by using protection macros */

/***************************** Include Files ********************************/
#include "xbasic_types.h"
#include "xstatus.h"
#include "xpci_l.h"

/************************** Constant Definitions ****************************/


/*
 * XPCI_CLEAR_ALL_ERRORS is for use with XPci_ErrorClear()
 */
#define XPCI_CLEAR_ALL_ERRORS ((XPciError*)3)


/**************************** Type Definitions ******************************/

/*
 * This typedef contains configuration information for the device.
 */
typedef struct
{
    Xuint16 DeviceId;     /**< Unique ID of device */
    Xuint32 RegBaseAddr;  /**< Register base address */
    Xuint32 DmaBaseAddr;  /**< DMA register base address */
    Xuint32 CfgBar0;      /**< PCI BAR 0 */
    Xuint32 CfgBar1;      /**< PCI BAR 1 */
    Xuint32 CfgBar2;      /**< PCI BAR 2 */
    Xuint8  CfgPrefetch0; /**< prefetchable setting for PCI BAR 0 */
    Xuint8  CfgPrefetch1; /**< prefetchable setting for PCI BAR 1 */
    Xuint8  CfgPrefetch2; /**< prefetchable setting for PCI BAR 2 */
    Xuint8  CfgSpace0;    /**< IO or memory space for PCI BAR 0 */
    Xuint8  CfgSpace1;    /**< IO or memory space for PCI BAR 1 */
    Xuint8  CfgSpace2;    /**< IO or memory space for PCI BAR 2 */
    Xuint8  DmaType;      /**< DMA type  */
} XPci_Config;

/**
 * The XPci driver instance data. The user is required to allocate a
 * variable of this type for every PCI device in the system that will be
 * using this API. A pointer to a variable of this type is passed to the driver
 * API functions defined here.
 */
typedef struct
{
    Xuint32 RegBaseAddr;   /**< Base address of registers */
    Xuint32 DmaRegBaseAddr;/**< Base address of DMA (if included) */
    Xuint32 IsReady;       /**< Device is initialized and ready */
    Xuint8  DmaType;       /**< Type of DMA (if enabled), see XPCI_DMA_TYPE
                                constants in xpci_l.h */
} XPci;

/**
 * XPciError is used to retrieve a snapshot of the bridge's error state.
 * Most of the attributes of this structure are copies of various bridge
 * registers. See XPci_ErrorGet() and XPci_ErrorClear().
 */
typedef struct
{
    Xboolean IsError;           /**< Global error indicator */
    Xuint32 LocalBusReason;     /**< Local bus master address definition */ 
    Xuint32 PciReason;          /**< PCI address definition */
    Xuint32 PciSerrReason;      /**< PCI System error definiton */
    Xuint32 LocalBusReadAddr;   /**< Local bus master read error address */
    Xuint32 LocalBusWriteAddr;  /**< Local bus master write error address */
    Xuint32 PciReadAddr;        /**< PCI read error address */
    Xuint32 PciWriteAddr;       /**< PCI write error address */
    Xuint32 PciSerrReadAddr;    /**< PCI initiater read SERR address */
    Xuint32 PciSerrWriteAddr;   /**< PCI initiater write SERR address */
} XPciError;

/***************** Macros (Inline Functions) Definitions ********************/


/************************** Function Prototypes *****************************/

/*
 * Initialization & raw PCI configuration functions.
 * This API is implemented in xpci.c
 */
XStatus XPci_Initialize(XPci *InstancePtr, Xuint16 DeviceId, int BusNo,
                        int SubBusNo);
void    XPci_Reset(XPci *InstancePtr);
Xuint32 XPci_ConfigPack(unsigned Bus, unsigned Device, unsigned Function);
Xuint32 XPci_ConfigIn(XPci *InstancePtr, Xuint32 ConfigAddress, Xuint8 Offset);
void    XPci_ConfigOut(XPci *InstancePtr, Xuint32 ConfigAddress, Xuint8 Offset,
                       Xuint32 ConfigData);
void    XPci_ErrorGet(XPci *InstancePtr, XPciError *ErrorDataPtr);
void    XPci_ErrorClear(XPci *InstancePtr, XPciError *ErrorDataPtr);
void    XPci_InhibitAfterError(XPci *InstancePtr, Xuint32 Mask);
void    XPci_SetBusNumber(XPci *InstancePtr, int BusNo, int SubBusNo);
void    XPci_GetBusNumber(XPci *InstancePtr, int *BusNoPtr, int *SubBusNoPtr);
void    XPci_GetDmaImplementation(XPci *InstancePtr, Xuint32 *BaseAddr,
                                  Xuint8 *DmaType);
XPci_Config *XPci_LookupConfig(Xuint16 DeviceId);

/*
 * PCI bus configuration functions.
 * This API is implemented in xpci_config.c
 */
XStatus XPci_ConfigIn8(XPci *InstancePtr, unsigned Bus, unsigned Device,
                       unsigned Func, unsigned Offset, Xuint8 *Data);
XStatus XPci_ConfigIn16(XPci *InstancePtr, unsigned Bus, unsigned Device,
                        unsigned Func, unsigned Offset, Xuint16 *Data);
XStatus XPci_ConfigIn32(XPci *InstancePtr, unsigned Bus, unsigned Device,
                        unsigned Func, unsigned Offset, Xuint32 *Data);

XStatus XPci_ConfigOut8(XPci *InstancePtr, unsigned Bus, unsigned Device,
                        unsigned Func, unsigned Offset, Xuint8 Data);
XStatus XPci_ConfigOut16(XPci *InstancePtr, unsigned Bus, unsigned Device,
                         unsigned Func, unsigned Offset, Xuint16 Data);
XStatus XPci_ConfigOut32(XPci *InstancePtr, unsigned Bus, unsigned Device,
                         unsigned Func, unsigned Offset, Xuint32 Data);

/*
 * Interrupt processing and special cycle functions
 * This API is implemented in xpci_intr.c
 */
void    XPci_InterruptGlobalEnable(XPci *InstancePtr);
void    XPci_InterruptGlobalDisable(XPci *InstancePtr);

void    XPci_InterruptEnable(XPci *InstancePtr, Xuint32 Mask);
void    XPci_InterruptDisable(XPci *InstancePtr, Xuint32 Mask);
void    XPci_InterruptClear(XPci *InstancePtr, Xuint32 Mask);
Xuint32 XPci_InterruptGetEnabled(XPci *InstancePtr);
Xuint32 XPci_InterruptGetStatus(XPci *InstancePtr);
Xuint32 XPci_InterruptGetPending(XPci *InstancePtr);
Xuint32 XPci_InterruptGetHighestPending(XPci *InstancePtr);

void    XPci_InterruptPciEnable(XPci *InstancePtr, Xuint32 Mask);
void    XPci_InterruptPciDisable(XPci *InstancePtr, Xuint32 Mask);
void    XPci_InterruptPciClear(XPci *InstancePtr, Xuint32 Mask);
Xuint32 XPci_InterruptPciGetEnabled(XPci *InstancePtr);
Xuint32 XPci_InterruptPciGetStatus(XPci *InstancePtr);

void    XPci_AckSend(XPci *InstancePtr, Xuint32 Vector);
Xuint32 XPci_AckRead(XPci *InstancePtr);
void    XPci_SpecialCycle(XPci *InstancePtr, Xuint32 Data);

/*
 * V3 core access functions
 * This API implementedin xpci_v3.c
 */
Xuint32 XPci_V3StatusCommandGet(XPci *InstancePtr);
Xuint32 XPci_V3TransactionStatusGet(XPci *InstancePtr);
void    XPci_V3TransactionStatusClear(XPci *InstancePtr, Xuint32 Data);

/*
 * Selftest
 * This API is implemented in xpci_selftest.c
 */
XStatus XPci_SelfTest(XPci *InstancePtr);

#endif            /* end of protection macro */
