//----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
// 
//----------------------------------------------------------------------------
// Filename:        my_jtag_logic.v
// 
// Description:
//   
//-----------------------------------------------------------------------------
// Structure:   
// 
//-----------------------------------------------------------------------------
// Author:      KD
// History:
//
//-----------------------------------------------------------------------------
// Naming Conventions:
//      active low signals:                     "*_n"
//      clock signals:                          "clk", "clk_div#", "clk_#x" 
//      reset signals:                          "rst", "rst_n" 
//      generics/parameters:                    "C_*" 
//      user defined types:                     "*_TYPE" 
//      state machine next state:               "*_ns" 
//      state machine current state:            "*_cs" 
//      combinatorial signals:                  "*_com" 
//      pipelined or register delay signals:    "*_d#" 
//      counter signals:                        "*cnt*"
//      clock enable signals:                   "*_ce" 
//      internal version of output port         "*_i"
//      device pins:                            "*_pin" 
//      ports:                                  - Names begin with Uppercase 
//      processes:                              "*_PROCESS" 
//      component instantiations:               "<ENTITY_>I_<#|FUNC>
//-----------------------------------------------------------------------------

`timescale 1ns/1ps

///////////////////////////////////////////////////////////////////////////////
// Module Declaration
///////////////////////////////////////////////////////////////////////////////

module my_jtag_logic (
  // System Signals
  CLK,              // I
  RST,              // I
  // External Debug Signals - Remember to add pullups for these signals in the .UCF file
  CPU_HALT,         // I - Active Low
  CPU_TDO,          // O
  CPU_TDI,          // I
  CPU_TCK,          // I
  CPU_TMS,          // I
  CPU_TRST,         // I - Active Low
  // CPU Debug Signals
  DBGC405DEBUGHALT, // O
  JTGC405TRSTNEG,   // O
  JTGC405TCK,       // O
  JTGC405TDI,       // O
  JTGC405TMS,       // O
  C405JTGTDO,       // I
  C405JTGTDOEN      // I
  );

///////////////////////////////////////////////////////////////////////////////
// Port Declaration
///////////////////////////////////////////////////////////////////////////////

  // System Signals
  input  CLK;
  input  RST;
  // External Debug Signals
  input  CPU_HALT;
  output CPU_TDO;
  input  CPU_TDI;
  input  CPU_TCK;
  input  CPU_TMS;
  input  CPU_TRST;
  // CPU Debug Signals
  output DBGC405DEBUGHALT;
  output JTGC405TRSTNEG;
  output JTGC405TCK;
  output JTGC405TDI;
  output JTGC405TMS;
  input  C405JTGTDO;
  input  C405JTGTDOEN;

///////////////////////////////////////////////////////////////////////////////
// Parameter Declaration
///////////////////////////////////////////////////////////////////////////////

  parameter C_JTAGPPC_ONLY = 0;

///////////////////////////////////////////////////////////////////////////////
// Signal Declaration
///////////////////////////////////////////////////////////////////////////////

  wire TDOTS_o;
  wire TCK_j;
  wire TDI_j;
  wire TMS_j;
  wire TDO_j;
  wire TDOTS_j;
  wire TDITDO_j;
  wire SelTAP_i;
   wire sig_JTAGPPC_ONLY = C_JTAGPPC_ONLY;
   
///////////////////////////////////////////////////////////////////////////////
// Main Body of Code
///////////////////////////////////////////////////////////////////////////////

assign DBGC405DEBUGHALT = ~CPU_HALT;
assign JTGC405TRSTNEG   =  CPU_TRST;

// Don't need a tristate with pullup since this signal is buffered on the
// ML300 board anyways.
assign CPU_TDO = C405JTGTDO | TDOTS_o;
// If tri-stateable TDO is needed something like the following would have
// to be wired up at the top level (not here).
// OBUFT  TDO_obuft   (.I(C405JTGTDO), .O(CPU_TDO), .T(TDOTS_o));

// Instantiate internal JTAG access point from FPGA
JTAGPPC JTAG (
  .TCK      (TCK_j),  // O
  .TDIPPC   (TDI_j),  // O
  .TMS      (TMS_j),  // O
  .TDOPPC   (TDO_j),  // I
  .TDOTSPPC (TDOTS_j) // I
  );

// If PPC JTAG is instaniated, a four stage shift register is inserted
// to compensate for the CPU when it is not in the chain
SRL16 FFS4 (
  .D   (TDI_j),   // I
  .CLK (TCK_j),   // I
  .A3  (1'b0),    // I
  .A2  (1'b0),    // I
  .A1  (1'b1),    // I
  .A0  (1'b1),    // I
  .Q   (TDITDO_j) // O
  );

// SelTAP_i chooses between the processor being in the FPGA's JTAG pin
// or the processor's JTAG directly connected to I/Os
// SelTAP_i = 0 => use JTAGPPC
// SelTAP_I = 1 => use FPGA I/O Pins
assign TDO_j            = (SelTAP_i) ? TDITDO_j      : C405JTGTDO;
assign TDOTS_j          = (SelTAP_i) ? 1'b1          : C405JTGTDOEN;
assign TDOTS_o          = (SelTAP_i) ? ~C405JTGTDOEN : 1'b1;
assign JTGC405TMS       = (SelTAP_i) ? CPU_TMS       : TMS_j;
assign JTGC405TDI       = (SelTAP_i) ? CPU_TDI       : TDI_j;
assign JTGC405TCK       = (SelTAP_i) ? CPU_TCK       : TCK_j;

// This is the auto detect for the JTAG chain.
reg [0:23] tck_idle_cntr;
reg        TCK_j_d1;
reg        TCK_j_d2;
always @(posedge CLK)
begin
  TCK_j_d1 <= TCK_j;
  TCK_j_d2 <= TCK_j_d1;
  if (RST | (TCK_j_d1 & ~TCK_j_d2 & ~tck_idle_cntr[0]))
    tck_idle_cntr <= 24'b0;
  else if (~tck_idle_cntr[0])
    tck_idle_cntr <= tck_idle_cntr + 1;
end

FDCP JTAG_sel_FDP (.Q(SelTAP_i), .C(1'b0), .PRE (~CPU_TCK & tck_idle_cntr[0]), 
                   .CLR(RST | sig_JTAGPPC_ONLY), .D(1'b0));

endmodule

