//-----------------------------------------------------------------------------
// General Purpose I/O (OPB Version) - Module
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
// 
//-----------------------------------------------------------------------------
// Filename:     opb_par_port_ref.v
// 
// Description:    
//   This module instantiates an OPB based Slave SRAM IPIF and
//   the parallel port IP
//
//-----------------------------------------------------------------------------
// Author:      KD
// History:
//   KD        2/27/01 -- FAE IPIF Training Demo
//
//-----------------------------------------------------------------------------
// Naming Conventions:
//      active low signals:                     "*_n"
//      clock signals:                          "clk", "clk_div#", "clk_#x" 
//      reset signals:                          "rst", "rst_n" 
//      generics/parameters:                    "C_*" 
//      user defined types:                     "*_TYPE" 
//      state machine next state:               "*_ns" 
//      state machine current state:            "*_cs" 
//      combinatorial signals:                  "*_com" 
//      pipelined or register delay signals:    "*_d#" 
//      counter signals:                        "*cnt*"
//      clock enable signals:                   "*_ce" 
//      internal version of output port         "*_i"
//      device pins:                            "*_pin" 
//      ports:                                  - Names begin with Uppercase 
//      processes:                              "*_PROCESS" 
//      component instantiations:               "<ENTITY_>I_<#|FUNC>
//-----------------------------------------------------------------------------


///////////////////////////////////////////////////////////////////////////////
// Module Declaration
///////////////////////////////////////////////////////////////////////////////

module opb_par_port_ref (
  // OPB Slave Signals
  OPB_ABus,    // I  [0:31]
  OPB_BE,      // I  [0:3]
  OPB_Clk,     // I
  OPB_DBus,    // I  [0:31]
  OPB_RNW,     // I
  OPB_Rst,     // I
  OPB_select,  // I
  OPB_seqAddr, // I
  Sl_DBus,     // O  [0:31]
  Sl_DBusEn,   // O
  Sl_errAck,   // O
  Sl_retry,    // O
  Sl_toutSup,  // O
  Sl_xferAck,  // O
  // Parallel Port
  PP_INTR,     // I       SPP : nAck
  PP_nWAIT,    // I       SPP : Busy
  PP_USER1,    // I       SPP : Out of Paper or PError
  PP_USER3,    // I       SPP : Select
  PP_DATA,     // O [7:0] SPP : Data
  PP_DIR,      // O       Transceiver Chip Direction
  PP_HD,       // O       Transceiver Chip High Drive
  PP_nASTRB,   // O       SPP : nSelect
  PP_nDSTRB,   // O       SPP : nAutoFeed
  PP_nINIT,    // O       SPP : nInitialize 
  PP_nWRITE,   // O       SPP : nStrobe
  PP_USER2     // I       SPP : nError
  );

///////////////////////////////////////////////////////////////////////////////
// Port Declarations
///////////////////////////////////////////////////////////////////////////////

  // OPB Slave Signals
  input  [0:31] OPB_ABus;
  input  [0:3]  OPB_BE;
  input         OPB_Clk;
  input  [0:31] OPB_DBus;
  input         OPB_RNW;
  input         OPB_Rst;
  input         OPB_select;
  input         OPB_seqAddr;
  output [0:31] Sl_DBus;
  output        Sl_DBusEn;
  output        Sl_errAck;
  output        Sl_retry;
  output        Sl_toutSup;
  output        Sl_xferAck;
  // Parallel Port Signals
  input         PP_INTR;
  input         PP_nWAIT;
  input         PP_USER1;
  input         PP_USER3;
  output [7:0]  PP_DATA;
  output        PP_DIR;
  output        PP_HD;
  output        PP_nASTRB;
  output        PP_nDSTRB;
  output        PP_nINIT;
  output        PP_nWRITE;
  input         PP_USER2;

///////////////////////////////////////////////////////////////////////////////
// Parameter Declarations
///////////////////////////////////////////////////////////////////////////////

  parameter     C_BASEADDR = 32'h0000_0000;
  parameter     C_HIGHADDR = 32'h0000_0000;


///////////////////////////////////////////////////////////////////////////////
// Signal Declarations
///////////////////////////////////////////////////////////////////////////////

  wire   [0:31] Bus2IP_Addr;
  wire   [0:3]  Bus2IP_BE;
  wire          Bus2IP_Clk;
  wire   [0:31] Bus2IP_Data;
  wire          Bus2IP_RdReq;
  wire          Bus2IP_Reset;
  wire          Bus2IP_SRAM_CE;
  wire          Bus2IP_WrReq;
  wire   [0:31] IP2Bus_Data;
  wire          IP2Bus_Error;
  wire          IP2Bus_RdAck;
  wire          IP2Bus_Retry;
  wire          IP2Bus_ToutSup;
  wire          IP2Bus_WrAck;


///////////////////////////////////////////////////////////////////////////////
// Module Instantiations
///////////////////////////////////////////////////////////////////////////////

// Instantiate IPIF Slave SRAM with a 256 Byte Address Decode Range
opb_ipif_slv_sram #(C_BASEADDR, 32'hFFFF_FF00) opb_ipif_slv_sram (
  // OPB Slave Signals
  .OPB_ABus       (OPB_ABus),       // I [0:31]
  .OPB_BE         (OPB_BE),         // I [0:3]
  .OPB_Clk        (OPB_Clk),        // I
  .OPB_DBus       (OPB_DBus),       // I [0:31]
  .OPB_RNW        (OPB_RNW),        // I
  .OPB_Rst        (OPB_Rst),        // I
  .OPB_select     (OPB_select),     // I
  .OPB_seqAddr    (OPB_seqAddr),    // I
  .Sl_DBus        (Sl_DBus),        // O [0:31]
  .Sl_DBusEn      (Sl_DBusEn),      // O
  .Sl_errAck      (Sl_errAck),      // O
  .Sl_retry       (Sl_retry),       // O
  .Sl_toutSup     (Sl_toutSup),     // O
  .Sl_xferAck     (Sl_xferAck),     // O
  // IP Related Signals
  .Bus2IP_Addr    (Bus2IP_Addr),    // O [0:31]
  .Bus2IP_BE      (Bus2IP_BE),      // O [0:3]
  .Bus2IP_Clk     (Bus2IP_Clk),     // O
  .Bus2IP_Data    (Bus2IP_Data),    // O [0:31]
  .Bus2IP_RdReq   (Bus2IP_RdReq),   // O
  .Bus2IP_Reset   (Bus2IP_Reset),   // O
  .Bus2IP_SRAM_CE (Bus2IP_SRAM_CE), // O
  .Bus2IP_WrReq   (Bus2IP_WrReq),   // O
  .IP2Bus_Data    (IP2Bus_Data),    // I [0:31]
  .IP2Bus_Error   (IP2Bus_Error),   // I
  .IP2Bus_RdAck   (IP2Bus_RdAck),   // I
  .IP2Bus_Retry   (IP2Bus_Retry),   // I
  .IP2Bus_ToutSup (IP2Bus_ToutSup), // I
  .IP2Bus_WrAck   (IP2Bus_WrAck)    // I
  );

// Instantiate User Design with IPIF interface
par_port_ipif par_port_ipif (
  // IPIF Signals
  .Bus2IP_Addr    (Bus2IP_Addr),    // I  [0:31]
  .Bus2IP_BE      (Bus2IP_BE),      // I  [0:3]
  .Bus2IP_Clk     (Bus2IP_Clk),     // I
  .Bus2IP_Data    (Bus2IP_Data),    // I  [0:31]
  .Bus2IP_RdReq   (Bus2IP_RdReq),   // I
  .Bus2IP_Reset   (Bus2IP_Reset),   // I
  .Bus2IP_SRAM_CE (Bus2IP_SRAM_CE), // I
  .Bus2IP_WrReq   (Bus2IP_WrReq),   // I
  .IP2Bus_Data    (IP2Bus_Data),    // O  [0:31]
  .IP2Bus_Error   (IP2Bus_Error),   // O
  .IP2Bus_RdAck   (IP2Bus_RdAck),   // O
  .IP2Bus_Retry   (IP2Bus_Retry),   // O
  .IP2Bus_ToutSup (IP2Bus_ToutSup), // O
  .IP2Bus_WrAck   (IP2Bus_WrAck),   // O
  // Parallel Port Signals
  .PP_INTR        (PP_INTR),        // I
  .PP_nWAIT       (PP_nWAIT),       // I
  .PP_USER1       (PP_USER1),       // I
  .PP_USER3       (PP_USER3),       // I
  .PP_DATA        (PP_DATA),        // O [7:0]
  .PP_DIR         (PP_DIR),         // O
  .PP_HD          (PP_HD),          // O
  .PP_nASTRB      (PP_nASTRB),      // O 
  .PP_nDSTRB      (PP_nDSTRB),      // O
  .PP_nINIT       (PP_nINIT),       // O
  .PP_nWRITE      (PP_nWRITE),      // O
  .PP_USER2       (PP_USER2)        // I
  );

endmodule

