//----------------------------------------------------------------------------
// OPB Master IPIF - Module
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
// 
//----------------------------------------------------------------------------
// Filename:     opb_ipif_slv_sram.v
// 
// Description:    
//   This module implements an IPIF with a master interface
//
// Design Notes:
//   This design does not perform bursts currently
//
//-----------------------------------------------------------------------------
// Structure:   
// 
//            -- opb_ipif_master.v
//
//-----------------------------------------------------------------------------
// Author:      KD
// History:
//   KD       10/18/02 -- First Version
//
//-----------------------------------------------------------------------------
// Naming Conventions:
//      active low signals:                     "*_n"
//      clock signals:                          "clk", "clk_div#", "clk_#x" 
//      reset signals:                          "rst", "rst_n" 
//      generics/parameters:                    "C_*" 
//      user defined types:                     "*_TYPE" 
//      state machine next state:               "*_ns" 
//      state machine current state:            "*_cs" 
//      combinatorial signals:                  "*_com" 
//      pipelined or register delay signals:    "*_d#" 
//      counter signals:                        "*cnt*"
//      clock enable signals:                   "*_ce" 
//      internal version of output port         "*_i"
//      device pins:                            "*_pin" 
//      ports:                                  - Names begin with Uppercase 
//      processes:                              "*_PROCESS" 
//      component instantiations:               "<ENTITY_>I_<#|FUNC>
//-----------------------------------------------------------------------------


///////////////////////////////////////////////////////////////////////////////
// Module Declaration
///////////////////////////////////////////////////////////////////////////////

module opb_ipif_master (
  // OPB Master Interface
  M_ABus,          // O [0:31]
  M_BE,            // O [0:3]
  M_busLock,       // O
  M_DBus,          // O[0:31]
  M_DBusEn,        // O
  M_request,       // O
  M_RNW,           // O
  M_select,        // O
  M_seqAddr,       // O
  OPB_Clk,         // I
  OPB_DBus,        // I [0:31]
  OPB_errAck,      // I
  OPB_MnGrant,     // I
  OPB_retry,       // I
  OPB_Rst,         // I
  OPB_timeout,     // I
  OPB_xferAck,     // I
  // IP Related Signal
  Bus2IP_Clk,      // O
  Bus2IP_Data,     // O [0:31]
  Bus2IP_Error,    // O
  Bus2IP_MstRdAck, // O
  Bus2IP_MstWrAck, // O
  Bus2IP_Reset,    // O
  Bus2IP_Retry,    // O
  Bus2IP_TimeOut,  // O
  IP2Bus_Addr,     // I [0:31]
  IP2Bus_BusLock,  // I
  IP2Bus_Data,     // I [0:31]
  IP2Bus_MstRdReq, // I
  IP2Bus_MstWrReq, // I
  IP2Bus_MstBE     // I [0:3]
  );
   

///////////////////////////////////////////////////////////////////////////////
// Port Declarations
///////////////////////////////////////////////////////////////////////////////

  // OPB Master Interface
  output [0:31] M_ABus;
  output [0:3]  M_BE;
  output        M_busLock;
  output [0:31] M_DBus;
  output        M_DBusEn;
  output        M_request;
  output        M_RNW;
  output        M_select;
  output        M_seqAddr;
  input         OPB_Clk;
  input  [0:31] OPB_DBus;
  input         OPB_errAck;
  input         OPB_MnGrant;
  input         OPB_retry;
  input         OPB_Rst;
  input         OPB_timeout;
  input         OPB_xferAck;
  // IP Related Signal
  output        Bus2IP_Clk;
  output [0:31] Bus2IP_Data;
  output        Bus2IP_Error;
  output        Bus2IP_MstRdAck;
  output        Bus2IP_MstWrAck;
  output        Bus2IP_Reset;
  output        Bus2IP_Retry;
  output        Bus2IP_TimeOut;
  input  [0:31] IP2Bus_Addr;
  input         IP2Bus_BusLock;
  input  [0:31] IP2Bus_Data;
  input         IP2Bus_MstRdReq;
  input         IP2Bus_MstWrReq;
  input [0:3]   IP2Bus_MstBE;


///////////////////////////////////////////////////////////////////////////////
// Signal Declarations
///////////////////////////////////////////////////////////////////////////////

  reg  [0:31] Bus2IP_Data;
  reg         Bus2IP_Error;
  reg         Bus2IP_MstRdAck;
  reg         Bus2IP_MstWrAck;
  reg         Bus2IP_Retry;
  reg         Bus2IP_TimeOut;
  reg  [0:31] M_ABus;
  reg  [0:3]  M_BE;
  reg         M_busLock;
  reg  [0:31] M_DBus;
  reg         M_request;
  reg         M_RNW;
  reg         M_select;
  wire        m_select_i;
  reg         rd_xfer_on;
  reg         wr_xfer_on;
  wire        xfer_done;

///////////////////////////////////////////////////////////////////////////////
// Main Body of Code
///////////////////////////////////////////////////////////////////////////////

assign Bus2IP_Clk   = OPB_Clk;
assign Bus2IP_Reset = OPB_Rst;
assign M_DBusEn     = 1'b1;
assign M_seqAddr    = 1'b0;

always @(posedge OPB_Clk)
  if (OPB_Rst | !m_select_i) begin
    M_ABus    <= 32'b0;
    M_BE      <= 4'b0;
    M_busLock <= 1'b0;
    M_RNW     <= 1'b0;
  end
  else begin
    M_ABus[0:29]  <= IP2Bus_Addr[0:29];
    M_ABus[30:31] <= IP2Bus_MstBE[0]? 2'b00 :
                     IP2Bus_MstBE[1]? 2'b01 :
                     IP2Bus_MstBE[2]? 2'b10 :
                                      2'b11;
    M_BE          <= IP2Bus_MstBE;
    M_busLock     <= IP2Bus_BusLock;
    M_RNW         <= rd_xfer_on | IP2Bus_MstRdReq;
  end

always @(posedge OPB_Clk)
  if (OPB_Rst | !m_select_i | rd_xfer_on)
    M_DBus    <= 32'b0;
  else
    M_DBus    <= IP2Bus_Data;

always @(posedge OPB_Clk)
begin
  M_select        <= m_select_i & ~OPB_Rst;
  Bus2IP_Data     <= OPB_DBus;
  Bus2IP_Error    <= M_select & OPB_errAck;
  Bus2IP_MstRdAck <= M_select &  M_RNW & OPB_xferAck;
  Bus2IP_MstWrAck <= M_select & ~M_RNW & OPB_xferAck;
  Bus2IP_Retry    <= M_select & OPB_retry;
  Bus2IP_TimeOut  <= M_select & OPB_timeout & ~OPB_xferAck;
end

assign xfer_done = M_select & (OPB_xferAck | OPB_retry | OPB_timeout);

always @(posedge OPB_Clk)
  if (OPB_Rst | xfer_done)
    rd_xfer_on <= 1'b0;
  else if (IP2Bus_MstRdReq)
    rd_xfer_on <= 1'b1;

always @(posedge OPB_Clk)
  if (OPB_Rst | xfer_done)
    wr_xfer_on <= 1'b0;
  else if (IP2Bus_MstWrReq)
    wr_xfer_on <= 1'b1;

always @(posedge OPB_Clk)
  M_request <= (IP2Bus_MstWrReq | IP2Bus_MstRdReq | rd_xfer_on | wr_xfer_on)
             & ~M_select;

assign m_select_i = ((M_request & OPB_MnGrant) | M_select) & ~xfer_done;

endmodule
