//-----------------------------------------------------------------------------
// OPB-to-PCI Bridge Top Level - Module
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
// 
//-----------------------------------------------------------------------------
// Filename:     opb_pci_brg_top.v
// 
// Description:    
//   This module instantiates an OPB based IPIF, PCI Core and an
//   IPIF-to-PCI module to build a bridge from OPB to PCI. 
//
// Design Notes:
//
//   The PCI interface generates a 32 bit non-burst PCI transaction for
//   each OPB transfer. It can generate PCI Configuration, I/O, and
//   memory transactions depending on the memory space the request
//   occurs in. For future upgradeability, the PCI Core is a 64 bit version
//   even though only 32 bit transactions are used. Note that the user must
//   perform a self-configuration cycle to enable the PCI master port on the
//   bridge before beginning any external PCI transactions.
//
//-----------------------------------------------------------------------------
// Structure:   
// 
//            -- opb_pci_brg_top.v
//                -- pcim_lc_33_3_s.v
//                    -- pci_lc_i.v (PCI Core)
//                -- pci_cfg.v
//                -- pci_brg_ipif.v
//                -- opb_ipif_slv_sram.v
//
//-----------------------------------------------------------------------------
// Author:      KD
// History:
//   KD        5/18/01 -- EA 2 Release
//
//-----------------------------------------------------------------------------
// Naming Conventions:
//      active low signals:                     "*_n"
//      clock signals:                          "clk", "clk_div#", "clk_#x" 
//      reset signals:                          "rst", "rst_n" 
//      generics/parameters:                    "C_*" 
//      user defined types:                     "*_TYPE" 
//      state machine next state:               "*_ns" 
//      state machine current state:            "*_cs" 
//      combinatorial signals:                  "*_com" 
//      pipelined or register delay signals:    "*_d#" 
//      counter signals:                        "*cnt*"
//      clock enable signals:                   "*_ce" 
//      internal version of output port         "*_i"
//      device pins:                            "*_pin" 
//      ports:                                  - Names begin with Uppercase 
//      processes:                              "*_PROCESS" 
//      component instantiations:               "<ENTITY_>I_<#|FUNC>
//-----------------------------------------------------------------------------

`timescale 1ns/1ps

///////////////////////////////////////////////////////////////////////////////
// Module Declaration
///////////////////////////////////////////////////////////////////////////////

module opb_pci_ref (
  FRAMEQ_N,    // O
  IRDYQ_N,     // O
  // OPB Slave Signals
  OPB_ABus,    // I [0:31]
  OPB_BE,      // I [0:3]
  OPB_Clk,     // I
  OPB_DBus,    // I [0:31]
  OPB_RNW,     // I
  OPB_Rst,     // I
  OPB_select,  // I
  OPB_seqAddr, // I
  Sl_DBus,     // O [0:31]
  Sl_errAck,   // O
  Sl_retry,    // O
  Sl_toutSup,  // O
  Sl_xferAck,  // O
  // OPB Master Signals
  OPB_errAck,  // I
  OPB_MnGrant, // I
  OPB_retry,   // I
  OPB_timeout, // I
  OPB_xferAck, // I
  M_ABus,      // O [0:31]
  M_BE,        // O [0:3]
  M_busLock,   // O
  M_request,   // O
  M_RNW,       // O
  M_select,    // O
  M_seqAddr,   // O
  // PCI Signals
  GNT_N,       // I
  // INTR_A    // O generate interrupt internally (intr_out)
  PCLK_IN,     // I
  REQ_N,       // O
  RST_N,       // O
  ACK64_N,     // IO
  AD,          // IO [31:0]  zero others internlly [63:0]
  CBE,         // IO [3:0]  zero others internlly [7:0]
  DEVSEL_N,    // IO
  FRAME_N,     // IO
  IDSEL,       // IO
  IRDY_N,      // IO
  PAR,         // IO
  PERR_N,      // IO
  REQ64_N,     // IO
  SERR_N,      // IO
  STOP_N,      // IO
  TRDY_N,      // IO
// ML300 specific signals
  INTA,        // I
  INTB,        // I
  INTC,        // I
  INTD,        // I
  INTR_OUT,    // O
  GRST_N       // O
  );
  // synthesis syn_edif_bit_format = "%u<%i>"
  // synthesis syn_edif_scalar_format = "%u"


///////////////////////////////////////////////////////////////////////////////
// Port Declarations
///////////////////////////////////////////////////////////////////////////////

  // Signals To PCI Arbiter
  output        FRAMEQ_N;
  output        IRDYQ_N;
  // OPB Slave Signals
  input  [0:31] OPB_ABus;
  input  [0:3]  OPB_BE;
  input         OPB_Clk;
  input  [0:31] OPB_DBus;
  input         OPB_RNW;
  input         OPB_Rst;
  input         OPB_select;
  input         OPB_seqAddr;
  output [0:31] Sl_DBus;
  output        Sl_errAck;
  output        Sl_retry;
  output        Sl_toutSup;
  output        Sl_xferAck;
  // OPB Master Signals
  input         OPB_errAck;
  input         OPB_MnGrant;
  input         OPB_retry;
  input         OPB_timeout;
  input         OPB_xferAck;
  output [0:31] M_ABus;
  output [0:3]  M_BE;
  output        M_busLock;
  output        M_request;
  output        M_RNW;
  output        M_select;
  output        M_seqAddr;
  // PCI Signals
  input         GNT_N;
  input         PCLK_IN;
//  output        INTR_A; internal
  output        REQ_N;
  output        RST_N;
  inout         ACK64_N;
  inout  [31:0] AD; // [63:32] internal
  inout   [3:0] CBE; // [7:4] internal
  inout         DEVSEL_N;
  inout         FRAME_N;
  inout         IDSEL;
  inout         IRDY_N;
  inout         PAR;
  inout         PERR_N;
  inout         REQ64_N;
  inout         SERR_N;
  inout         STOP_N;
  inout         TRDY_N;
// ML300 specific signals
  input         INTA;
  input         INTB;
  input         INTC;
  input         INTD;
  output        INTR_OUT;
  output        GRST_N;

///////////////////////////////////////////////////////////////////////////////
// Parameter Declarations
///////////////////////////////////////////////////////////////////////////////

  parameter     C_BASEADDR = 32'h2000_0000;
  parameter     C_HIGHADDR = 32'h3FFF_FFFF;


///////////////////////////////////////////////////////////////////////////////
// Signal Declarations
///////////////////////////////////////////////////////////////////////////////

  wire  [0:31] Sl_DBus_i;
  wire  [0:31] M_DBus;
  wire         M_DBusEn;
  wire         Sl_DBusEn;

  wire   [31:0] ADDR;
  wire   [63:0] ADIO;
  wire    [7:0] BASE_HIT;
  wire  [255:0] CFG;
  wire          CFG_SELF;
  wire   [39:0] CSR;
  wire    [7:0] M_CBE;
  wire   [15:0] PCI_CMD;
  wire    [7:0] S_CBE;
  wire   [31:0] SUB_DATA;

  // IPIF Slave Signals and Signals used for Clock Synchronization
  wire   [0:31] Bus2IP_Addr;
  wire   [0:3]  Bus2IP_BE;
  wire          Bus2IP_Clk;
  wire   [0:31] Bus2IP_Data;
  wire          Bus2IP_RdReq;
  reg           Bus2IP_RdReq_hold;
  reg           Bus2IP_RdReq_hold_d1;
  reg           Bus2IP_RdReq_hold_d2;
  wire          Bus2IP_Reset;
  wire          Bus2IP_SRAM_CE;
  wire          Bus2IP_WrReq;
  reg           Bus2IP_WrReq_hold;
  reg           Bus2IP_WrReq_hold_d1;
  reg           Bus2IP_WrReq_hold_d2;
  wire   [0:31] BusSync2IP_Addr;
  wire   [0:3]  BusSync2IP_BE;
  wire   [0:31] BusSync2IP_Data;
  wire          BusSync2IP_RdReq;
  wire          BusSync2IP_SRAM_CE;
  reg           BusSync2IP_SRAM_CE_i;
  wire          BusSync2IP_WrReq;
  reg    [0:31] IP2Bus_Data;
  reg           IP2Bus_Error;
  wire          IP2Bus_RdAck;
  wire          IP2Bus_Retry;
  reg           IP2Bus_ToutSup;
  wire          IP2Bus_WrAck;
  wire   [0:31] IP2BusSync_Data;
  wire          IP2BusSync_Error;
  wire          IP2BusSync_RdAck;
  reg           IP2BusSync_RdAck_d1;
  reg           IP2BusSync_RdAck_d2;
  reg           IP2BusSync_RdAck_d3;
  wire          IP2BusSync_Retry;
  reg           IP2BusSync_Retry_d1;
  reg           IP2BusSync_Retry_d2;
  reg           IP2BusSync_Retry_d3;
  wire          IP2BusSync_WrAck;
  reg           IP2BusSync_WrAck_d1;
  reg           IP2BusSync_WrAck_d2;
  reg           IP2BusSync_WrAck_d3;

  // IPIF Master Signals and Signals used for Clock Synchronization
  wire          Bus2IP_Error;
  reg           Bus2IP_Error_hold;
  wire   [0:31] Bus2IP_MstData;
  wire          Bus2IP_MstRdAck;
  reg           Bus2IP_MstRdAck_hold;
  wire          Bus2IP_MstWrAck;
  reg           Bus2IP_MstWrAck_hold;
  wire          Bus2IP_Retry;
  reg           Bus2IP_Retry_hold;
  wire          Bus2IP_TimeOut;
  reg           Bus2IP_TimeOut_hold;
  reg           BusSync2IP_Error;
  reg    [0:31] BusSync2IP_MstData;
  reg           BusSync2IP_MstRdAck;
  reg           BusSync2IP_MstWrAck;
  reg           BusSync2IP_Retry;
  reg           BusSync2IP_TimeOut;
  wire   [0:31] IP2Bus_Addr;
  wire          IP2Bus_BusLock;
  wire          IP2Bus_MstRdReq;
  reg           IP2Bus_MstRdReq_d1;
  reg           IP2Bus_MstRdReq_d2;
  wire          IP2Bus_MstWrReq;
  reg           IP2Bus_MstWrReq_d1;
  reg           IP2Bus_MstWrReq_d2;
  wire   [0:31] IP2Bus_MstData;
  wire   [0:3]  IP2Bus_MstBE;
  wire          IP2BusSync_MstRdReq;
  wire          IP2BusSync_MstWrReq;

// from top_ip.v or ip_wrapper.v
   reg          inta_d1;
   reg          intb_d1;
   reg          intc_d1;
   reg          intd_d1;
   reg          INTR_OUT_d1;
   wire         INTR_A;
   wire [3:0]   nc_cbe;
   wire [31:0]  nc_ad;
   wire         rst_n_i;     
   reg  [14:0]  rst_counter;
   reg          RST_N;

///////////////////////////////////////////////////////////////////////////////
// Instantiate 64 Bit, 33 MHz Xilinx PCI Core and PCI I/O Bufs
///////////////////////////////////////////////////////////////////////////////

pcim_lc PCI_CORE (
  .ACK64Q_N    (ACK64Q_N),    // O
  .ACK64_IO    (ACK64_N),     // IO
  .ADDR        (ADDR),        // O  [31:0]
  .ADDR_VLD    (ADDR_VLD),    // O
  .ADIO        (ADIO),        // IO [63:0]
  .AD_IO       ({nc_ad, AD[31:0]}),          // IO [63:0]
  .BACKOFF     (BACKOFF),     // O
  .BASE_HIT    (BASE_HIT),    // O  [7:0]
  .B_BUSY      (B_BUSY),      // O
  .CBE_IO      ({nc_cbe, CBE[3:0]}),         // IO [7:0]
  .CFG         (CFG),         // I  [255:0]
  .CFG_HIT     (CFG_HIT),     // O
  .CFG_SELF    (CFG_SELF),    // I
  .CFG_VLD     (CFG_VLD),     // O
  .CLK         (PCLK_IN),     // I
  .COMPLETE    (COMPLETE),    // I
  .CSR         (CSR),         // O  [39:0]
  .C_READY     (C_READY),     // I
  .C_TERM      (C_TERM),      // I
  .DEVSELQ_N   (DEVSELQ_N),   // O
  .DEVSEL_IO   (DEVSEL_N),    // IO
  .DR_BUS      (DR_BUS),      // O
  .FRAMEQ_N    (FRAMEQ_N),    // O
  .FRAME_IO    (FRAME_N),     // IO
  .GNT_I       (GNT_N),       // I
  .IDLE        (IDLE),        // O
  .IDSEL_IO    (IDSEL),       // IO
  .INTA_O      (INTR_A),      // O
  .INTR_N      (INTR_N),      // I
  .IRDYQ_N     (IRDYQ_N),     // O
  .IRDY_IO     (IRDY_N),      // IO
  .I_IDLE      (I_IDLE),      // O
  .KEEPOUT     (KEEPOUT),     // I
  .M_ADDR_N    (M_ADDR_N),    // O
  .M_CBE       (M_CBE),       // I  [7:0]
  .M_DATA      (M_DATA),      // O
  .M_DATA_VLD  (M_DATA_VLD),  // O
  .M_FAIL64    (M_FAIL64),    // O
  .M_READY     (M_READY),     // I
  .M_SRC_EN    (M_SRC_EN),    // O
  .M_WRDN      (M_WRDN),      // I
  .PAR64_IO    (PAR64),       // IO
  .PAR_IO      (PAR),         // IO
  .PCI_CMD     (PCI_CMD),     // O  [15:0]
  .PERRQ_N     (PERRQ_N),     // O
  .PERR_IO     (PERR_N),      // IO
  .REQ64Q_N    (REQ64Q_N),    // O
  .REQ64_IO    (REQ64_N),     // IO
  .REQUEST     (REQUEST),     // I
  .REQUEST64   (REQUEST64),   // I
  .REQUESTHOLD (REQUESTHOLD), // I
  .REQ_O       (REQ_N),       // O
  .RST         (Bus2IP_Reset),// I
  .RST_O       (rst_n_i),       // O
  .SERRQ_N     (SERRQ_N),     // O
  .SERR_IO     (SERR_N),      // IO
  .SLOT64      (SLOT64),      // I
  .STOPQ_N     (STOPQ_N),     // O
  .STOP_IO     (STOP_N),      // IO
  .SUB_DATA    (SUB_DATA),    // I  [31:0]
  .S_ABORT     (S_ABORT),     // I
  .S_CBE       (S_CBE),       // O  [7:0]
  .S_CYCLE64   (S_CYCLE64),   // O
  .S_DATA      (S_DATA),      // O
  .S_DATA_VLD  (S_DATA_VLD),  // O
  .S_READY     (S_READY),     // I
  .S_SRC_EN    (S_SRC_EN),    // O
  .S_TERM      (S_TERM),      // I
  .S_WRDN      (S_WRDN),      // O
  .TIME_OUT    (TIME_OUT),    // O
  .TRDYQ_N     (TRDYQ_N),     // O
  .TRDY_IO     (TRDY_N)       // IO
  );


///////////////////////////////////////////////////////////////////////////////
// Instantiation of the PCI configuration module
///////////////////////////////////////////////////////////////////////////////

cfg CFG_INST (
  .CFG (CFG)    // O [255:0]
  );


///////////////////////////////////////////////////////////////////////////////
// Instantiation OPB IPIF Module (Slave SRAM Interface)
///////////////////////////////////////////////////////////////////////////////

assign Sl_DBus = Sl_DBus_i | M_DBus;
   
opb_ipif_slv_sram #(C_BASEADDR, 32'hE000_0000) opb_ipif_slv_sram (
  // OPB Slave Signals
  .OPB_ABus       (OPB_ABus),       // I [0:31]
  .OPB_BE         (OPB_BE),         // I [0:3]
  .OPB_Clk        (OPB_Clk),        // I
  .OPB_DBus       (OPB_DBus),       // I [0:31]
  .OPB_RNW        (OPB_RNW),        // I
  .OPB_Rst        (OPB_Rst),        // I
  .OPB_select     (OPB_select),     // I
  .OPB_seqAddr    (OPB_seqAddr),    // I
  .Sl_DBus        (Sl_DBus_i),      // O [0:31]
  .Sl_DBusEn      (Sl_DBusEn),      // O
  .Sl_errAck      (Sl_errAck),      // O
  .Sl_retry       (Sl_retry),       // O
  .Sl_toutSup     (Sl_toutSup),     // O
  .Sl_xferAck     (Sl_xferAck),     // O
  // IP Related Signals
  .Bus2IP_Addr    (Bus2IP_Addr),    // O [0:31]
  .Bus2IP_BE      (Bus2IP_BE),      // O [0:3]
  .Bus2IP_Clk     (Bus2IP_Clk),     // O
  .Bus2IP_Data    (Bus2IP_Data),    // O [0:31]
  .Bus2IP_RdReq   (Bus2IP_RdReq),   // O
  .Bus2IP_Reset   (Bus2IP_Reset),   // O
  .Bus2IP_SRAM_CE (Bus2IP_SRAM_CE), // O
  .Bus2IP_WrReq   (Bus2IP_WrReq),   // O
  .IP2Bus_Data    (IP2Bus_Data),    // I [0:31]
  .IP2Bus_Error   (IP2Bus_Error),   // I
  .IP2Bus_RdAck   (IP2Bus_RdAck),   // I
  .IP2Bus_Retry   (IP2Bus_Retry),   // I
  .IP2Bus_ToutSup (IP2Bus_ToutSup), // I
  .IP2Bus_WrAck   (IP2Bus_WrAck)    // I
  );


///////////////////////////////////////////////////////////////////////////////
// Instantiation OPB IPIF Module (Master Interface)
///////////////////////////////////////////////////////////////////////////////

opb_ipif_master opb_ipif_master (
  // OPB Master Interface
  .M_ABus                (M_ABus),                // O [0:31]
  .M_BE                  (M_BE),                  // O [0:3]
  .M_busLock             (M_busLock),             // O
  .M_DBus                (M_DBus),                // O[0:31]
  .M_DBusEn              (M_DBusEn),              // O
  .M_request             (M_request),             // O
  .M_RNW                 (M_RNW),                 // O
  .M_select              (M_select),              // O
  .M_seqAddr             (M_seqAddr),             // O
  .OPB_Clk               (OPB_Clk),               // I
  .OPB_DBus              (OPB_DBus),              // I [0:31]
  .OPB_errAck            (OPB_errAck),            // I
  .OPB_MnGrant           (OPB_MnGrant),           // I
  .OPB_retry             (OPB_retry),             // I
  .OPB_Rst               (OPB_Rst),               // I
  .OPB_timeout           (OPB_timeout),           // I
  .OPB_xferAck           (OPB_xferAck),           // I
  // IP Related Signal
  .Bus2IP_Clk            (),                      // O
  .Bus2IP_Data           (Bus2IP_MstData),        // O [0:31]
  .Bus2IP_Error          (Bus2IP_Error),          // O
  .Bus2IP_MstRdAck       (Bus2IP_MstRdAck),       // O
  .Bus2IP_MstWrAck       (Bus2IP_MstWrAck),       // O
  .Bus2IP_Reset          (),                      // O
  .Bus2IP_Retry          (Bus2IP_Retry),          // O
  .Bus2IP_TimeOut        (Bus2IP_TimeOut),        // O
  .IP2Bus_Addr           (IP2Bus_Addr),           // I [0:31]
  .IP2Bus_BusLock        (IP2Bus_BusLock),        // I
  .IP2Bus_Data           (IP2Bus_MstData),        // I [0:31]
  .IP2Bus_MstRdReq       (IP2BusSync_MstRdReq),   // I
  .IP2Bus_MstWrReq       (IP2BusSync_MstWrReq),   // I
  .IP2Bus_MstBE          (IP2Bus_MstBE)           // I [0:3]
);

///////////////////////////////////////////////////////////////////////////////
// Instantiate Logic to connect IPIF to PCI Core
///////////////////////////////////////////////////////////////////////////////

pci_brg_ipif pci_brg_ipif (
  // PCI Signals
  .ACK64Q_N       (ACK64Q_N),             // I
  .ADDR           (ADDR),                 // I  [31:0]
  .ADDR_VLD       (ADDR_VLD),             // I
  .ADIO           (ADIO),                 // IO [63:0]
  .BACKOFF        (BACKOFF),              // I
  .BASE_HIT       (BASE_HIT),             // I  [7:0]
  .B_BUSY         (B_BUSY),               // I
  .CFG            (CFG),                  // I  [255:0]
  .CFG_HIT        (CFG_HIT),              // I
  .CFG_SELF       (CFG_SELF),             // O
  .CFG_VLD        (CFG_VLD),              // I
  .COMPLETE       (COMPLETE),             // O
  .CSR            (CSR),                  // I  [39:0]
  .C_READY        (C_READY),              // O
  .C_TERM         (C_TERM),               // O
  .DEVSELQ_N      (DEVSELQ_N),            // I
  .DR_BUS         (DR_BUS),               // I
  .FRAMEQ_N       (FRAMEQ_N),             // I
  .IDLE           (IDLE),                 // I
  .INTR_N         (INTR_N),               // O
  .IRDYQ_N        (IRDYQ_N),              // I
  .I_IDLE         (I_IDLE),               // I
  .KEEPOUT        (KEEPOUT),              // O
  .M_ADDR_N       (M_ADDR_N),             // I
  .M_CBE          (M_CBE),                // O  [7:0]
  .M_DATA         (M_DATA),               // I
  .M_DATA_VLD     (M_DATA_VLD),           // I
  .M_FAIL64       (M_FAIL64),             // I
  .M_READY        (M_READY),              // O
  .M_SRC_EN       (M_SRC_EN),             // I
  .M_WRDN         (M_WRDN),               // O
  .PCI_CMD        (PCI_CMD),              // I  [15:0]
  .PERRQ_N        (PERRQ_N),              // I
  .REQ64Q_N       (REQ64Q_N),             // I
  .REQUEST        (REQUEST),              // O
  .REQUEST64      (REQUEST64),            // O
  .REQUESTHOLD    (REQUESTHOLD),          // O
  .SERRQ_N        (SERRQ_N),              // I
  .SLOT64         (SLOT64),               // O
  .STOPQ_N        (STOPQ_N),              // I
  .SUB_DATA       (SUB_DATA),             // O  [31:0]
  .S_ABORT        (S_ABORT),              // O
  .S_CBE          (S_CBE),                // I  [7:0]
  .S_CYCLE64      (S_CYCLE64),            // I
  .S_DATA         (S_DATA),               // I
  .S_DATA_VLD     (S_DATA_VLD),           // I
  .S_READY        (S_READY),              // O
  .S_SRC_EN       (S_SRC_EN),             // I
  .S_TERM         (S_TERM),               // O
  .S_WRDN         (S_WRDN),               // I
  .TIME_OUT       (TIME_OUT),             // I
  .TRDYQ_N        (TRDYQ_N),              // I
  // IPIF Signals - Slave
  .Bus2IP_Addr    (BusSync2IP_Addr),      // I  [0:31]
  .Bus2IP_BE      (BusSync2IP_BE),        // I  [0:3]
  .Bus2IP_Clk     (PCLK_IN),              // I
  .Bus2IP_Data    (BusSync2IP_Data),      // I  [0:31]
  .Bus2IP_RdReq   (BusSync2IP_RdReq),     // I
  .Bus2IP_Reset   (Bus2IP_Reset),         // I
  .Bus2IP_SRAM_CE (BusSync2IP_SRAM_CE),   // I
  .Bus2IP_WrReq   (BusSync2IP_WrReq),     // I
  .IP2Bus_Data    (IP2BusSync_Data),      // O  [0:31]
  .IP2Bus_Error   (IP2BusSync_Error),     // O
  .IP2Bus_RdAck   (IP2BusSync_RdAck),     // O
  .IP2Bus_Retry   (IP2BusSync_Retry),     // O
  .IP2Bus_ToutSup (IP2BusSync_ToutSup),   // O
  .IP2Bus_WrAck   (IP2BusSync_WrAck),     // O
  // IPIF Signals - Master
  .Bus2IP_MstData  (BusSync2IP_MstData),  // I [0:31]
  .Bus2IP_Error    (BusSync2IP_Error),    // I
  .Bus2IP_MstRdAck (BusSync2IP_MstRdAck), // I
  .Bus2IP_MstWrAck (BusSync2IP_MstWrAck), // I
  .Bus2IP_Retry    (BusSync2IP_Retry),    // I
  .Bus2IP_TimeOut  (BusSync2IP_TimeOut),  // I
  .IP2Bus_Addr     (IP2Bus_Addr),         // O [0:31]
  .IP2Bus_BusLock  (IP2Bus_BusLock),      // O
  .IP2Bus_MstData  (IP2Bus_MstData),      // O [0:31]
  .IP2Bus_MstRdReq (IP2Bus_MstRdReq),     // O
  .IP2Bus_MstWrReq (IP2Bus_MstWrReq),     // O
  .IP2Bus_MstBE    (IP2Bus_MstBE)         // O [0:3]
  );


///////////////////////////////////////////////////////////////////////////////
// Main Body of Code
///////////////////////////////////////////////////////////////////////////////


  always @(posedge OPB_Clk)
    if (Bus2IP_Reset)
      rst_counter <= 15'b0;
    else
      rst_counter <= rst_counter + 1;

  always @(posedge OPB_Clk)
    if (Bus2IP_Reset)
      RST_N <= 1'b0;
    else if (rst_counter[14:12] == 3'b101)
      RST_N <= 1'b1;

   assign GRST_N = RST_N;

   always @(posedge PCLK_IN)
     begin
       inta_d1 <= ~INTA;
       intb_d1 <= ~INTB;
       intc_d1 <= ~INTC;
       intd_d1 <= ~INTD;
     end

   always @(posedge PCLK_IN)
     INTR_OUT_d1 <= (~INTR_A) | inta_d1 | intb_d1 | intc_d1 | intd_d1;

   assign INTR_OUT = INTR_OUT_d1;
   
// Logic to synchronize IPIF clock down to slower PCI Clock.
// Note: PCI clock must be divided from IPIF clock by an integer and both
//       clocks must be edge aligned for this logic to work

// PCI Slave Side Sync Logic // 

// Pass RdReq/WrReq Signal From IPIF to PCI Clock Domain
always @(posedge Bus2IP_Clk)
  if (Bus2IP_Reset | BusSync2IP_RdReq | ~Bus2IP_SRAM_CE)
    Bus2IP_RdReq_hold <= 1'b0;
  else if (Bus2IP_RdReq)
    Bus2IP_RdReq_hold <= 1'b1;

always @(posedge Bus2IP_Clk)
  if (Bus2IP_Reset | BusSync2IP_WrReq | ~Bus2IP_SRAM_CE)
    Bus2IP_WrReq_hold <= 1'b0;
  else if (Bus2IP_WrReq)
    Bus2IP_WrReq_hold <= 1'b1;

always @(posedge PCLK_IN)
begin
  Bus2IP_RdReq_hold_d1 <= Bus2IP_RdReq_hold;
  Bus2IP_RdReq_hold_d2 <= Bus2IP_RdReq_hold_d1;
  Bus2IP_WrReq_hold_d1 <= Bus2IP_WrReq_hold;
  Bus2IP_WrReq_hold_d2 <= Bus2IP_WrReq_hold_d1;
end

assign BusSync2IP_RdReq = ~Bus2IP_RdReq_hold_d2 & Bus2IP_RdReq_hold_d1;
assign BusSync2IP_WrReq = ~Bus2IP_WrReq_hold_d2 & Bus2IP_WrReq_hold_d1;

// Generate SRAM_CE in PCI Clock Domain
always @(posedge PCLK_IN)
  if (Bus2IP_Reset | IP2BusSync_RdAck | IP2BusSync_WrAck | IP2BusSync_Retry)
    BusSync2IP_SRAM_CE_i <= 1'b0;
  else if (BusSync2IP_RdReq | BusSync2IP_WrReq)
    BusSync2IP_SRAM_CE_i <= 1'b1;

assign BusSync2IP_SRAM_CE = BusSync2IP_SRAM_CE_i
                          | BusSync2IP_RdReq | BusSync2IP_WrReq;

// Pass data through to PCI since it is held valid and glitchless by
// IPIF (running in OPB clock domain)
assign BusSync2IP_Addr = Bus2IP_Addr;
assign BusSync2IP_BE   = Bus2IP_BE;
assign BusSync2IP_Data = Bus2IP_Data;

// Latch Data returned from PCI
always @(posedge PCLK_IN)
  if (IP2BusSync_RdAck)
    IP2Bus_Data  <= IP2BusSync_Data;

always @(posedge PCLK_IN)
  if (IP2BusSync_RdAck | IP2BusSync_WrAck)
    IP2Bus_Error <= IP2BusSync_Error;

always @(posedge PCLK_IN)
  IP2Bus_ToutSup <= IP2BusSync_ToutSup;
   
always @(posedge PCLK_IN)
begin
  IP2BusSync_Retry_d1   <= IP2BusSync_Retry;
  IP2BusSync_RdAck_d1   <= IP2BusSync_RdAck;
  IP2BusSync_WrAck_d1   <= IP2BusSync_WrAck;
end

always @(posedge Bus2IP_Clk)
begin
  IP2BusSync_Retry_d2 <= IP2BusSync_Retry_d1;
  IP2BusSync_RdAck_d2 <= IP2BusSync_RdAck_d1;
  IP2BusSync_WrAck_d2 <= IP2BusSync_WrAck_d1;
  IP2BusSync_Retry_d3 <= IP2BusSync_Retry_d2;
  IP2BusSync_RdAck_d3 <= IP2BusSync_RdAck_d2;
  IP2BusSync_WrAck_d3 <= IP2BusSync_WrAck_d2;
end

assign IP2Bus_Retry = ~IP2BusSync_Retry_d3 & IP2BusSync_Retry_d2;
assign IP2Bus_RdAck = ~IP2BusSync_RdAck_d3 & IP2BusSync_RdAck_d2;
assign IP2Bus_WrAck = ~IP2BusSync_WrAck_d3 & IP2BusSync_WrAck_d2;

// PCI Master Side Sync Logic //
always @(posedge Bus2IP_Clk)
begin
  IP2Bus_MstRdReq_d1 <= IP2Bus_MstRdReq;
  IP2Bus_MstRdReq_d2 <= IP2Bus_MstRdReq_d1;
  IP2Bus_MstWrReq_d1 <= IP2Bus_MstWrReq;
  IP2Bus_MstWrReq_d2 <= IP2Bus_MstWrReq_d1;
end

assign IP2BusSync_MstRdReq = IP2Bus_MstRdReq_d1 & ~IP2Bus_MstRdReq_d2;
assign IP2BusSync_MstWrReq = IP2Bus_MstWrReq_d1 & ~IP2Bus_MstWrReq_d2;

always @(posedge Bus2IP_Clk)
  if (Bus2IP_MstRdAck | Bus2IP_MstWrAck | Bus2IP_Error)
    BusSync2IP_MstData <= Bus2IP_MstData;

always @(posedge Bus2IP_Clk)
begin
  if (Bus2IP_Reset | BusSync2IP_Error)
    Bus2IP_Error_hold <= 1'b0;
  else if (Bus2IP_Error)
    Bus2IP_Error_hold <= 1'b1;

  if (Bus2IP_Reset | BusSync2IP_MstRdAck)
    Bus2IP_MstRdAck_hold <= 1'b0;
  else if (Bus2IP_MstRdAck)
    Bus2IP_MstRdAck_hold <= 1'b1;

  if (Bus2IP_Reset | BusSync2IP_MstWrAck)
    Bus2IP_MstWrAck_hold <= 1'b0;
  else if (Bus2IP_MstWrAck)
    Bus2IP_MstWrAck_hold <= 1'b1;

  if (Bus2IP_Reset | BusSync2IP_Retry)
    Bus2IP_Retry_hold <= 1'b0;
  else if (Bus2IP_Retry)
    Bus2IP_Retry_hold <= 1'b1;

  if (Bus2IP_Reset | BusSync2IP_TimeOut)
    Bus2IP_TimeOut_hold <= 1'b0;
  else if (Bus2IP_TimeOut)
    Bus2IP_TimeOut_hold <= 1'b1;
end

always @(posedge PCLK_IN)
begin
  BusSync2IP_Error    <= Bus2IP_Error_hold;
  BusSync2IP_MstRdAck <= Bus2IP_MstRdAck_hold;
  BusSync2IP_MstWrAck <= Bus2IP_MstWrAck_hold;
  BusSync2IP_Retry    <= Bus2IP_Retry_hold;
  BusSync2IP_TimeOut  <= Bus2IP_TimeOut_hold;
end

endmodule
