//---------------------------------------------------------------------
// TSD Interface - Module
//---------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
// 
//--------------------------------------------------------------------
// Filename:    ipif_if_tsd.v
// 
// Description:    
//   This module implements the TSD Interface 
//
// Design Notes:
//   This module implements the TSD Interface Module. This module connects
//   to the IPIF Slave SRAM Interface and the TSD.
//
//---------------------------------------------------------------------
// Structure:   
// 
//            -- ipif_if_tsd.v
//
//---------------------------------------------------------------------
// Author:      PG
// History:
//   PG        10/04/01 -- First Revision
//
//---------------------------------------------------------------------
// Naming Conventions:
//  active low signals:                     "*_n"
//  clock signals:                          "clk", "clk_div#", "clk_#x" 
//  reset signals:                          "rst", "rst_n" 
//  generics/parameters:                    "C_*" 
//  user defined types:                     "*_TYPE" 
//  state machine next state:               "*_ns" 
//  state machine current state:            "*_cs" 
//  combinatorial signals:                  "*_com" 
//  pipelined or register delay signals:    "*_d#" 
//  counter signals:                        "*cnt*"
//  clock enable signals:                   "*_ce" 
//  internal version of output port         "*_i"
//  device pins:                            "*_pin" 
//  ports:                                  - Names begin with Uppercase 
//  processes:                              "*_PROCESS" 
//  component instantiations:               "<ENTITY_>I_<#|FUNC>
//---------------------------------------------------------------------


///////////////////////////////////////////////////////////////////////
// Module Declaration
///////////////////////////////////////////////////////////////////////

module ipif_if_tsd (
  // TSD Slave Signals
  DCLK,             // O
  DIN,              // O 
  CS,               // O
  DOUT,             // I 
  BUSY,             // I
  PENIRQ,           // I
  // IPIF Related Signals
  IP2Bus_Data,      // O [0:31]
  IP2Bus_Error,     // O
  IP2Bus_RdAck,     // O
  IP2Bus_Retry,     // O
  IP2Bus_ToutSup,   // O
  IP2Bus_WrAck,     // O
  IP2Bus_Intr,      // O
  Bus2IP_Addr,      // I [0:31]
  Bus2IP_BE,        // I [0:3]
  Bus2IP_Clk,       // I
  Bus2IP_Data,      // I [0:31]
  Bus2IP_RdReq,     // I
  Bus2IP_Reset,     // I
  Bus2IP_SRAM_CE,   // I
  Bus2IP_WrReq      // I
  );
   

///////////////////////////////////////////////////////////////////////
// Port Declarations
///////////////////////////////////////////////////////////////////////

  // TSD Slave Signals
  output         DIN;
  output         CS;
  output         DCLK;
  input          DOUT;
  input          BUSY;
  input          PENIRQ;
  // IPIF Related Signals
  output  [0:31] IP2Bus_Data;
  output         IP2Bus_Error;
  output         IP2Bus_RdAck;
  output         IP2Bus_Retry;
  output         IP2Bus_ToutSup;
  output         IP2Bus_WrAck;
  output         IP2Bus_Intr;      
  input [0:31]   Bus2IP_Addr;
  input [0:3]    Bus2IP_BE;
  input          Bus2IP_Clk;
  input [0:31]   Bus2IP_Data;
  input          Bus2IP_RdReq;
  input          Bus2IP_Reset;
  input          Bus2IP_SRAM_CE;
  input          Bus2IP_WrReq;

///////////////////////////////////////////////////////////////////////
// Signal Declarations
///////////////////////////////////////////////////////////////////////

   reg          DIN;
   reg          DIN_i;
   reg          DIN_i_d1;
   reg          DIN_i_d2;
   reg  [0:7]   DIN_Int;
   reg  [0:15]  DOUT_Int;
   reg          DOUT_d1;
   reg          DOUT_d2;
   reg          CS;
   reg          CS_i;
   reg          DCLK;
   reg          DCLK_Int;
   reg          DCLK_i;
   reg          PENIRQ_i;

   reg  [0:31]  Bus2IP_Data_Int;
   wire         IP2Bus_ToutSup;
   reg          IP2Bus_WrAck_state;
   reg  [3:0]   DCLK_Count8;
   reg  [3:0]   DCLK_Count16;
   reg  [6:0]   count;
   reg          two_mhz_en;
   reg          two_mhz_en_d1;
   reg          two_mhz_en_d2;
   reg          two_mhz_en_d3;
   reg          two_mhz_en_d4;
   reg          Count_to_8;
   reg          Count_to_16;
   reg          DATA_IN;
   wire         retry_rst;
   reg          Bus2IP_RdReq_hold;
   reg          Bus2IP_WrReq_hold;
   reg  [7:0]   state_cs;
   reg  [7:0]   state_ns;
   reg          PENIRQ_i_d1;
   reg          PENIRQ_i_d2;
   reg          penirq_rise_edge;
   reg          penirq_fall_edge;
   wire         rise_edge_irq;
   wire         rise_edge_irq_clr;
   wire         fall_edge_irq;
   wire         fall_edge_irq_clr;

/* synthesis translate_off */
   initial 
   begin
       DCLK_Count8 = 0;
       DCLK_Count16 = 0;
   end
/* synthesis translate_on */

///////////////////////////////////////////////////////////////////////
// Main Body of Code
///////////////////////////////////////////////////////////////////////

// Hold The bus on writes which take some time to get started
    assign IP2Bus_ToutSup = 1'b1;

// Generate a 2Mhz clock using a 2Mhz clock enable
    always@(posedge Bus2IP_Clk)
        if (Bus2IP_Reset) begin
            two_mhz_en <= 0;
            count      <= 0;
          end
        else if (count == 100) begin
           two_mhz_en <= 1;
           count      <= 0;
          end
        else begin
            count      <= count + 1;
            two_mhz_en <= 0;
          end

    always@(posedge Bus2IP_Clk)
    begin
      two_mhz_en_d1 <= two_mhz_en;
      two_mhz_en_d2 <= two_mhz_en_d1;
      two_mhz_en_d3 <= two_mhz_en_d2;
      two_mhz_en_d4 <= two_mhz_en_d3;
    end

    always@(posedge Bus2IP_Clk)
        if (Bus2IP_Reset)
            DCLK_Int <= 0;
        else if (two_mhz_en_d1) 
            DCLK_Int <= ~DCLK_Int;

// Latch Write Data
    always @(posedge Bus2IP_Clk)
      if (IP2Bus_WrAck | Bus2IP_Reset)
        Bus2IP_WrReq_hold <= 1'b0;
      else if (Bus2IP_WrReq & ~Bus2IP_Addr[29])
        Bus2IP_WrReq_hold <= 1'b1;

     always@(posedge Bus2IP_Clk)
        if (Bus2IP_WrReq_hold && state_cs[0])
              DIN_Int <= Bus2IP_Data[0:7];

// Generate the IP2Bus_WrAck. Logic generates a 1 clock pulse Ack.

    assign IP2Bus_WrAck = IP2Bus_WrAck_state | (Bus2IP_WrReq & Bus2IP_Addr[29]);

// Count 8 DCLKs when transmitting control bits
    always@(posedge Bus2IP_Clk)
       if (two_mhz_en_d1 & DCLK_Int) begin
         if (Count_to_8 == 0)
           DCLK_Count8 <= 0;
         else
           DCLK_Count8 <= DCLK_Count8 + 1;
       end

// Count 16 DCLKs when receiving data from the TSD
    always@(posedge Bus2IP_Clk)
      if (two_mhz_en_d1 & DCLK_Int) begin
        if (Count_to_16 == 0)
          DCLK_Count16 <= 0;
        else
          DCLK_Count16 <= DCLK_Count16 + 1;
      end

// State Machine to output DIN_Int and generate CS signal
    always @(posedge Bus2IP_Clk)
      if (Bus2IP_Reset) 
        state_cs <= 8'b0000_0001;
      else  
        state_cs <= state_ns;

    always@( DCLK_Int or DIN_Int or Bus2IP_WrReq_hold or DCLK_Count8 or
             DCLK_Count16 or BUSY or state_cs or two_mhz_en)
        case (state_cs)
            8'b0000_0001 : begin       // Reset State
                DCLK_i = 0;
                CS_i = 0;
                DIN_i = 0;
                Count_to_8 = 0;
                Count_to_16 = 0;
                DATA_IN = 0;
                IP2Bus_WrAck_state = 0; 
                if (Bus2IP_WrReq_hold & DCLK_Int)
                    state_ns =  8'b0000_0010 ;
                else
                    state_ns =  8'b0000_0001 ;
                end
            8'b0000_0010 : begin       // Controls start of DCLK generation
                DCLK_i = 0;
                CS_i = 0;
                DIN_i = 0;
                Count_to_8 = 0;
                Count_to_16 = 0;
                DATA_IN = 0;
                IP2Bus_WrAck_state = 0; 
                if (!DCLK_Int)
                    state_ns =  8'b0000_0100 ;
                else
                    state_ns =  8'b0000_0010 ;
                end
            8'b0000_0100 : begin       // Generates DCLK and CS
                DCLK_i = DCLK_Int;
                CS_i = 0;
                DIN_i = 0;
                Count_to_8 = 0;
                Count_to_16 = 0;
                DATA_IN = 0;
                IP2Bus_WrAck_state = 1; 
                state_ns =  8'b0000_1000 ;
                end
            8'b0000_1000 : begin       // Transfers data from OPB to TSD
                DCLK_i = DCLK_Int;
                CS_i = 0;
                if (DCLK_Count8[3])
                  DIN_i = 0;
                else
                  case (DCLK_Count8[2:0])
                    3'b000  : DIN_i = DIN_Int[0];
                    3'b001  : DIN_i = DIN_Int[1];
                    3'b010  : DIN_i = DIN_Int[2];
                    3'b011  : DIN_i = DIN_Int[3];
                    3'b100  : DIN_i = DIN_Int[4];
                    3'b101  : DIN_i = DIN_Int[5];
                    3'b110  : DIN_i = DIN_Int[6];
                    3'b111  : DIN_i = DIN_Int[7];
                    default : DIN_i = 0;
                  endcase
                Count_to_8 = 1;
                Count_to_16 = 0;
                DATA_IN = 1;
                IP2Bus_WrAck_state = 0; 
                if (DCLK_Count8[3] & DCLK_Int) // DCLK_Count8=8
                  state_ns =  8'b0001_0000 ;
                else
                  state_ns =  8'b0000_1000 ;
                end
            8'b0001_0000 : begin       // Waits for BUSY signal from TSD
                DCLK_i = 1;
                CS_i = 0;
                DIN_i = 0;
                Count_to_8 = 1;
                Count_to_16 = 0;
                DATA_IN = 1;
                IP2Bus_WrAck_state = 0; 
                if ((BUSY & !DCLK_Int) | (DCLK_Count8[3:1] == 3'b111))
                    state_ns =  8'b0010_0000 ;
                else
                    state_ns =  8'b0001_0000 ;
                end
            8'b0010_0000 : begin       // Waits for BUSY to go low, this signifies the start
                DCLK_i = 0;             // of data from the TSD
                CS_i = 0;
                DIN_i = 0;
                Count_to_8 = 1;
                Count_to_16 = 0;
                DATA_IN = 1;
                IP2Bus_WrAck_state = 0; 
                if ((!BUSY & DCLK_Int) | (DCLK_Count8[3:1] == 3'b111))
                    state_ns =  8'b1000_0000 ;
                else
                    state_ns =  8'b0010_0000 ;
                end
//            8'b0100_0000 : begin       // Generate DCLK
//                DCLK_i = DCLK_Int;
//                CS_i = 0;
//                DIN_i = 0;   
//                Count_to_8 = 0;
//                Count_to_16 = 0;
//                DATA_IN = 1;
//                IP2Bus_WrAck_state = 0; 
//                state_ns =  8'b1000_0000 ;
//                end
            8'b1000_0000 : begin       // Clock in 15 data bits
                DCLK_i = DCLK_Int;
                CS_i = 0;
                DIN_i = 0;
                Count_to_8 = 0;
                Count_to_16 = 1;
                DATA_IN = 1;
                IP2Bus_WrAck_state = 0; 
                if ((DCLK_Count16 == 15) & two_mhz_en & ~DCLK_Int)
                    state_ns =  8'b0000_0001 ;
                else
                    state_ns =  8'b1000_0000 ;
                end
            default : begin
                DCLK_i = 0;
                CS_i = 0;
                DIN_i = 0;
                Count_to_8 = 0;
                Count_to_16 = 0;
                DATA_IN = 0;
                IP2Bus_WrAck_state = 0; 
                state_ns =  8'b0000_0001 ;
            end
        endcase

        //
        // Used to register DCLK, DIN, CS, DOUT for better timing off chip
        // 
        always@(posedge Bus2IP_Clk)
          if (Bus2IP_Reset) begin
            DCLK     <= 1'b0;
            DIN_i_d1 <= 1'b0;
            DIN_i_d2 <= 1'b0;
            DIN      <= 1'b0;
            CS       <= 1'b0;
            DOUT_d1  <= 1'b0;
            DOUT_d2  <= 1'b0;
          end
          else begin
            DCLK     <= DCLK_i & ~CS_i;
            DIN_i_d1 <= DIN_i & ~CS_i;
            DIN_i_d2 <= DIN_i_d1;
            DIN      <= DIN_i_d2;
            CS       <= CS_i;
            DOUT_d1  <= DOUT;
            DOUT_d2  <= DOUT_d1;
          end
        
        //
        // Grabs the data coming from TSD and stores it until requested
        //
        always@(posedge Bus2IP_Clk)
          if (two_mhz_en_d3 & DCLK_Int & state_cs[7])
            DOUT_Int[DCLK_Count16] <= DOUT_d2;

        //
        // Output the most recently latched data to the OPB Bus on reads.
        //
        always @(posedge Bus2IP_Clk)
          if (IP2Bus_RdAck | Bus2IP_Reset)
            Bus2IP_RdReq_hold <= 1'b0;
          else if (Bus2IP_RdReq)
            Bus2IP_RdReq_hold <= 1'b1;

        always @(posedge Bus2IP_Clk)
          PENIRQ_i <= ~PENIRQ;

        assign IP2Bus_RdAck = Bus2IP_RdReq_hold & ~DATA_IN;
        assign IP2Bus_Data = Bus2IP_Addr[29]? {30'b0, rise_edge_irq, fall_edge_irq} :
                                              {4'b0, DOUT_Int[0:11], 15'b0, PENIRQ_i};

        //
        // IP2Bus_Retry - Can not request while data is being processed
        //
        assign IP2Bus_Retry = 1'b0;

        //
        // IP2Bus_Error - Don't Generate Errors
        //
        assign IP2Bus_Error = 1'b0;

        //
        // Generate a OPB Bus Interrupt when a PENIRQ changes state
        // PenIRQ is resync'ed to the 2 MHz clock for debouncing
        //
        assign IP2Bus_Intr = rise_edge_irq | fall_edge_irq;
        
        always @(posedge Bus2IP_Clk)
        begin
          if (Bus2IP_Reset) begin
            PENIRQ_i_d1 <= 1'b0;
            PENIRQ_i_d2 <= 1'b0;
          end
          if (two_mhz_en_d2 & state_cs[0]) begin // Freeze value of penirq during TSD accesses so noise
                                                 // from ADC does not affect the PENIRQ input
            PENIRQ_i_d1 <= PENIRQ_i;
            PENIRQ_i_d2 <= PENIRQ_i_d1;
          end
          penirq_rise_edge <=  PENIRQ_i_d1 & ~PENIRQ_i_d2 & two_mhz_en_d3 & state_cs[0];
          penirq_fall_edge <= ~PENIRQ_i_d1 &  PENIRQ_i_d2 & two_mhz_en_d3 & state_cs[0];
        end

        FDRSE rise_edge_irq_ff (.Q(rise_edge_irq),    .C(Bus2IP_Clk), .D(1'b0), .CE(1'b0),
                                .S(penirq_rise_edge), .R(rise_edge_irq_clr));
        FDRSE fall_edge_irq_ff (.Q(fall_edge_irq),    .C(Bus2IP_Clk), .D(1'b0), .CE(1'b0),
                                .S(penirq_fall_edge), .R(fall_edge_irq_clr));

        assign rise_edge_irq_clr = (Bus2IP_WrReq & Bus2IP_Addr[29] & Bus2IP_Data[30]) | Bus2IP_Reset;
        assign fall_edge_irq_clr = (Bus2IP_WrReq & Bus2IP_Addr[29] & Bus2IP_Data[31]) | Bus2IP_Reset;

endmodule

