/******************************************************************************
*     $Date: 2004/01/15 14:27:10 $
*     $RCSfile: ps2_scancodes_int.c,v $
*     This program initializes a PS/2 device and sets up an interrupt handler
*     to echo input characters, e.g. a set of 3 bytes each time a keystroke 
*     occurs on a PS/2 keyboard.
*******************************************************************************

*******************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2004 Xilinx, Inc.
*     All rights reserved.
*
******************************************************************************/
/***************************** Include Files ********************************/

#include <stdio.h>
#include "xbasic_types.h"
#include "xexception_l.h"
#include "xintc.h"
#include "xparameters.h"
#include "xps2.h"
#include "xuartns550_l.h"

/************************** Constant Definitions ****************************/

#define BUFFER_SIZE   10
#define ESCAPE        0x1b

#define PS2_DEVICE_ID      XPAR_OPB_PS2_DUAL_REF_0_DEVICE_ID_0
#define PS2_INTERRUPT_ID   XPAR_DCR_INTC_0_OPB_PS2_DUAL_REF_0_SYS_INTR1_INTR

#define UART_BASEADDR      XPAR_OPB_UART16550_0_BASEADDR
#define UART_CLOCK         XPAR_XUARTNS550_CLOCK_HZ
#define UART_BAUDRATE      9600

/**************************** Type Definitions ******************************/

/***************** Macros (Inline Functions) Definitions ********************/

#define STATUS_CHECK(Status)         \
    {                                \
        if (Status != XST_SUCCESS)   \
        {                            \
            return Status;           \
        }                            \
    }                                

/************************** Variable Definitions ****************************/

static XPs2 Ps2;

Xuint8 SendBuffer[BUFFER_SIZE];
Xuint8 RecvBuffer[BUFFER_SIZE];
Xuint8 SendBuffer1[BUFFER_SIZE];
Xuint8 RecvBuffer1[BUFFER_SIZE];

/************************** Function Prototypes *****************************/

void Handler(void *CallBackRef, Xuint32 Event, unsigned int EventData);

/****************************************************************************/

static XStatus SetupInterruptSystem(XIntc *Intc, XPs2 *Ps2Ptr)
{
    XStatus Status;

    /*
     * Initialize the exception handler
     */
    XExc_Init();

    /*
     * Initialize the interrupt controller driver so that
     * it's ready to use, specify the device ID that is generated in
     * xparameters.h
     */
    Status = XIntc_Initialize(Intc, XPAR_DCR_INTC_0_DEVICE_ID);
    STATUS_CHECK(Status);

    /*
     * Connect the Intc handler to the exception handler
     */
    XExc_RegisterHandler(XEXC_ID_NON_CRITICAL_INT, 
                         (XExceptionHandler)XIntc_InterruptHandler,
                         (void *)Intc);

    /*
     * Connect a device driver handler that will be called when an interrupt 
     * for the device occurs, the device driver handler performs the specifc
     * interrupt processing for the device
     */
    Status = XIntc_Connect(Intc, PS2_INTERRUPT_ID,
                           (XInterruptHandler)XPs2_InterruptHandler,
                           (void *)Ps2Ptr);
    STATUS_CHECK(Status);

    /*
     * Start the interrupt controller such that interrupts are enabled for
     * all devices that cause interrupts, specify real mode so that
     * PS/2 can cause interrupt thru the interrupts controller.
     */
    Status = XIntc_Start(Intc, XIN_REAL_MODE);
    STATUS_CHECK(Status);

    /*
     * Enable the interrupt for PS/2 port 0 
     */
    XIntc_Enable(Intc, PS2_INTERRUPT_ID);

    /*
     * Enable non-critical exceptions
     */
    XExc_mEnableExceptions(XEXC_ALL);

    return Status;
}

void Handler(void *CallBackRef, Xuint32 Event, unsigned int EventData)
{
    int i;

    if (Event == XPS2_EVENT_SENT_DATA)
    {
    }

    if (Event == XPS2_EVENT_RECV_DATA)
    {
        printf("XPS2_EVENT_RECV_DATA: %i\n", EventData);
        for (i=0; i<EventData; i++)
        {
            printf("%X  ", RecvBuffer[i]);
        }

        printf("\n");

        XPs2_Recv(&Ps2, RecvBuffer, 3);
    }
}

int main()
{
    static XIntc Intc;
    XStatus Status;
    Xuint8 LastErrors;
    int i = 0;

    /*
     * Initialize the UART driver, so we can use printf.
     */
    XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
    XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);
    printf("\n\nPS/2 Demo using interrupt mode\n\r");

    /*
     * Initialize the PS/2 driver so that it's ready to use,
     * specify the device ID that is generated in xparameters.h
     */
    Status = XPs2_Initialize(&Ps2, PS2_DEVICE_ID);
    STATUS_CHECK(Status);

    Status = SetupInterruptSystem(&Intc, &Ps2);
    STATUS_CHECK(Status);

    /*
     * Setup the handlers for PS/2 that will be called from the interrupt
     * context when data has been sent and received, specify a pointer to the 
     * PS/2 driver instance as the callback reference so that handlers are able
     * to access the instance data
     */
    XPs2_SetHandler(&Ps2, Handler, &Ps2);

    /*
     * Enable the PS/2 receive interrupt 
     */
    XPs2_mEnableIntr(Ps2.BaseAddress, XPS2_INT_ALL|XPS2_INT_WDT_TOUT);

    /*
     * Initialize the buffers
     */
    for (i = 0; i < BUFFER_SIZE; i++)
    {
        SendBuffer[i] = 0;
        RecvBuffer[i] = 0;
    }

    /*
     * Prepare to send AT Keyboard Init Code
     */
    SendBuffer[0] = 0xF4;

    /*
     * Prepare for ACK byte from Keyboard
     */
    XPs2_Recv(&Ps2, RecvBuffer, 1);

    /*
     * Send Init Code
     */
    XPs2_Send(&Ps2, SendBuffer, 1);

    /*
     * Check for errors
     */
    LastErrors = XPs2_GetLastErrors(&Ps2);
    if (LastErrors != XPS2_ERROR_NONE)
    {
        printf("ERROR WHILE SENDING DATA: %X\n", LastErrors);
    }
    else
    {
        printf("Keyboard initialized!\n");
    }

    /* 
     * Start receiving data ignoring the number of bytes received as 
     * the return value since we are using interrupt mode. The Scancode
     * from the keyboard is always 3 bytes. 
     */
    XPs2_Recv(&Ps2, RecvBuffer, 3);

    /*
     * stay in an endless loop, keep the interrupts coming in...
     */
    while(1)
    {
    }

    return Status;
}

