/******************************************************************************
*     $Date: 2004/01/01 01:29:56 $
*     $RCSfile: touchscreen_int.c,v $
*******************************************************************************

*******************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2004 Xilinx, Inc.
*     All rights reserved.
*
******************************************************************************/
/***************************** Include Files ********************************/

#include <stdio.h>
#include "sleep.h"
#include "xbasic_types.h"
#include "xexception_l.h"
#include "xintc.h"
#include "xparameters.h"
#include "xtouchscreen.h"
#include "xuartns550_l.h"

/************************** Constant Definitions ****************************/
#define SHOW_PEN_IRQS              XFALSE

#define UP                         XFALSE
#define DOWN                       XTRUE

#define UART_BASEADDR              XPAR_OPB_UART16550_0_BASEADDR
#define UART_CLOCK                 XPAR_XUARTNS550_CLOCK_HZ
#if !SIM
#define UART_BAUDRATE              9600                      /* real hardware */
#else
#define UART_BAUDRATE              (UART_CLOCK / 16 / 3)     /* simulation */
#endif
#define TOUCHSCREEN_DEVICE_ID      XPAR_OPB_TSD_REF_0_DEVICE_ID
#define TOUCHSCREEN_INTERRUPT_ID   XPAR_DCR_INTC_0_OPB_TSD_REF_0_INTR_INTR

#define INTC_DEVICE_ID             XPAR_DCR_INTC_0_DEVICE_ID

/**************************** Type Definitions ******************************/

#define STATUS_CHECK(Status)         \
    {                                \
        if (Status != XST_SUCCESS)   \
        {                            \
            return Status;           \
        }                            \
    }                                

/***************** Macros (Inline Functions) Definitions ********************/

/************************** Variable Definitions ****************************/

static XTouchscreen Touchscreen;
static Xboolean PenDown;

/************************** Function Prototypes *****************************/

void Handler(void *CallBackRef, Xuint32 Event, unsigned int EventData);
static XStatus SetupInterruptSystem(XIntc *Intc, XTouchscreen *TouchscreenPtr);

/****************************************************************************/

/* Display touchscreen variables: x, y coordinates and pressure */
void PrintCoordinates(void)
{
    Xuint32 x;
    Xuint32 y;
    Xuint32 z;

    XTouchscreen_GetPosition_3D(&Touchscreen, &x, &y, &z);
    printf("X=%5lu   Y=%5lu   Z=%5lu\n", x, y, z);
}

/****************************************************************************/

long IntCount = 0;

int main()
{
    static XIntc Intc;
    XStatus Status;

    PenDown = XFALSE;
    /*
     * Initialize the UART driver, so we can use printf.
     */
    XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
    XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);
    printf("\n\nTouchscreen Demo using interrupt mode\n\r");

    /*
     * Initialize the Touchscreen driver so that it's ready to use,
     * specify the device ID that is generated in xparameters.h
     */
    Status = XTouchscreen_Initialize(&Touchscreen, TOUCHSCREEN_DEVICE_ID);
    STATUS_CHECK(Status);

    /*
     * Setup the handlers that will be called from the interrupt
     * context when data has been sent and received, specify a pointer to the 
     * driver instance as the callback reference so that handlers are able
     * to access the instance data
     */
    XTouchscreen_SetHandler(&Touchscreen, Handler, &Touchscreen);

    Status = SetupInterruptSystem(&Intc, &Touchscreen);
    STATUS_CHECK(Status);

    /* stay in an endless loop, keep the interrupts coming in... */
    while(XTRUE) {
      /* Polling: samples and display coordinates while touchscreen is 
       * being touched e.g. when pointer is dragged across the 
       * touchscreen. */
      while (PenDown) {
	PrintCoordinates(); // prints variables immediately as soon as 
	                    // interrupt handler triggers
	usleep(100000);
      }
    }
    return Status;
}

/* Interrupt handler: Sets PenDown variable based on the status of 
 * touchscreen access. 
 * Touchscreen variables are not displayed in the interrupt handler
 * but are polled in main()                                         */
void Handler(void *CallBackRef, Xuint32 Event, unsigned int EventData)
{
    if (Event == XTOUCHSCREEN_EVENT_PEN_DOWN) {
      PenDown = XTRUE;
#if SHOW_PEN_IRQS
      printf("PEN DOWN\n");
#endif
    }
    
    if (Event == XTOUCHSCREEN_EVENT_PEN_UP) {
      PenDown = XFALSE;
#if SHOW_PEN_IRQS
      printf("PEN UP\n");
#endif
    }
}


static XStatus SetupInterruptSystem(XIntc *Intc, XTouchscreen *TouchscreenPtr)
{
    XStatus Status;

    /*
     * Initialize the exception handler
     */
    XExc_Init();

    /*
     * Initialize the interrupt controller driver so that
     * it's ready to use, specify the device ID that is generated in
     * xparameters.h
     */
    Status = XIntc_Initialize(Intc, INTC_DEVICE_ID);
    STATUS_CHECK(Status);

    /*
     * Connect the Intc handler to the exception handler
     */
    XExc_RegisterHandler(XEXC_ID_NON_CRITICAL_INT, 
                         (XExceptionHandler)XIntc_InterruptHandler,
                         (void *)Intc);

    /*
     * Connect a device driver handler that will be called when an interrupt 
     * for the device occurs, the device driver handler performs the specifc
     * interrupt processing for the device
     */
    Status = XIntc_Connect(Intc, TOUCHSCREEN_INTERRUPT_ID,
                           (XInterruptHandler)XTouchscreen_InterruptHandler,
                           (void *)TouchscreenPtr);
    STATUS_CHECK(Status);

    /*
     * Start the interrupt controller such that interrupts are enabled for
     * all devices that cause interrupts, specific real mode so that
     * TSD can cause interrupt thru the interrupts controller.
     */
    Status = XIntc_Start(Intc, XIN_REAL_MODE);
    STATUS_CHECK(Status);

    /*
     * Enable the interrupts for Touchscreen 
     */
    XIntc_Enable(Intc, TOUCHSCREEN_INTERRUPT_ID);

    /*
     * Enable non-critical exceptions
     */
    XExc_mEnableExceptions(XEXC_ALL);

    return Status;
}


