/******************************************************************************
*
*     Author: Xilinx, Inc.
*
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
*     COURTESY TO YOU. BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
*     ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION OR STANDARD,
*     XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
*     FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE FOR OBTAINING
*     ANY THIRD PARTY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
*     XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
*     THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY
*     WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM
*     CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND
*     FITNESS FOR A PARTICULAR PURPOSE.
*
*
*     Xilinx hardware products are not intended for use in life support
*     appliances, devices, or systems. Use in such applications is
*     expressly prohibited.
*
*
*     (c) Copyright 2002-2004 Xilinx Inc.
*     All rights reserved.
*
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     675 Mass Ave, Cambridge, MA 02139, USA.
*
******************************************************************************/

#include <common.h>
#include <environment.h>
#include <configs/ml300.h>
#include "xparameters.h"

#ifdef CFG_ENV_IS_IN_EEPROM

const u8 hex[] = "0123456789ABCDEF"; /* lookup table for ML300 CRC */

extern int send(uchar, u32, int, u8 *, u32);
extern int receive(uchar, u32, int, u8 *, u32);
extern uchar *env_get_addr(int);
extern int eeprom_write (unsigned, unsigned, uchar*, unsigned);

/************************************************************************
 * Key-value pairs are separated by "=" sign.
 */
static int
findKey(uchar * buffer, uchar key, u32 len)
{
	u32 i;
	enum { BEGIN_KEY, IN_KEY, IN_VALUE } state;

	state = BEGIN_KEY;

	for (i=0; i<len; i++) {

		switch (state) {
		case BEGIN_KEY:
			state = IN_KEY;
			if (buffer[i] == key && buffer[i+1] == '=') {
				return (i+1);
			}

			/* otherwise, fall through to validate the char. */
		case IN_KEY:
			switch (buffer[i]) {
			case 'A'...'Z':
			case 'a'...'z':
			case '0'...'9':
			case '_':
				break; /* Valid char.  Do nothing. */
			case '=':
				state = IN_VALUE;
				break;
			default:
				return -1; /* Invalid character.  Error. */
			}
			break;
		case IN_VALUE:
			if (buffer[i] == '\0')
				state = BEGIN_KEY;
			break;
		}
	}

	return -1;
}

/************************************************************************
 * Compute a new ML300 CRC when user calls the saveenv command.
 * Also update EEPROM with new CRC value.
 */
static u8
update_crc(u32 len, uchar * data)
{
	uchar temp[6] = { 'C', '=', 0x00, 0x00, 0x00, 0x00 };
	u32 crc;		/* new crc value */
	u32 i;

	crc = 0;

	/* calculate new CRC */
	for (i = 0; i < len; i++)
		crc += data[i];

	/* CRC includes key for check sum */
	crc += 'C' + '=';

	/* compose new CRC to be updated */
	temp[2] = hex[(crc >> 4) & 0xf];
	temp[3] = hex[crc & 0xf];

	/* check to see if env size exceeded */
	if (len + 6 > CFG_BOARD_PARAM_SIZE) {
		printf("ERROR: not enough space to store CRC on EEPROM");
		return 1;
	}

	memcpy(data + len, temp, 6);
	return 0;
}

/************************************************************************
 * Read out ML300 CRC and compare it with a runtime calculated ML300 CRC.  
 * If equal, then pass 1, otherwise pass back 0 to indicate CRC error.
*/
u8
read_crc(void)
{
	u8 buffer[CFG_BOARD_PARAM_SIZE];
	u32 crc, i;		/* runtime crc */
	int loc;

	crc = 0;

	/* calculate runtime CRC according to ML300 and read back
	   old CRC stored in the EEPROM */
	receive(CFG_I2C_EEPROM_ADDR, XPAR_PERSISTENT_0_IIC_0_BASEADDR, 
		CFG_I2C_EEPROM_ADDR_LEN, buffer, CFG_BOARD_PARAM_SIZE);

	loc = findKey(buffer, 'C', CFG_BOARD_PARAM_SIZE);

	if (loc == -1)
		return 1;

	for (i=0; i < CFG_BOARD_PARAM_SIZE; i++) {
		if (buffer[i] == '\0' && buffer[i+1] == '\0')
			break;
		crc += buffer[i];
	}

	/* exclude old CRC from runtime calculation */
	crc -= (buffer[loc+1] + buffer[loc+2]);

	/* match CRC values */
	if ((buffer[loc+1] == hex[(crc >> 4) & 0xf])
		&& (buffer[loc+2] == hex[crc & 0xf]))
		return 0;

	return 1;
}

/************************************************************************
 * Compose board specific parameters for write back
 */
static void
copy_board(uchar *key, uchar *data, uchar *eprom, u32 *len)
{
	memcpy(eprom+*len, key, strlen(key));
	*len += strlen(key);
	memcpy(eprom+*len, data, strlen(data)+1);
	*len += strlen(data) + 1;
}

/************************************************************************
 * Update ethaddr to u-boot standard.
 */
static void
convert_env(void)
{
	uchar *s;		/* pointer to env value */
	uchar temp[20];

	/********************* E -> ethaddr *********************/
	s = getenv("E");
	if (s != NULL) {
		sprintf(temp, "%c%c.%c%c.%c%c.%c%c.%c%c.%c%c",
			*s, *(s+1), *(s+2), *(s+3), *(s+4), *(s+5),
			*(s+6), *(s+7), *(s+8), *(s+9), *(s+10), *(s+11));
		if (s[0] != 0x00)
			setenv("ethaddr", temp);
		setenv("E", NULL);
	}

	/* Don't include C (CRC) */
	setenv("C", NULL);
}

/************************************************************************
 * Save user modified board parameters back to EEPROM.
 */
u8
save_env(void)
{
	uchar eprom[CFG_BOARD_PARAM_SIZE];	/* write back to EEPROM */
	uchar buffer[CFG_BOARD_PARAM_SIZE];	/* read out old value*/
	uchar *s, temp[20];
	int loc;
	u32 len;
	uchar choice;

	printf("\nOnly MAC address and CRC value will be written, \n");
	printf("do you want to let U-Boot update your board specific \n");
	printf("area with only these two parameters? <y/n> ");
	choice = getc();
	printf("%c\n\n", choice);

	if (choice != 'y')
		return 0;

	receive(XPAR_PERSISTENT_0_IIC_0_EEPROMADDR >> 1, 
		XPAR_PERSISTENT_0_IIC_0_BASEADDR, 
		CFG_I2C_EEPROM_ADDR_LEN, 
		buffer, 
		CFG_BOARD_PARAM_SIZE);

	len = 0;

	/********************* ethaddr -> E *********************/
	s = getenv("ethaddr");
	loc = findKey(buffer, 'E', CFG_BOARD_PARAM_SIZE);
	if (s != NULL)
		sprintf(temp, "%c%c%c%c%c%c%c%c%c%c%c%c",
			*s, *(s + 1), *(s + 3), *(s + 4), *(s + 6), *(s + 7),
			*(s + 9), *(s + 10), *(s + 12), *(s + 13), *(s + 15),
			*(s + 16));
	else if (loc > 0)
		sprintf(temp, "%s", &buffer[loc+1]);

	copy_board("E=", temp, eprom, &len);

	/*******************************************************/

	/* update EEPROM env values if there is enough space */
	if (update_crc(len, eprom) != 0)
		return 1;

	eeprom_write((XPAR_PERSISTENT_0_IIC_0_EEPROMADDR >> 1),
			XPAR_PERSISTENT_0_IIC_0_BASEADDR,
			eprom,
			len+6);

	return 0;
}

/************************************************************************
 * Add board specific parameters to u-boot
 */
u8
add_board_env(void)
{
	u32 i;
	uchar *env, *nxt = NULL;
	uchar *env_data = env_get_addr(0);
	uchar data[CFG_BOARD_PARAM_SIZE];

	if (!env_data)	/* need copy in RAM */
		return 1;

	for (env=env_data; *env; env=nxt+1) {
		for (nxt=env; *nxt; ++nxt)
			;
	}

	receive(XPAR_PERSISTENT_0_IIC_0_EEPROMADDR >> 1, 
		XPAR_PERSISTENT_0_IIC_0_BASEADDR, 
		CFG_I2C_EEPROM_ADDR_LEN, 
		data, 
		CFG_BOARD_PARAM_SIZE);

	for (i=1; i<CFG_BOARD_PARAM_SIZE; i++)
		if ((data[i-1] == '\0') && (data[i] == '\0'))
			break;

	if ((i+1) > (&env_data[ENV_SIZE]-env)) {
		printf ("## Error: environment overflow, board parameters not copied\n");
		return 1;
	}
	
	memcpy(env, data, i+1);

	convert_env();

	return 0;
}

#endif	/* CFG_ENV_IS_IN_EEPROM */
